/**
 * The FinessePotion class represents a potion that can be consumed by the kobold,
 * which increases its finesse stat.
 */
function StrengthPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Strength Potion";
    var itemDescription = "A small glass vial shaped like a heart. Red liquid sloshes within. The label on the side says that it'll make you more powerful.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The red fluid within smells like blood.
                            [b]Fresh[/b] blood.
                            It sickens you.
                        [/p]
                        [p]
                            Disgust visible on your face, you steel your resolve.
                            You lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            It tastes exactly the way it looks.
                            Blood.
                            You've had a lick of your own in the past, but you've never quite tasted anyone else's.
                            You gag as you swallow the first load.
                            You almost throw up, but barely manage to keep your stomach's contents.
                        [/p]
                        [p]
                            Through sheer will and determination alone, you down a second, and thankfully final gulp.
                            In a flash of anger, you toss the empty bottle to the ground, upon which it shatters into a thousand tiny little pieces.
                            You are left retching, the taste of blood still lingering in your mouth.
                            You try to wipe your tongue with your palm, but it does little to help.
                        [/p]
                        [p]
                            You feel completely [b]sickened[/b] with yourself.
                            You knew darn well what that stuff was before you drank it, yet you chugged it down anyway.
                            What kind of [b]monster[/b] are you becoming?!
                            You've got to get a hold of yourself before it's too late.
                        [/p]
                        [p]
                            [b]You have gained 1 strength.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as strong as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.STRENGTH_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 185px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var str = global.kobold.stats.get(StatsEnum.STRENGTH);
        var maxed = (str >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}
