/**
 * The MassRedu class represents a potion that can be consumed by the kobold,
 * which increases its agility, but lowers its endurance.
 */
function MassRedu() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "MassRedu Potion";
    var itemDescription = `A glass bottle filled to the brim with a black-looking gas-like substance.
                           Promises to make you more agile.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            It smells like [b]nothing[/b].
                            While the black stuff looks like gas, sniffing it alone doesn't seem to be able to rouse it from its container.
                            Oh well, probably some alchemical trickery going on.
                            You press the bottle to your lips, and gulp down.
                        [/p]
                        [p]
                            It has no taste.
                            You barely even [b]feel[/b] it vanishing in your mouth, and disappearing down your throat.
                            If you didn't see the bottle emptying before your very eyes, you wouldn't even believe you were [i]drinking[/i] something.
                        [/p]
                        [p]
                            As the final bit of the gas travels down into your stomach,
                            you start to feel [b]weird[/b].
                            Your limbs [b]tingle[/b], but on the [b]inside[/b].
                            They feel lighter, somehow.
                            You can move them around with greater ease!
                            You don't even feel any weaker or anything.
                            The potion worked without a hitch!
                        [/p]
                        [p]
                            Or so you believe, until you realize what is actually happening.
                            To reduce your mass, the concotion is not draining your muscles,
                            but the insides of your very bones!
                            Your usually solid skeleton is sapped of strength, growing considerably weaker, and a lot more brittle.
                            A mixed blessing.
                            At least you can run faster now.
                        [/p]
                        [p]
                            [b]You have gained 1 agility.[/b][nl]
                            [warn][b]You have lost 1 endurance.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as agile as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            Your bones are so brittle, that lifting the potion to your lips would break your arm.
                            You do not drink it.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.MASSREDU;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var agi = global.kobold.stats.get(StatsEnum.AGILITY);
        var end = global.kobold.stats.get(StatsEnum.ENDURANCE);
        var maxed = (agi >= global.kobold.stats.statMax);
        var minned = (end <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
            return false;
        } else {
            global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
            global.kobold.stats.decreaseStat(StatsEnum.ENDURANCE, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Potion-stripped} potion A Potion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}
