/**
 * End section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementSlipSucceedContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;

    var continueId = arguments.callee.name + "ButtonOne"

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.enslavement.nightOne.knockedOut = true;
        var toReturn = `[p]
                            After caressing your wrists for a few moments,
                            slightly easing the pain of the minor rope burn you sustained while struggling to break your bonds,
                            you rise to your feet within the cell-like wagon.
                            Bad idea.
                        [/p]
                        [p]
                            The equine guard stationed outside of your cart just so happened to be leering inside of your confined area the moment you decided to stand upright.
                            Noticing your unbound hands, he draws a hefty-looking ${global.kobold.inventory.createItemWithTooltip(new Club())} from his belt.
                            For a moment, he stares at you, and you stare at him.
                            It takes a few seconds for you to register the fact that you've been caught.
                            Then, panic begins to settle in.
                        [/p]
                        [p]
                            The horse-sentry calmly walks over to the cart's single door.
                            He unlocks and opens it.
                            This is your chance.
                            Your one shot.
                            If you can get past him, you can escape into the wilderness, and regain your freedom!
                        [/p]
                        [p]
                            Alas, instead of attacking the horse, bolting past him, reasoning with him or even [b]seducing[/b] him,
                            your indecisive mind causes your body to freeze up in fright.
                            Instead of making a heroic getaway, you cower in fear, like a whelp that knows it [b]fucked up[/b].
                            Still, the equine shows no mercy.
                            He throws an overhead swing with his ${global.kobold.inventory.createItemWithTooltip(new Club())},
                            the wood crashing against the top of your head.
                        [/p]
                        [p]
                            You hear something [b]crack[/b].
                            You doubt it's the wood.
                            Your vision goes black.
                            You slump down onto the floor, knocked unconscious in a single blow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.slipSucceedContinue.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
