/**
 * Tight section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementTightness(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.postMouth;
    var subController = ctrl.tight;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new CaravanConfinementClaw(ctrl);
    var secondNext = new CaravanConfinementStandStill(ctrl);
    var thirdNext = new CaravanConfinementMoan(ctrl);

    var firstChoice = CaravanConfinementTightnessChoice.CLAW;
    var secondChoice = CaravanConfinementTightnessChoice.STILL;
    var thirdChoice = CaravanConfinementTightnessChoice.MOAN;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var fucked = global.kobold.history.shared.enslavement.captureFucked;
        var toReturn = `[p]
                            "[color=brown]Anyway, there's one more thing we need to take a look at.[/color]"
                            The caravan master gives the canine a small nod.
                            The wolf, who seemingly understands the human's intentions, steps behind you.
                            Grabbing your tail, the strong lupine [b]yanks[/b] it out from in between your legs, exposing your loins.
                            The expert in slavery, meanwhile, drops to his knees, coming face-to-face with you.
                            [i]He smells.[/i]
                        [/p]`;
        
        if(fucked) {
            toReturn += `[p]
                            The man forcibly parts your thighs with one of his burly hands.
                            "[color=brown]Damnit, Fenrir!
                            Do you [b]really[/b] have to [b]fuck[/b] everything you get your filthy hands on?![/color]"
                            You hear the canine behind you chuckle.
                            "[color=gblue]What can I say?
                            I'm a wolf.
                            That's what we do.
                            We [b]fuck[/b] things.[/color]"
                            The human seems unimpressed by the bandit's lackluster explanation.
                            "[color=gblue]Besides, that nun in the village we passed was practically [b]begging[/b] for it.[/color]"
                        [/p]
                        [p]
                            "[color=brown]Should have brought [b]her[/b] back to the caravan instead of this scaly [i]thief[/i].
                            Let's hope the [i]other[/i] hole is still tight enough to attract a couple of potential buyers.
                            A good mouth alone won't draw in the [i]perverts[/i], and those are the people who [b]really[/b] [i]jack up[/i] the price.[/color]"
                        [/p]`;
        } else {
            toReturn += `[p]
                            The man forcibly parts your thighs with one of his burly hands.
                            "[color=gblue]It said it was a virgin, back when I caught it.[/color]" the wolf shares.
                            The human inquisitively raises an eyebrow.
                            "[color=brown]A virgin kobold?
                            And you [b]bought[/b] that lie?
                            Come on, Fenrir, I thought you were smarter than that.[/color]"
                            The canine squeezes your tail, wordlessly threatening to harm you should you have [i]lied[/i] to him.
                            "[color=brown]Let's find out if we have an [b]honest thief[/b] or a [b]rotten apple[/b] on our hands.[/color]"
                        [/p]`;
        }
        
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Scratch him</button>`;

        var tooltip = global.parser.parse(`Claw at the human, hopefully dissuading him from probing your nether region.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Stand still</button>`;

        var tooltip = global.parser.parse(`Let the man do his thing. It'll be over a lot faster if you don't resist.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Moan</button>`;

        var tooltip = global.parser.parse(`Show him you know what a slave is to do when someone touches you.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

var CaravanConfinementTightnessChoice = {
    CLAW: 1,
    STILL: 2,
    MOAN: 3
};
