/**
 * Farmer's maid ending.
 * @param {FarmerMaidController} controller The Varanar, Afternoon Three controller.
 */
function FarmerMaidDrunkSatisfyDad(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preDeath;
    var subController = ctrl.death;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new FarmerMaidDrunkSatisfyFree(ctrl);
    var secondNext = new FarmerMaidDrunkSatisfyStay(ctrl);

    var firstChoice = FarmerMaidDrunkSatisfyDadChoice.LEAVE;
    var secondChoice = FarmerMaidDrunkSatisfyDadChoice.STAY;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var name = ctrl.misc.name();
        var toReturn = `[p]
                            Romeo had an [i]unfortunate accident[/i] while tending to his field.
                            Somehow, he [i]stabbed himself[/i] with his rake, while watering the crops.
                            The circumstances are [b]suspicious[/b], to say the least,
                            but you are reminded these things tend to happen around here.
                            Farming near the hamlet, ironically, is about as dangerous as being a city guard in Varanar.
                            Whether this was a robbery gone wrong, a rival offing your master for personal gains,
                            or an [i]actual[/i] accident, you will never know.
                        [/p]
                        [p]
                            Falco takes the news rather well.
                            He never really saw eye-to-eye with the man of the house.
                            Juliet, however, is devastated.
                            Neither you nor her son able to console her,
                            the woman moves out of the house, and back in with her parents,
                            somewhere in Varanar.
                            That leaves you and him, all alone.
                        [/p]
                        [p]
                            Romeo is buried in the backyard by his closest farmer friends.
                            After the emotional burial, Falco confronts you.
                            He's clearly touched, but he's not letting it show.
                            Sincerely, the bird speaks:
                            "[color=yellow]Look, ${name}, I...[b]really[/b] like you.
                            But there's something you should know.[/color]"
                            You aren't sure what to expect.
                            His wording makes it sound [i]ominous[/i], but his tone seems [i]uplifting[/i].
                        [/p]
                        [p]
                            "[color=yellow]Your [i]slave contract[/i] only bound you to my father.
                            Technically, you are [b]free[/b] now.
                            I understand that this past year has been a [i]hectic[/i] one, for sure,
                            and if you want to leave, I won't stand in your way.[/color]"
                            You're [b]free[/b].
                            Really?!
                            "[color=yellow]But since you've...well, because I...since you're [b]pregnant[/b] and all,
                            I wouldn't mind you [i]staying[/i] here.
                            With me.[/color]"
                            Wait.
                            Is he inviting you to [b]live with him[/b]?
                            Oh goodness!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Leave</button>`;

        var tooltip = global.parser.parse(`You can't stay here. You'd only cause trouble for him in the long run.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Stay</button>`;

        var tooltip = global.parser.parse(`Let's raise the baby together, as a family.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var FarmerMaidDrunkSatisfyDadChoice = {
    LEAVE: 1,
    STAY: 2
};
