/**
 * Stage section of Enslavement, Noon Two.
 * @param {SlaveNoonTwoController} controller The Enslavement, Noon Two controller.
 */
function SlaveNoonTwoSmile(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = SlaveNoonTwoStageChoice.SMILE;
    var reqController = ctrl.stage;
    var subController = ctrl.preResult;
    var changeController = ctrl.smile;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.results;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.enslavement.noonTwo.stageSmiled = true;
        var injected = ctrl.gag.has();
        var eager = global.kobold.history.shared.enslavement.noonTwo.kissedBallsEager;
        var toReturn = `[p]
                            Smile?
                            For these [i]oafs[/i] that, after having been riled up by Bartholomew, pretty much want you [b]dead[/b]?
                            ${eager ? `Well, it would probably make Fenrir happy` : `Well, it [b]will[/b] up your odds of being [i]sold[/i]`},
                            and showing off how [i]nice[/i] you are [b]might[/b] make whoever ends up buying you just a little bit less inclined to [b]cave your head in[/b].
                            If you [b]really[/b] have to, out of sheer self-preservation, you think you [i]might[/i] be able to force out a weak happy look.
                        [/p]
                        [p]
                            ${injected ? `Even with the hefty gag in your mouth,
                            you still manage to form something remotely managing a smile.` : `Slowly, a small, obviously forced smile forms on your face.`}
                            In order to [i]accentuate[/i] your [i]happiness[/i] further, you form the fingers of both your hands into peace signs.
                            "[color=brown]You see?
                            Totally compliant![/color]"
                            Bartholomew comments, the onlookers nodding contently, some more pleased than others.
                            The father of the family with the avian son, in particular, seems happy with your actions.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
