/**
 * Information about the class.
 * @param {string} sceneId The id of the scene.  
 */
function ForestBanditInterveneController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;

    var interveneSourceId = "banditInterveneSource";
    var interveneId = "banditInterveneChoice";

    var bargainId = "banditBargainChoice";

    var relieveId = "banditRelieveChoice";
    var relieveGenderId = "banditRelieveGender";
    var relieveDickId = "banditRelieveDick";
    var relievePantsId = "banditRelievePants";
    var relieveUnderwearId = "banditRelieveUnderwear";
    var relieveContinueId = "banditRelieveContinue";
    var relieveChangeId = "banditRelieveChange";

    var replaceId = "banditReplaceChoice";
    var replaceGenderId = "banditReplaceGender";
    var replaceDickId = "banditReplaceDick";
    var replacePantsId = "banditReplacePants";
    var replaceUnderwearId = "banditReplaceUnderwear";
    var replaceBraId = "banditReplaceBra";
    var replaceRemoveId = "banditReplaceRemove";
    var replaceChangeId = "banditReplaceChange";

    var acceptId = "banditAcceptChoice";
    var acceptContinueId = "banditAcceptContinue";

    var protestContinueId = "banditProtestContinue";
    var protestChangeId = "banditProtestChange";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Sets the source for the intervene.
     * @param {ForestBanditInterveneSourceEnum} source The source scene.
     */
    this.setInterveneSource = function (source) {
        var history = global.kobold.history.get(id);
        history[interveneSourceId] = source;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the intervene source from history and returns it.
     * May return undefined if the source has not yet been set.
     * @returns {ForestBanditInterveneSourceEnum} The source scene.
     */
    this.getInterveneSource = function () {
        var history = global.kobold.history.get(id);
        return history[interveneSourceId];
    };

    /**
     * Returns true if the intervene source has been set.
     */
    this.hasInterveneSource = function () {
        return global.kobold.history.has(id, interveneSourceId);
    };

    /**
     * Makes the intervene choice.
     * @param {ForestBanditInterveneChoiceEnum} choice The choice.
     */
    this.makeInterveneChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[interveneId] = choice;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the intervene choice from history and returns it.
     * May return undefined if the intervene choice has not yet been set.
     */
    this.getInterveneChoice = function () {
        var history = global.kobold.history.get(id);
        return history[interveneId];
    };

    /**
     * Returns true if the intervene choice has been made.
     */
    this.hasMadeInterveneChoice = function () {
        return global.kobold.history.has(id, interveneId);
    };


    /**
     * Makes the bargain choice.
     * @param {ForestBanditBargainChoice} choice The choice.
     */
    this.makeBargainChoice = function (choice) {
        global.kobold.history.setKey(id, bargainId, choice);
    };

    /**
     * Fetches the intervene choice from history and returns it.
     * May return undefined if the intervene choice has not yet been set.
     */
    this.getBargainChoice = function () {
        return global.kobold.history.getKey(id, bargainId);
    };

    /**
     * Returns true if the intervene choice has been made.
     */
    this.hasMadeBargainChoice = function () {
        return global.kobold.history.has(id, bargainId);
    };


    /**
     * Makes the relieve choice.
     * @param {ForestBanditRelieveChoice} choice The choice.
     */
    this.makeRelieveChoice = function (choice) {
        global.kobold.history.setKey(id, relieveId, choice);
        if (choice === ForestBanditRelieveChoice.EAGER) {
            global.kobold.body.oralVirgin = false;
        }
    };

    /**
     * Fetches the relieve choice from history and returns it.
     * May return undefined if the relieve choice has not yet been set.
     */
    this.getRelieveChoice = function () {
        return global.kobold.history.getKey(id, relieveId);
    };

    /**
     * Returns true if the relieve choice has been made.
     */
    this.hasMadeRelieveChoice = function () {
        return global.kobold.history.has(id, relieveId);
    };


    /**
     * Relieve gender.
     */
    this.relieveGender = function () {
        var gender = global.kobold.body.getBaseGender();
        global.kobold.history.setIfNotSet(id, relieveGenderId, gender);
        return global.kobold.history.getKey(id, relieveGenderId);
    };

    /**
     * Relieve dick.
     */
    this.relieveDick = function () {
        var dick = global.kobold.body.hasDick();
        global.kobold.history.setIfNotSet(id, relieveDickId, dick);
        return global.kobold.history.getKey(id, relieveDickId);
    };

    /**
     * Relieve pants.
     */
    this.relievePants = function () {
        var pants = global.kobold.inventory.equipment.pants;
        global.kobold.history.setIfNotSet(id, relievePantsId, pants);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, relievePantsId));
    };

    /**
     * Relieve underwear.
     */
    this.relieveUnderwear = function () {
        var underwear = global.kobold.inventory.equipment.underwear;
        global.kobold.history.setIfNotSet(id, relieveUnderwearId, underwear);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, relieveUnderwearId));
    };

    function relieveChangeEager() {
        global.kobold.history.doIfNotSet(id, relieveChangeId, true, function () {
            global.kobold.status.exertion.increase(25, "Eager bandit threesome");
            global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.UNHELPED;
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.EAGER_THREESOME;
        });
        return global.ui.change.buildNotification([
            new Change("Threesome", "Sexual exhaustion", +25, true)
        ]);
    }

    function relieveChangeReluctant() {
        global.kobold.history.doIfNotSet(id, relieveChangeId, true, function () {
            global.kobold.status.exertion.increase(10, "Reluctant bandit handjobs");
            global.kobold.status.mental.decrease(10, "Reluctant bandit handjobs");
            global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.UNHELPED;
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.RELUCTANT_THREESOME;
        });
        return global.ui.change.buildNotification([
            new Change("Handjobs", "Sexual exhaustion", +10, true),
            new Change("Rough treatment", "Sanity", -10)
        ]);
    }

    /**
     * Relieve change.
     */
    this.relieveChange = function () {
        if (that.getRelieveChoice() == ForestBanditRelieveChoice.EAGER) {
            return relieveChangeEager();
        } else {
            return relieveChangeReluctant();
        }
    };

    /**
     * Relieve continue.
     */
    this.relieveContinue = function () {
        global.kobold.history.setKey(id, relieveContinueId, true);
    };

    /**
     * Returns true if the relieve screen has continued.
     */
    this.relieveHasContinued = function () {
        return global.kobold.history.has(id, relieveContinueId);
    };


    /**
     * Makes the replace choice.
     * @param {ForestBanditReplaceChoice} choice The choice.
     */
    this.makeReplaceChoice = function (choice) {
        global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.RAN_AWAY;
        global.kobold.history.setKey(id, replaceId, choice);
        if (choice === ForestBanditReplaceChoice.PROTEST) {
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.GANGRAPE;
        }
    };

    /**
     * Fetches the replace choice from history and returns it.
     * May return undefined if the replace choice has not yet been set.
     */
    this.getReplaceChoice = function () {
        return global.kobold.history.getKey(id, replaceId);
    };

    /**
     * Returns true if the replace choice has been made.
     */
    this.hasMadeReplaceChoice = function () {
        return global.kobold.history.has(id, replaceId);
    };


    function acceptEnjoy() {
        global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.EAGER_GANGBANG;
        global.kobold.history.shared.tenPawGang.footsie = TenPawGangFootsieEnum.OBSERVED;
    }

    function acceptHate() {
		if (that.replaceGender() === GenderEnum.MALE) {
            global.kobold.body.analVirgin = false;
        } else {
            global.kobold.body.vaginalVirgin = false;
        }
        global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.RELUCTANT_GANGBANG;
    }

    /**
     * Makes the accept choice.
     * @param {ForestBanditAcceptChoice} choice The choice.
     */
    this.makeAcceptChoice = function (choice) {
        global.kobold.history.setKey(id, acceptId, choice);

        global.kobold.body.oralVirgin = false;

        if (choice === ForestBanditAcceptChoice.ENJOY) {
            acceptEnjoy();
        } else {
            acceptHate();
        }
    };

    /**
     * Fetches the accept choice from history and returns it.
     * May return undefined if the accept choice has not yet been set.
     */
    this.getAcceptChoice = function () {
        return global.kobold.history.getKey(id, acceptId);
    };

    /**
     * Returns true if the accept choice has been made.
     */
    this.hasMadeAcceptChoice = function () {
        return global.kobold.history.has(id, acceptId);
    };


    /**
     * Accept continue.
     */
    this.acceptContinue = function () {
        global.kobold.history.setKey(id, acceptContinueId, true);
    };

    /**
     * Returns true if the accept screen has continued.
     */
    this.acceptHasContinued = function () {
        return global.kobold.history.has(id, acceptContinueId);
    };


    /**
     * Protest continue.
     */
    this.protestContinue = function () {
        global.kobold.history.setKey(id, protestContinueId, true);
    };

    /**
     * Returns true if the protest screen has continued.
     */
    this.protestHasContinued = function () {
        return global.kobold.history.has(id, protestContinueId);
    };


    this.protestChange = function () {
        global.kobold.history.doIfNotSet(id, protestChangeId, true, function () {
            global.kobold.status.exertion.increase(50, "Raped");
            global.kobold.status.mental.decrease(50, "Raped");
            global.kobold.status.physical.decrease(20, "Punched in the gut");
            if(global.kobold.body.dick) {
                global.kobold.body.analVirgin = false;
            } else {
                global.kobold.body.vaginalVirgin = false;
            }
        });

        return global.ui.change.buildNotification([
            new Change("Raped", "Sexual exhaustion", +50, true),
            new Change("Raped", "Sanity", -50),
            new Change("Punched in the gut", "Health", -20)
        ]);
    };


    /**
     * Replace gender.
     */
    this.replaceGender = function () {
        var gender = global.kobold.body.getBaseGender();
        global.kobold.history.setIfNotSet(id, replaceGenderId, gender);
        return global.kobold.history.getKey(id, replaceGenderId);
    };

    /**
     * Replace dick.
     */
    this.replaceDick = function () {
        var dick = global.kobold.body.hasDick();
        global.kobold.history.setIfNotSet(id, replaceDickId, dick);
        return global.kobold.history.getKey(id, replaceDickId);
    };

    /**
     * Replace pants.
     */
    this.replacePants = function () {
        var pants = global.kobold.inventory.equipment.pants;
        global.kobold.history.setIfNotSet(id, replacePantsId, pants);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, replacePantsId));
    };

    /**
     * Replace bra.
     */
    this.replaceBra = function () {
        var bra = global.kobold.inventory.equipment.bra;
        global.kobold.history.setIfNotSet(id, replaceBraId, bra);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, replaceBraId));
    };

    /**
     * Strips the kobold.
     */
    this.replaceRemoveClothing = function () {
        global.kobold.history.doIfNotSet(id, replaceRemoveId, true, function () {
            global.kobold.inventory.equipment.stripNaked();
        });
    };

    /**
     * Replace underwear.
     */
    this.replaceUnderwear = function () {
        var underwear = global.kobold.inventory.equipment.underwear;
        global.kobold.history.setIfNotSet(id, replaceUnderwearId, underwear);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, replaceUnderwearId));
    };

    this.replaceEagerChange = function () {
        global.kobold.history.doIfNotSet(id, replaceChangeId, true, function () {
            global.kobold.status.exertion.increase(50, "Bandit foursome");
            global.kobold.status.stamina.decrease(20, "Bandit foursome");
        });

        return global.ui.change.buildNotification([
            new Change("Bandit foursome", "Sexual exhaustion", +50, true),
            new Change("Bandit foursome", "Stamina", -20)
        ]);
    };

    this.replaceHateChange = function () {
        global.kobold.history.doIfNotSet(id, replaceChangeId, true, function () {
            global.kobold.status.exertion.increase(60, "Cruel bandit threesome");
            global.kobold.status.stamina.decrease(20, "Nearly choked to death");
            global.kobold.status.physical.decrease(20, "Nearly choked to death");
            global.kobold.status.mental.decrease(30, "Nearly choked to death");
        });

        return global.ui.change.buildNotification([
            new Change("Cruel threesome", "Sexual exhaustion", +60, true),
            new Change("Nearly choked to death", "Health", -20),
            new Change("Nearly choked to death", "Sanity", -30),
            new Change("Nearly choked to death", "Stamina", -20)
        ]);
    };
}
