/**
 * Information about the class.
 * @param {ForestController} controller The forest controller.
 * @param {ForestBanditIntervene} intervene The intervene section.
 * @param {ForestBanditReward} reward The reward section.
 */
function ForestBanditRock(controller, intervene, reward) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toIntervene = intervene;
    var toReward = reward;

    var scaryId = "banditRockScaryButton";
    var interveneId = "banditRockInterveneButton";

    var content = ` <div class=text>
                        Luckily, you picked up a ${global.kobold.inventory.createItemWithTooltip(new Rock())} earlier,
                        The perfect weapon to draw first blood without even having to leave your concealed location.
                        You step out from behind the tree, your body concealed still by the bush right next to it.
                        You clutch the ${global.kobold.inventory.createItemWithTooltip(new Rock())} in your right hand, 
                        and set your sights on one of the kobolds pinning the dragoness' arms.
                    </div>
                    <div class=text>
                        Uncertainty and panic slowly settle in your mind as you take aim.
                        You've never really <span class="italic">thrown</span>, well, <span class="italic">anything</span> with the intent of hurting someone.
                        And if you miss, you could just as easily hit the woman.
                    </div>
                    <div class=text>
                        As your arm is already swinging forwards to propel the ${global.kobold.inventory.createItemWithTooltip(new Rock())} towards the gangmembers,
                        you close your eyes out of sheer self-doubt, your mind flipping sides from <span class="italic">do it</span> to <span class="italic">don't</span>.
                        But at that point, it was already too late.
                    </div>
                    <div class=text>
                        At the very apex of your pitch, your scaled fingers untangle themselves from the cold, hard surface of the heavy stone,
                        which is sent flying through the air.
                        With your eyes closed, you can only hope you didn't <span class="italic">screw up</span>.
                    </div>
                    <div class="text">
                        A loud, somewhat masculine scream rings throughout the forest, 
                        followed by the soft thud of a lightweight body falling atop the leafy forest floor.
                        Instinctively, you press yourself against the tree-trunk once more, 
                        re-entering cover before opening your fearfully shut eyes and peeking out from behind it.
                    </div>
                    <div class="text">
                        You managed to score a direct hit on your target, the kobold knocked out cold from the harsh impact to his cranium.
                        He lies motionlessly on the forest floor, one of the dragoness' arms freed once more, 
                        kept only in check by the threatening blade pressed to her throat.
                    </div>
                    <div class="text">
                        The knocked out bandit's companions gaze around the clearing fearfully.
                        A few panicked lines of dialogue come to pass before their leader brings his men in line,
                        the knife-wielding kobold clearly more than a little startled himself.
                        He gazes around warily, seemingly unable to pinpoint your position.
                        He then speaks out loudly, on the verge of shouting, in the deepest voice he can seemingly muster:
                    </div>
                    <div class="text center">
                        <span class="bold brightred">
                            Who's there?! 
                            Come out and fight!
                            Quit being a coward and face us! 
                            Come out or I'll kill the woman!!
                        </span>
                    </div>
                    <div class="text">
                        The other three, meanwhile, are practically cowering in fear.
                        They whisper amongst themselves about stories of ghosts and spectres,
                        about how they've <span class="bold">angered</span> the forest.
                    </div>`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        return content;
    }

    /**
     * Builds and returns the scary noise choice button.
     */
    function buildScaryNoiseChoice() {
        var disabled = ctrl.bandit.hasMadeRockChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.getRockChoice() == ForestBanditRockChoiceEnum.SCARYNOISE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${scaryId}" ${disabled}>Scare them</button>`;

        var tooltip = `Making a loud noise might scare the kobolds enough to send them running.
                       Then again, you'd be giving away your position at the same time.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 215px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the intervene choice button.
     */
    function buildInterveneChoice() {
        var disabled = ctrl.bandit.hasMadeRockChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.getRockChoice() == ForestBanditRockChoiceEnum.INTERVENE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${interveneId}" ${disabled}>Intervene</button>`;

        var tooltip = `With one of the kobolds knocked out, you might stand a better chance at arguing with them, or perhaps even fighting them.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 225px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildScaryNoiseChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildInterveneChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables both buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + scaryId), $("#" + interveneId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the rock button.
     */
    function registerScaryListener() {
        $("#" + scaryId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bandit.setRewardSource(ForestBanditRewardSourceEnum.SCARYNOISE);
            ctrl.bandit.makeRockChoice(ForestBanditRockChoiceEnum.SCARYNOISE);
            loadNextPart(toReward);
        });
    }

    /**
     * Adds a listener to the rock button.
     */
    function registerInterveneListener() {
        $("#" + interveneId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bandit.intervene.setInterveneSource(ForestBanditInterveneSourceEnum.ROCK);
            ctrl.bandit.makeRockChoice(ForestBanditRockChoiceEnum.INTERVENE);
            loadNextPart(toIntervene);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "banditRock";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.bandit.getArrivalChoice() !== ForestBanditArrivalChoiceEnum.ROCK) {
            return "";
        }

        var change = global.ui.change.buildNotification([
            new Change("Thrown rock", new Rock().name(true), -1)
        ]);
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += change;
        part += buildChoice();
        part += `</div>`;
        if (ctrl.bandit.getRockChoice() === ForestBanditRockChoiceEnum.INTERVENE) {
            part += toIntervene.build();
        }
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.bandit.getArrivalChoice() !== ForestBanditArrivalChoiceEnum.ROCK) {
            return;
        }

        // toIntervene.registerListeners(); // Already registered in arrival.
        // toReward.registerListeners(); // Already registered in arrival.

        if (ctrl.bandit.hasMadeRockChoice()) {
            return;
        }

        registerScaryListener();
        registerInterveneListener();
    };
}

/**
 * Choices after the rock has been throwin in the bandit encounter.
 */
var ForestBanditRockChoiceEnum = {
    SCARYNOISE: 1,
    INTERVENE: 2
};

