/**
 * Hide section of the woods.
 * @param {ForestController} controller The Forest controller.
 */
function ForestWoodsThrowContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You continue to run for a solid few minutes, ensuring you've shaken off both the guards [i]and[/i] the bees before coming to a brief halt.
                            All things considered, that went pretty well!
                            While they might not even [i]know[/i] it was [b]you[/b] who did it, you gave those guards a run for their money,
                            using nothing but your wits, your athleticism, and your forest knowledge.
                            Heck, even without the bees, they probably would have given up before finding your excellent hiding spot!
                        [/p]
                        [p]
                            Sure, you kind of feel bad for those guys.
                            They were probably just trying to do their job.
                            You [i]did[/i] steal from them, after all.
                            But still, who's going to miss a single two hundred gold bag from a transport carrying tens of thousands of coins?
                            Nobody, that's who.
                            You did what you had to.
                            And more importantly: you got away with it, at least for now.
                        [/p]
                        [p]
                            After catching your breath, you realize the direction in which you ran from the bees conveniently brought you closer to the path leading home.
                            Given that the sun is slowly starting to go down, you figure it's best if you return to your own little shack in the woods.
                            You don't want to stay out here at night, after all.
                            That's when the [i]worst[/i] of the bandits come out, and you [i]don't[/i] want to mess with [i]those[/i] guys,
                            especially not with two hundred gold dangling around your neck.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.homeSceneName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.preContinue.continued()) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.preContinue.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}
