/**
 * Dodge section of the unknown scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestUnknownDodgeContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = `runIntoWoodsContinueButton`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            For the second time today, you find yourself running for your life.
                            You don't know whether or not the plant is still giving chase, but fright and adrenaline combined make you [b]keep running[/b] without looking back.
                            It isn't until the sun has gone down, that you come to a [b]completely[/b] exhausted stop.
                        [/p]
                        [p]
                            You slump down against a nearby tree, 
                            tightly clutching the tattered remains of what was once your shirt while loudly panting.
                            You [b]ruined[/b] it.
                            You've had that sweater for [b]years[/b].
                            It's served as a blanket, a garment and even a towel on rainy days.
                            It was the closest thing to a stuffed toy you've ever owned.
                            And now it's gone.
                        [/p]
                        [p]
                            After taking a few more minutes to catch your breath and choke back tears for your lost sweater, you slowly rise to your feet once more.
                            You are tired.
                            [b]Very[/b] tired.
                            But you can't rest.
                            Not yet.
                            Not here.
                            It's too dangerous.
                            You take a look around.
                            You have no clue where you are.
                            All directions you gaze in look exactly the same.
                            Trees and darkness all around you.
                            You are officially [b]lost[/b].
                        [/p]
                        [p]
                            You carefully put on the remnants of your ${global.kobold.inventory.createItemWithTooltip(new RippedWoolSweater())}.
                            It no longer keeps you warm, nor will it provide much defence from whatever the world hurls at you.
                            But you don't wear it for that.
                            You wear it as a memento.
                            To the best shirt that ever was, and the best shirt that ever will be.
                        [/p]
                        [p]
                            Looks like you're going to have to pull an all nighter,
                            unless you can find a relatively safe place to catch some sleep in.
                            You're bound to run out of steam at some point, but that's a problem you'll face when it emerges.      
                            For now, you need to keep going.                      
                            As long as you keep heading in one direction, you'll reach the edge of the forest at one point or another, right?
                            You hope so, at least.
                        [/p]
                        [p]
                            With your body in shambles, and your spirit demoralized, you begin to stride even deeper into the forest,
                            in hopes of ultimately finding your way back home, lest you [i]become[/i] one of the savages that live here, 
                            completely isolated from civilization.
                            At least you learned a thing or two by dodging those tentacles.
                            If you keep being as alert and uncatchable as you were while fighting that monster, you're bound to make it out of here alive.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.unknown.hasContinued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.continue();
            loadNextPart("RiversideMarsh", "Deepwoods");
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "forestUnknownDodgeContinue";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.unknown.yank.dodge.continued()) {
            return "";
        }

        global.ui.applyTheme("forest_night");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.unknown.yank.dodge.continueChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.unknown.yank.dodge.continued()) {
            return;
        }
        // REGISTER CHILDREN HERE

        if (ctrl.unknown.hasContinued()) {
            return;
        }

        registerContinueListener();
    };
}
