/**
 * Barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function BarnRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = "barnRunContinueButton";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You decide not to push your luck any further.
                            You'd rather not risk wasting away in one of these animal pens, or perhaps even getting flayed alive, just so you can get some revenge.
                            Looking around for avenues of escape, you notice the barn has a small back door, the large iron lock of which is clearly open.
                            Without alerting Cassidy, you creep towards it.
                        [/p]
                        [p]
                            The moment you reach the back exit, the vixen rises from her crouched searching pose,
                            ${global.kobold.inventory.createItemWithTooltip(new FemPlus())} in hand.
                            She turns around, just in time to witness you [b]slamming[/b] the door shut behind you.
                            You take in a deep breath of fresh air, the sight of the afternoon sky inspiring a sense of safety inside of you.
                            Like you've made it out.
                            Like you're a free kobold once more.
                            But you're not out of trouble yet.
                            For all you know, that crazy [b]bitch[/b] could be rushing after you this very instant!
                        [/p]
                        [p]
                            While Cassidy may be twice your size, you're confident you can at least [b]outrun[/b] the canine.
                            Especially if she's still wearing those cowboy boots of hers.
                            Knocking over a nearby barrel to somewhat [i]block[/i] the door you just came from,
                            you begin to mindlessly sprint away, in the general direction of Varanar.
                            You aren't sure whether or not she's in pursuit, but you'd rather not take any chances.
                        [/p]
                        [p]
                            After about half an hour of intensive running through fields, farms and small patches of woodland,
                            you think you've finally escaped the wicked woman fully, if she was even on your tail to begin with.
                            Better safe than sorry.
                            You find yourself back on the eastern road to Varanar, just a few miles away from the city limits.
                            You are exhausted.
                            [b]Very[/b] exhausted.
                            You lean against a lone nearby tree to catch your breath.
                        [/p]
                        [p]
                            You keep replaying the barn-scenario in your head.
                            It was a close call.
                            [b]Too[/b] close.
                            If you hadn't figured out a way to escape that table, who knows what Cassidy would have done to you?!
                            You might have ended up spending your entire life getting fucked nonstop by a bunch of your drug-infused peers.
                        [/p]
                        [p]
                            You take a moment to lament the other kobolds imprisoned by the vixen.
                            Sadly, there isn't much you can do for them.
                            What Cassidy is doing is completely [b]legal[/b], despite being abhorrent, undeniably [b]evil[/b] and totally sickening.
                            Disheartened, you let out a loud sigh.
                            This world is unfair, especially for kobolds.
                            At least [b]you[/b] got out of there alive, and that's what really matters.
                        [/p]
                        [p]
                            Having recuperated enough to continue your journey, you begin to walk down the road towards Varanar.
                            At least there aren't any kobold breeding farms there.
                            Or so you hope.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "barnRun";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unaware.get() !== BarnUnawareChoice.RUN) {
            return "";
        }

        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.unawareRun.change();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unaware.get() !== BarnUnawareChoice.RUN) {
            return;
        }

        //toEager.registerListeners();

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}
