/**
 * Not barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function NotBarnTeaDoor(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var disagreeId = "notBarnDisagreeButton";
    var agreeId = "notBarnAgreeButton";
    var attackId = "notBarnAttackButton";

    var toDisagree = new NotBarnTeaDisagree(ctrl);
    var toAgree = new NotBarnTeaAgree(ctrl);
    var toAttack = new NotBarnAttack(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var name = ctrl.misc.name();
        var toReturn = `[p]
                            Well, you [b]have[/b] spent most of your day thus far walking through the blistering cold.
                            You could use a hot beverage, if just to warm yourself up a bit.
                            You nod in agreement.
                            A smile forms on the vulpine's face.
                            But she better not try to seduce you or anything!
                            Her time to roll her eyes.
                        [/p]
                        [p]
                            Cassidy walks past you, beckoning you to follow her.
                            The both of you step outside.
                            You traverse the small dirt path, heading back towards the fancy mansion adjacent to the much older barn.
                            As you walk, you decide to strike up a small conversation.
                            You ask the vixen what [i]exactly[/i] it is she breeds here.
                            You never really [i]did[/i] get a good look at the inside of those pens.
                        [/p]
                        [p]
                            The canine takes a moment to think.
                            She speaks slowly, carefully selecting her words.
                            "[color=orange]Oh, you know.
                            Small, cute things.
                            Some people like keeping them as pets.
                            Others like eating them.
                            A select group of individuals, [small]myself included[/small], even enjoys [i]sleeping[/i] with them.[/color]"
                        [/p]
                        [p]
                            It's as though she doesn't [b]want[/b] to tell you what is hidden in that barn of hers.
                            But that makes you even [b]more[/b] curious.
                            You begin to think.
                            A small, cute pet.
                            Rabbits?
                            People like eating rabbits.
                            And earlier on, you even thought that that might have been it.
                            But people don't have [b]sex[/b] with rabbits, [i]or do they[/i]?
                            You're pretty sure that they don't.
                            And even if they did, you doubt Cassidy would outright tell you she's a [i]bunnyfucker[/i].
                        [/p]
                        [p]
                            As the two of you near the front door, you are just as clueless as you were [i]before[/i] her hints.
                            With an inquisitive look on your face, you repeat your question: what [b]does[/b] she breed?
                            Her smile turns into a nervous grin.
                            You can [i]feel[/i] that she's uncomfortable discussing this topic, but you really want to know.
                            You insist.
                        [/p]
                        [p]
                            The vixen takes a step forward, towards the door, her bare back facing you.
                            "[color=orange]Well...[/color]"
                            she begins, whilst deeply bending over, lifting her tail up high to give you a clear look at her furred rump,
                            the fertile loins placed in between her thick thighs distracting you more than a little, 
                            almost as if she's trying to trick you, to prevent you from focussing on her reply.
                            While your eyes are glued to to the vulpine's groin, sheer inquisitiveness causes your attention to remain on-point.
                            Finally, the answer is revealed to you.
                            "[color=orange]...[small]kobolds[/small]. I breed kobolds.[/color]"
                        [/p]
                        [p]
                            What?
                            Kobolds?
                            Your kind?
                            Wait.
                            She said she bred things for fucking, for selling them as pets, and for...[b]eating[/b]?!
                            Reaching into her boot, Cassidy produces a small metal object from her footwear.
                            The key to her house.
                            Looking back at you, she notices how upset you are.
                            She realizes her [i]distraction[/i] must not have worked.
                        [/p]
                        [p]
                            The canine sighs loudly.
                            Stepping towards the door, she begins to ratify her actions as she fiddles with the lock, her tone changing considerably.
                            "[color=orange]It's a better life for [b]most[/b] of them.
                            Without me, they'd end up on the streets, or as bandits in the forest.
                            Sure, some of the young ones are marked for death, but the rest of the kobolds under my care are treated fairly.
                            They haven't a care in the world, and some of them may even end up being [i]adopted[/i].
                            Surely you can see the logic in my actions, right, ${name}?[/color]"
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildDisagreeChoice() {
        var disabled = ctrl.notbarn.has() ? "disabled" : "";
        var isSelected = ctrl.notbarn.get() === NotBarnTeaDoorChoice.DISAGREE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${disagreeId}" ${disabled}>Disagree</button>`;

        var tooltip = global.parser.parse(`What?! No! Your kind is [b]more[/b] than just [b]cattle[/b]!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildAgreeChoice() {
        var disabled = ctrl.notbarn.has() ? "disabled" : "";
        var isSelected = ctrl.notbarn.get() === NotBarnTeaDoorChoice.AGREE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${agreeId}" ${disabled}>Agree</button>`;
        var tooltip = global.parser.parse(`Constant sex, free food and nothing to worry about? Sounds like she's doing them a favor!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildAttackChoice() {
        var weapon = ctrl.misc.weapon();
        var disabled = ctrl.notbarn.has() || !weapon ? "disabled" : "";
        var isSelected = ctrl.notbarn.get() === NotBarnTeaDoorChoice.ATTACK;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${attackId}" ${disabled}>Attack her</button>`;

        var tooltip;
        if (weapon) {
            tooltip = global.parser.parse(`How [b]dare[/b] she keep your kind as [b]sex slaves[/b]?! Time to teach this bitch a lesson she won't soon forget.`);
        } else {
            tooltip = global.parser.parse(`You don't have any weapons to attack the woman with.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildDisagreeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAgreeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAttackChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + disagreeId), $("#" + agreeId), $("#" + attackId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerDisagreeListener() {
        $("#" + disagreeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.notbarn.set(NotBarnTeaDoorChoice.DISAGREE);
            loadNextPart(toDisagree);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerAgreeListener() {
        $("#" + agreeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.notbarn.set(NotBarnTeaDoorChoice.AGREE);
            loadNextPart(toAgree);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerAttackListener() {
        $("#" + attackId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.notbarn.set(NotBarnTeaDoorChoice.ATTACK);
            loadNextPart(toAttack);
        });
    }
    

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "notBarnTeaDoor";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.tea.get() !== NotBarnTeaChoice.ACCEPT) {
            return "";
        }
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toDisagree.build();
        part += toAgree.build();
        part += toAttack.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.tea.get() !== NotBarnTeaChoice.ACCEPT) {
            return;
        }
        
        toDisagree.registerListeners();
        toAgree.registerListeners();
        toAttack.registerListeners();

        if (ctrl.notbarn.has()) {
            return;
        }

        registerDisagreeListener();
        registerAgreeListener();
        registerAttackListener();
    };
}

var NotBarnTeaDoorChoice = {
    DISAGREE: 4,
    AGREE: 5,
    ATTACK: 6
};
