/**
 * Bottom section of Hamlet, the Well.
 * @param {HamletWellController} controller The Hamlet, Well controller.
 */
function HamletWellPants(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preContinue;
    var changeController = ctrl.pants;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new HamletWellStuck(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var pants = ctrl.misc.pants();
        var drank = global.kobold.history.shared.well.drankWater;
        var drowned = global.kobold.history.shared.well.drowned;
        var rocky = global.kobold.history.shared.well.fell;
        var arrivalChoice = ctrl.arrival.get();
        var wanted = arrivalChoice === HamletWellArrivalChoice.CLIMB || arrivalChoice === HamletWellArrivalChoice.ROPE;
        var arrivalDescriptor = `You may not have wanted to go down this damp hole, but now that you're here, you might as well explore it fully.`;
        if (wanted) {
            arrivalDescriptor = `You wanted to explore this place, and you're not going to back down just because the air in here is a little stale.`;
            if (rocky) {
                arrivalDescriptor = `Your entrance might not have been very [i]graceful[/i], `;
                arrivalDescriptor += `but you're not going to let a single small setback dissuade you from properly exploring this damp hole.`;
            }
        }
        var toReturn = `[p]
                            You don't concern yourself with escaping the well just yet.
                            ${arrivalDescriptor}
                            You take a few deep breaths, filling your ${drowned ? `still slightly aching` : ``} lungs with as much air as they can possibly hold.
                            Steeling your nerves, you dip your head
                            underwater${rocky ? ` yet again` : ``}${drowned ? `, your mind working overtime to suppress flashbacks
                            from your near-death experience just a few minutes ago` : ``}.
                            Athletically lifting the lower half of your body upwards, while simultaneously sinking your torso deeper,
                            you dive down and submerge yourself completely.
                        [/p]
                        [p]
                            With your eyes having adapted to the dark, and all kobolds luckily being able to see underwater,
                            you immediately notice something dinstinctly [b]shiny[/b] at the bottom of the well.
                            Coins, perhaps?
                            Maybe ${wanted ? `coming down here wasn't such a bad idea after all` : `falling down here wasn't such a bad thing after all`}.
                        [/p]
                        [p]
                            You swim down further, until you suddenly feel a dull scraping at the side of your leg.
                            Looking upwards, you find out that your thigh accidentally rubbed along a sharp rock protruding
                            from the edge of the well${rocky ? `, one you were lucky enough to miss on your earlier way down, ` : ``} without you noticing.
                            Thankfully, your ${global.kobold.inventory.createItemWithTooltip(pants)} protected your scaly hide.
                        [/p]
                        [p]
                            You were so focussed on the promise of gold that you momentarily forgot you're swimming around in a decrepit, age-old well.
                            You don't think the people that made this thing were too concerned about kobolds swimming around in it thousands of years in the future.
                            Alas, your ${global.kobold.inventory.createItemWithTooltip(pants)} were completely [b]torn apart[/b] by the unfortunate scraping.
                            You steer clear of the sharp rock, tear off the remnants of your pants and toss them to the side, letting them float to the bottom of the well.
                            If that [i]is[/i] gold you spy, you'll be able to buy a new pair later.
                        [/p]
                        [p]
                            With your ruined pants discarded, you continue swimming downwards.
                            A few strokes later, you arrive at the muddy ground.
                            To your pleasant surprise, the glint you saw from afar was indeed a shiny coin!
                            And it's not alone.
                            Brushing off some of the mud, you find piece after piece of gold, probably tossed down here by farmers wishing for good fortune.
                            Their prayers were answered, maybe not for them, but at least for you.
                        [/p]
                        [p]
                            You pick through the watery dirt, scrounging as many coins as you can before needing to go up for air.
                            Making sure to avoid hitting the same sharp stone again, you flip yourself upright once more,
                            kick off from the muddy ground, and swim back up to the surface.
                        [/p]
                        [p]
                            Surfacing from your dive, you take a closer look at the armful of coins you salvaged.
                            Most of them are coated with years worth of dirt.
                            You take the time to individually rinse them off in the water, before inspecting them further.
                            They seem to be pretty old, engraved with a design you've never quite seen before.
                            Oh well, you're certain they're still good for use.
                        [/p]
                        [p]
                            You notice that none of the coins have rusted,
                            despite being underwater for what you can only imagine to be quite a long time.
                            Come to think of it, you've never actually seen a rusty piece of gold.
                            You think that perhaps gold [i]can't[/i] rust, but you're not quite certain.
                            It would make sense to make money out of something that can't fall prey to the elements, right?
                            You briefly wonder why people don't make [i]unrustable[/i] weapons out of gold,
                            but you know pretty much nothing of smithing [i]or[/i] weaponry,
                            so you quickly drop the topic and get to work putting the coins into your pouch.
                        [/p]
                        [p]
                            Right as you were about to dive down a second time to gather more shinies, a loud [i]rumbling[/i] noise echoes throughout the well.
                            Everything begins to shake.
                            Small pebbles and rocks break off the wall and plunge into the water besides you.
                            Earthquake?!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        if (changeController) {
            part += changeController.change();
        }
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
