/**
 * Reflect section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveReflectionLeave(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = UndergroundCaveReflectionSucceedChoice.LEAVE;
    var reqController = ctrl.reflectSucceed;
    var setChoice = false;
    var subController = ctrl.preContinue;
    var changeController = ctrl.reflectLeave;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new UndergroundCaveReflectionLeaveContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the beast subdued, you have little reason to stick around.
                            You could probably [i]kill it[/i] or something,
                            but killing a defenceless, surrendering animal seems pointlessly cruel.
                            And you really don't want to tempt fate by trying anything [i]dumb[/i].
                        [/p]
                        [p]
                            You slowly back away from the basilisk,
                            holding the mirror shard up like a preacher would a crucifix in front of a vampire.
                            It doesn't move, instead staying in place, lying on its back, its tongue partially lolling out of its gaping maw.
                            The monster looks like it's playing dead, which is almost funny, in a way.
                        [/p]
                        [p]
                            You put some distance in between yourself and the crocodillian, backing off until its green-glowing eyes start to fade into the dark.
                            Despite no longer being able to see them, you can still picture those two intriguing emerald orbs in your head.
                            While the big reptile is no longer a threat to you, you don't think you'll ever be able to forget its terrifying, soul-gazing glare.
                            With the monster out of sight, you discard the glass shard, turn tail and flee.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            setChoice ? subController.set(setChoice) : subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }


        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
