/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorCutRopesFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.ropeFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var agiReq = ctrl.postMassageMisc.agility() > 5;
        var toReturn = `[p]
                            These ropes don't look so tough.
                            You reckon you can cut them using your sharp claws.
                            Wriggling around your hand, you inch your restraints upwards ever so slightly.
                            Now if only you can reach your wrist with your claws, you'll be able to slice the rope and get out of here!
                        [/p]`;

        if (!agiReq) {
            toReturn += `${global.ui.condFail(`Agility below 6`)}
                        [p]
                            You try and you try, but not matter how hard you strain your wrist,
                            you fail to get your claws anywhere [b]near[/b] the rope!
                            If only your fingers were a little bit more nimble, you'd easily be able to reach!
                            After a good five minutes of trying, an intense pain builds up in your stressed joints.
                            You decide to cease your attempts, for now, lest you pop a strained muscle or something.
                        [/p]`;
        } else {
            toReturn += `${global.ui.condSucc(`Agility greater than 5`)}
                        [p]
                            You expertly curl your wrist,
                            angling it in such a way that the claw at the end of the longest of your four fingers can reach the rope, with half an inch to spare!
                            Perfect!
                            Your superior agility saves the day once more.
                            But you're not out of the woods yet.
                            You still need to [i]sever[/i] the rope.
                        [/p]
                        ${global.ui.condFail(`Strength below 6`)}
                        [p]
                            You frantically begin to grind your sharp claw against the cord,
                            but this kind of rope in particular seems a lot more [b]firm[/b] than the regular stuff.
                            Like the kind used at the riverside docks, to tie boats to the piers.
                            Your claw, while sharp, isn't nearly sharp enough to cut through the thick fibers of the strong rope.
                            Perhaps if you had a little bit more elbow grease to back up your fingerwork, you'd book more progress.
                        [/p]
                        [p]
                            After a good five minutes of trying, you decide to relent.
                            The rope's not even scuffed.
                            Looks like you'll have to find a different way to get out of here.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
