/**
 * End section of Varanar, Noon Three.
 * @param {VaranarSoupKitchenController} controller The Varanar, Noon Three controller.
 */
function VaranarSoupKitchenEatNoMention(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = VaranarSoupKitchenEatOfferChoice.DONOT;
    var reqController = ctrl.offer;
    var changeController = ctrl.noMention;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var male = ctrl.misc.baseGender() === GenderEnum.MALE;
        var toReturn = `[p]
                            You lie and say the previous owner [i]gave it[/i] to you because she felt [b]bad[/b] for keeping kobolds as slaves.
                            You were actually on your way there to [b]free[/b] your enslaved kin, but hunger stopped you dead in your tracks.
                            And so you ended up here.
                        [/p]
                        [p]
                            You can tell that the sheep isn't buying you bullshit.
                            But part of him [b]wants[/b] to believe.
                            He closes his eyes, and his inquisitive gaze turns into a grateful smile.
                            "[color=ppink][i]Kuhbold[/i], ah cannae thank ye enough.
                            A home in the Hamlet is perfect, since that's where ah get all mah food from to begin with.[/color]"
                            He puts the bag with his wedding ring in it around his neck once more.
                        [/p]
                        [p]
                            "[color=ppink]Mah waif's work can continue.
                            Maybe, just maybe, the people of the world can someday truly go without hunger.
                            And if that day ever comes, you'll know you helped make it happen, [i]kuhbold[/i].[/color]"
                            The caprine rises from his seat, signalling for you to get to your feet as well.
                            The two of you walk to the door.
                        [/p]
                        [p]
                            "[color=ppink]You've done me a real favor today, ${male ? `lad` : `lass`}.
                            If ye ever ever need something, [b]anything[/b] at all, ye know where me.
                            For now, I'm going to go pack me things.
                            I'll take care of yer friends real good, don't you worry![/color]"
                            The two of you say goodbye.
                            Your chubby host opens the door for you,
                            and you stride back out into the coldness of the streets of Varanar, your heart heavy with mixed feelings.
                        [/p]
                        [p]
                            On one end, it feels good knowing that someone will look after those [i]enslaved[/i] kobolds Cassidy was keeping.
                            On the other end of the stick, you [i]kind of[/i] failed to tell him about the [b]dead body[/b] in the barn.
                            Who knows what the sheep'll do when he finds out you've handed him a dead person's home?
                            Come to think of it, you didn't even catch his name.
                            Oh well, you're certain things'll work out for the best.
                        [/p]
                        [p]
                            Your conscience somewhat cleared, your spirits lifted and your ego slightly boosted,
                            you set off for Varanar's grand evenue.
                            Last night, all the stores were closed,
                            but most of them should be open around now.
                            Time to see some sights!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}
