

// SCENE FarmerHouse

// scenes\HamletDay2\FarmerHouse\confront\intimidate\FarmerHouseIntimidate.js

/**
 * Confront section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseIntimidate(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toSuccess = new FarmerHouseScared(ctrl);
    var toFail = new FarmerHouseIntimidateFail(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}


    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.INTIMIDATE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (ctrl.misc.charisma() < 6) {
            part += toFail.build();
        } else {
            part += toSuccess.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.INTIMIDATE) {
            return;
        }

        if (ctrl.misc.charisma() < 6) {
            toFail.registerListeners();
        } else {
            toSuccess.registerListeners();
        }
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\intimidate\FarmerHouseIntimidate.js

// scenes\HamletDay2\FarmerHouse\confront\intimidate\FarmerHouseIntimidateFail.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseIntimidateFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You rise to your tip-toes to seem as large and imposing as you possibly can,
                            before loudly and threateningly [b]shouting[/b] at the woman to [b]stay put[/b] and to [b]keep quiet[/b].
                            Despite your best efforts, you fail to appear very menacing.
                            The human seems unimpressed.
                            She reaches behind her, grabbing a rather sharp-looking ${global.kobold.inventory.createItemWithTooltip(new KitchenKnife)} from a wooden block.
                            [i]Oh no.[/i]
                        [/p]
                        ${global.ui.condFail(`Charisma below 6`)}
                        [p]
                            The housewife menacingly advances upon you, knife in hand.
                            Every step she takes towards you, you take one backwards in fear.
                            Before long, you've been pushed back into the hallway you came from.
                            Unarmed, you could have easily taken her on, but with a knife [i]that big[/i],
                            she could [b]seriously[/b] hurt you, if not [b]kill you[/b] outright.
                        [/p]
                        [p]
                            Your fight or flight instincts kick in, and fighting a crazy lady armed with a sharp cutting tool is [b]not[/b] on your mind.
                            You spot the perfect opportunity to flee when the woman nearly trips over a crooked, protruding floorboard.
                            While she's busy recovering her balance, you turn tail and [b]bolt[/b] towards the rear exit.
                            As you open the door, the enraged human hurls her knife towards you,
                            the steel blade missing you by a hair, lodging itself into the wooden doorframe.
                        [/p]
                        [p]
                            For a moment, you consider retrieving the weapon and turning the tables on the housewife,
                            but the blade is stuck fast, and the woman lets out a shriek for help.
                            You decide to keep running, sprinting into the adjacent potato field.
                            A bunch of farmers emerge from their homes, rushing to the aid of the damsel in distress.
                            Luckily, by the time she's pointed out to them that you're escaping through the acreage,
                            you've already vanished into the small stretch of woodland on the opposite end.
                        [/p]
                        [p]
                            Despite having ditched your pursuers, you continue running all the way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least here you can [b]pretend[/b] to be a traveller instead of a [b]miserable failure[/b] of a thief.
                            Exhausted, you brace yourself against a nearby tree.
                        [/p]
                        [p]
                            You're glad you managed to get out of that one alive.
                            If you were a bit more intimidating and [b]persuasive[/b], perhaps your threatening appearance would have been taken a bit more seriously.
                            Alas, what's done is done.
                            You can't win [i]every[/i] confrontation you get into, and at least you didn't get [b]caught[/b] or [b]cut[/b].
                            You've gained nothing, but you haven't lost much either, besides an opportunity.
                            Your newfound fear of kitchen knives is sure to vanish in due time.
                        [/p]
                        [p]
                            Your brief period of respite over, you continue on the way to Varanar.
                            Hopefully, the humans there will be less [i]ballsy[/i].
                            That's what the hamlet's inhabitants constantly claim, anyway.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.kobold.history.shared.farmerhouse.failed = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.intimidateFail();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\intimidate\FarmerHouseIntimidateFail.js

// scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHouseFullLoot.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseFullLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseLootContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the woman out of the picture, your way into the rest of the house is cleared.
                            It'd be a shame to pass up an opportunity like this just because you can't control your libido.
                            You came here to steal stuff, and that's exactly what you are going to do.
                            Better make it fast, though.
                            While you tied her up good, she may still escape if given plenty of time.
                        [/p]
                        [p]
                            You step into the dining room.
                            Nothing worth a damn in here, at first glance.
                            You keep moving, entering the living room, or at least what's supposed to pass for it.
                            The room is dominated by a single padded chair, the most comfortable piece of furniture you've come across in the house, so far.
                            It stands before a small fireplace, probably the only source of [i]warmth[/i] apart from the oven in the kitchen.
                            Still, nothing interesting to take.
                        [/p]
                        [p]
                            There's only one room left, which you assume to be the bedroom.
                            Opening the shabby wooden door, your suspicions are confirmed.
                            Two beds.
                            A big one, and a small one.
                            A dresser, and a nightstand.
                            You decide to go for the latter.
                        [/p]
                        [p]
                            Opening the small drawer, you are surprised to find that it [b]isn't[/b] full of underwear.
                            Instead, the nightstand contains a pair of worn books,
                            atop which lay some out-of-place looking ${global.kobold.inventory.createItemWithTooltip(new FancyGlasses())}.
                            Your eyes open wide, avarice surging to a whole new level.
                            This find more than makes up for having to deal with the housewife.
                            If you knew these farmers had something [b]this[/b] expensive laying around,
                            you wouldn't have stopped attempting to steal from them after your first attempt.
                        [/p]
                        [p]
                            You figure they must be a family heirloom or something.
                            Regardless, they're [b]yours[/b] now.
                            You pocket the valuable item, and start thinking of a way out of this place.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.fullLoot.continued() ? "disabled" : "";
        var isSelected = ctrl.fullLoot.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.fullLoot.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.tied.get() !== FarmerHouseTiedUpChoice.LOOT) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.fullLoot.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.tied.get() !== FarmerHouseTiedUpChoice.LOOT) {
            return;
        }

        next.registerListeners();

        if (ctrl.fullLoot.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHouseFullLoot.js

// scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHouseLootContinue.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseLootContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    function buildFullLoot() {
        var toReturn = `[p]
                            You exit the bedroom, returning to the living room.
                            Just going out the front door would be risky.
                            [b]Too[/b] risky.
                            You make your way back through the dining room into the kitchen.
                            The woman has managed to free her arms, and she is currently in the middle of untying herself.
                            Looks like your time here is running out, and if you don't escape before she gets the rope out of her mouth,
                            you'll be [b]hunted down[/b] by whoever comes to her aid.
                            You bolt past her into the adjacent hallway, and exit the building the same way you entered it: quietly, and through the back.
                        [/p]
                        [p]
                            The moment you close the door behind you, a cry for help resonates from within the building.
                            You weren't planning on sticking around anyway.
                            You sprint into the adjacent potato field, and after crossing it with some difficulty,
                            you escape into a small patch of woodland with nobody but the housewife having seen you.
                        [/p]
                        [p]
                            Still, just because you [i]may[/i] not be getting chased, doesn't mean you don't expeditiously make your way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least while on the road you can [b]pretend[/b] to be a traveller instead of a [b]housewife-tying bandit[/b].
                            You're glad you didn't leave the farmer's house empty-handed.
                            Thank goodness you invested some of your coin in a coil of rope.
                        [/p]
                        [p]
                            Reaching the road, you continue on your way to Varanar.
                            Hopefully, your streak of good luck will continue in the city.
                            And maybe you won't even have to tie anyone up, the next time you attempt to break in somewhere.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    function buildPartialLoot() {
        var toReturn = `[p]
                            The moment you close the door behind you, a cry for help resonates from within the building.
                            You weren't planning on sticking around anyway.
                            You sprint into the adjacent potato field, and after crossing it with some difficulty,
                            you escape into a small patch of woodland with nobody but the housewife having seen you.
                        [/p]
                        [p]
                            Still, just because you [i]may[/i] not be getting chased, doesn't mean you don't expeditiously make your way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least while on the road you can [b]pretend[/b] to be a traveller instead of a [b]housewife-threatening bandit[/b].
                            You're glad you didn't leave the farmer's house empty-handed,
                            even if the loot you managed to score isn't as great as you'd hoped it would be.
                            You'd rather have some food than nothing at all.
                        [/p]
                        [p]
                            Reaching the road, you continue on your way to Varanar.
                            Hopefully, there will be some better pickings in the city.
                            And maybe you won't even have to intimidate anyone, the next time you attempt to break in somewhere.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        if (ctrl.tied.get() === FarmerHouseTiedUpChoice.LOOT) {
            return buildFullLoot();
        } else {
            return buildPartialLoot();
        }
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.partialLoot.continued() && !ctrl.fullLoot.continued()) {
            return "";
        }

        global.kobold.history.shared.farmerhouse.stole = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.partialLoot.continued() && !ctrl.fullLoot.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHouseLootContinue.js

// scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHousePartialLoot.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHousePartialLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseLootContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the woman in check, you decide it's time to do what you came here to do: loot whatever you can.
                            Reminding her once more to [b]stay down[/b], you step towards the kitchen counter.
                            Keeping an eye on the housewife, you begin to feel around for things to steal.
                            Sadly, the countertop contains pretty much [b]nothing[/b] of value.
                            The only thing your hand comes across is the ${global.kobold.inventory.createItemWithTooltip(new Bread())},
                            the ${global.kobold.inventory.createItemWithTooltip(new Butter())}
                            and the ${global.kobold.inventory.createItemWithTooltip(new Ham())}
                            the woman had brought with her.
                        [/p]
                        [p]
                            You can tell that the housewife, while still pacified for the moment being, is just [b]waiting[/b] for you to slip up,
                            so she can run, scream for help or perhaps even [b]attack[/b] you.
                            You don't dare go into the next room to see if the pickings there are better.
                            Not wanting to leave here empty-handed either, you decide to take the peasant family's food.
                        [/p]
                        [p]
                            You pocket the bread, the butter and the ham.
                            Frustrated that you weren't able to steal more, but afraid of what the woman might do, should you leave her out of your sight,
                            you slowly back away into the hallway from which you came.
                            After casting one last menacing glare her way, you face away from the housewife, and [b]bolt[/b] out of the backdoor as fast as you can.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.partialLoot.continued() ? "disabled" : "";
        var isSelected = ctrl.partialLoot.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.partialLoot.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.LOOT) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.partialLoot.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.LOOT) {
            return;
        }

        next.registerListeners();

        if (ctrl.partialLoot.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\loot\FarmerHousePartialLoot.js

// scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseSexContinue.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseSexContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var dick = ctrl.misc.dick();
        var pants = ctrl.misc.pants();
        var underwear = ctrl.misc.underwear();
        var toReturn = `[p]
                            ${dick ? `You give yourself a few final strokes,
                            dribbling a last few bits of spunk down on top of the woman.` : `You wipe your loin with the flat of your hand,
                            sprinkling a last bit of feminine arousal atop the woman.`}`;

        if (pants || underwear) {
            toReturn += ` You quickly redress, your garments soaking up the remnants of your excitement.`;
        }

        toReturn += `       With your lust sated, you decide to leave before the unconscious human wakes up.
                        [/p]
                        [p]
                            You return to the hallway from whence you came, and you slip out the back door, closing it behind you.
                            Assuming the housewife could be waking up any time now, you jog into the adjacent potato field, and after crossing it with some difficulty,
                            you escape into a small patch of woodland with nobody having seen you.
                        [/p]
                        [p]
                            Still, just because you [i]may[/i] not be getting chased, doesn't mean you don't expeditiously make your way back to the eastern Varanar road.
                            The villagers might call the guards to investigate when they find one of their own having been abused, and should anyone ask,
                            at least while on the road you can [b]pretend[/b] to be a traveller instead of a [b]rapist[/b].
                            You're glad you got to show that human who's boss.
                            You really needed something to lift your spirit, and turning the tables on society itself did just that.
                        [/p]
                        [p]
                            Reaching the road, you continue on your way to Varanar.
                            Hopefully, the city will be filled with equally meek human women for you to abuse.
                        [/p]`;

        if (!dick && ctrl.wrestle.get() === FarmerHouseWrestleSuccessChoice.RAPE) {
            toReturn = `[p]
                            Arousal fading and frustration rising in its stead, you pull yourself to your feet.
                            ${pants || underwear ? "You quickly redress and let out a loud sigh." : "You let out a loud sigh."}
                            You consider kicking the unconscious woman,
                            but waking her up and having her scream together the entire neighbourhood wouldn't really fix anything.
                            All she had to do was stay awake while you abused her butt, and she couldn't even do that.
                            Stupid human.
                        [/p]
                        [p]
                            Defeated and put off, you return to the hallway from whence you came.
                            You slip out the back door, closing it behind you.
                            Assuming the housewife could be waking up any time, you jog into the adjacent potato field, and after crossing it with some difficulty,
                            you escape into a small patch of woodland with nobody having seen you.
                        [/p]
                        [p]
                            Still, just because you [i]may[/i] not be getting chased, doesn't mean you don't expeditiously make your way back to the eastern Varanar road.
                            The villagers might call the guards to investigate when they find one of their own having been abused, and should anyone ask,
                            at least while on the road you can [b]pretend[/b] to be a traveller instead of a [b]butt-fingering molester[/b].
                            You're glad you [i]somewhat[/i] got to show that human who's boss,
                            but quite upset that she fainted before you had a chance to really [b]enjoy[/b] yourself.
                        [/p]
                        [p]
                            Reaching the road, you continue on your way to Varanar.
                            Hopefully, the city will have some human women that can take a few fingers up the ass without passing out.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.rape.continued()) {
            return "";
        }

        global.kobold.history.shared.farmerhouse.raped = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        //part += ctrl.runChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.rape.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseSexContinue.js

// scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseForcedOral.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseForcedOral(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseSexContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var dick = ctrl.misc.dick();
        var pants = ctrl.misc.pants();
        var underwear = ctrl.misc.underwear();
        var toReturn = `[p]
                            The feeling of being in command inspires a sadistic urge inside of you.
                            A certain [b]lust[/b] is growing, enhanced by the sight of a woman on her knees; a human nonetheless.
                            ${dick ? `You can feel your member stirring within your cockslit, its tip already protruding.
                                    The very [b]thought[/b] of abusing the housewife is causing your loins to swell.
                                    ` : `You can feel your loins moisten at the very [b]thought[/b] of abusing the housewife.`}
                            For a brief moment, you remember the [i]actual[/i] reason you came here,
                            but a few meagre scraps of loot pale in comparison to the amount of [b]fun[/b] you can have with the woman.
                        [/p]
                        [p]
                            A wicked smile creeps across your face as you ogle the housewife.
                            Her linen shirt, while well-stitched, is not nearly thick enough to hide the distinct, flesh-like color of her bare body underneath.
                            Her sizable breasts, obviously unsupported, are large enough to slightly stretch the fabric of her somewhat undersized shirt, pulling it taut.
                            You fantasize about her topless, sewing together a new shirt,
                            ending up making it a [i]little bit[/i] too small because she was running out of yarn.
                            Or maybe her husband just likes her dressing like a [b]cheap whore[/b].
                        [/p]
                        [p]
                            You tell the woman once again to [b]stay down[/b], reminding her of what you'll do to her should she move.
                            She seems fully pacified and compliant.
                            You advance upon her, `;

        if (pants && underwear) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(pants)} and your ${global.kobold.inventory.createItemWithTooltip(underwear)} mid-stride,
                         a horrified look forming on the woman's face as your clearly aroused loins are made bare to her.`;
        } else if (pants) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(pants)} mid-stride,
                         a horrified look forming on the woman's face as your clearly aroused loins are made bare to her.`;
        } else if (underwear) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(underwear)} mid-stride,
                         a horrified look forming on the woman's face as your clearly aroused loins are made bare to her.`;
        } else {
            toReturn += `a horrified look forming on the woman's face as she becomes aware of your clearly aroused loins.`;
        }

        toReturn += `       You can feel her hesitance and her urge to [b]resist[/b] rise, yet she's still far too intimidated by your approaching presence to react.
                        [/p]
                        [p]
                            After a few slow steps, you find yourself standing directly in front of the woman, her face mere inches away from your exposed groin.
                            You can nearly [b]feel[/b] her warm breath on your ${dick ? "semi-erect length" : "needy slit"}.
                            She tries to turn away to the side, yet you grab her head with both your hands and firmly keep it in place,
                            your sharp claws further incentivizing her compliance.
                        [/p]`;

        if (dick) {
            toReturn += `[p]
                            A small growl escapes your lips.
                            You move one of your hands to clutch the housewife's dark brown hair,
                            freeing up the other to grip your growing member with.
                            You give yourself a few much-needed strokes, coaxing the rest of your length from your slit,
                            the tip of your member ending up directly beneath the human's nose, pressing against her top lip.
                        [/p]
                        [p]
                            The housewife knows what's coming, yet still she tries to delay the inevitable.
                            You demand she open her mouth, but she stubbornly keeps her lips sealed.
                            Aggravated, you cruelly [b]twist[/b] the peasant's hair, her mouth opening to voice her anguish,
                            only for you to [b]ram[/b] your member past her lips, silencing her pained grunt.
                        [/p]
                        [p]
                            In one smooth rolling motion of your hips, you slide the entirety of your length into the woman's mouth.
                            Fully bottomed out, her face buried in your scaly groin,
                            you take a moment to savor the feeling of your shaft being enveloped by the warm wetness of a human's mouth.
                            You warn her that if she [b]bites[/b],
                            you'll [b]wring her neck[/b] and the last thing she'll experience before she [b]dies[/b] will be the taste of kobold cock mixed with blood.
                            She glares up at you, sheer [b]hatred[/b] clear in her eyes.
                            Still, you get the feeling that she got the message.
                        [/p]
                        [p]
                            Making sure her head isn't going anywhere, you slowly start to rock back and forth, thrusting in and out of the woman's mouth.
                            While her lips are simply [i]resting[/i] around your shaft, instead of actively suckling,
                            the feeling of them sliding up and down your length excites you to no end.
                            Her tongue remains idle, yet you thrust at just the right angle to frot your cock along its entirety,
                            [b]forcing[/b] her to taste every inch of your manhood
                        [/p]
                        [p]
                            The human's bountiful saliva slickens your member, and in turn her lips, allowing you to gradually increase your pace.
                            It isn't long until you're outright [b]fucking[/b] her face, deftly pumping your cock in and out of her mouth,
                            your spit-covered abdomen smacking against her face every time you bottom out, a few thick,
                            slobbery strings connecting the human's skin to your scales.
                        [/p]
                        [p]
                            While the facefucking itself is pleasurable,
                            the thing that [b]sates[/b] your lust the most is the fact that the person sucking your dick is a [b]human[/b].
                            Just [b]thinking[/b] about how [i]wrong[/i] this situation is excites you to the point of barely being able to contain yourself.
                            You wildly [b]hump[/b] away at the housewife's face.
                            Various demeaning and degrading insults escape your lips in between heated grunts and a constant, low-pitched growling.
                        [/p]
                        [p]
                            Finally, you can take no more.
                            You grab the woman's hair with both of your hands, and forcibly [b]pull[/b] her onto your length as you reach your end,
                            squirting rope after rope of thick, heady spunk onto the far-end of her tongue, some of your cum shooting directly down her throat.
                            The woman struggles against your hurtful grasp, but you hold her down for the entire duration of your orgasm,
                            wasting not a single drop of potent kobold semen on her face.
                        [/p]
                        [p]
                            Your climax fading, you slowly remove yourself from the housewife's mouth.
                            It seems that while you were lost in pleasure, her struggles must have ceased somewhere along the road.
                            In fact, as you let go of your grip on her hair, the woman falls limply to the floor.
                            Fainted.
                            This entire ordeal must have been a little too much for her to process.
                            Heh.
                            Human can't handle kobold cock.
                        [/p]`;
            if(ctrl.misc.dickVirgin()) {
                toReturn += `[p][c][b]You are no longer a virgin.[/b][/c][/p]`;
            }
        } else {
            toReturn += `[p]
                            A small growl escapes your lips.
                            You move one of your hands to clutch the housewife's dark brown hair,
                            while grabbing the back of her head with the other, roughly pulling her face in between your thighs.
                            You let out a small, pleasured grunt as you can feel the human's nose press in between your folds.
                        [/p]
                        [p]
                            The housewife knows what's coming, yet still she tries to delay the inevitable.
                            You tell her to start [b]licking[/b], but she stubbornly keep her mouth closed.
                            Aggravated, you cruelly [b]twist[/b] the peasant's hair, her mouth opening to voice her anguish,
                            only for you to start [b]grinding[/b] your groin against her parted lips.
                        [/p]
                        [p]
                            The woman glares up at you, with tears in her eyes.
                            You cease your movements for a moment, to gauge her attitude.
                            Realizing resistance is futile, she slowly extends her tongue, and plants a single, tentative lick across the entire length of your slit.
                            The feeling of the warm, wet, slimy appendage pressing against your folds fuels your desire.
                        [/p]
                        [p]
                            You [b]demand[/b] she keep going, and the fearful woman complies to your whims.
                            She begins to lap at your cunt, shallowly scraping her tongue along your slit.
                            Letting out a soft moan, you slightly part your thighs, giving her better access to your loins.
                            Simultaneously, you force the human's head [b]deeper[/b] in between your legs, telling her to go [b]deeper[/b] with a single word.
                        [/p]
                        [p]
                            Once again, she complies, licking you with a bit more [b]conviction[/b],
                            her tongue now [b]parting[/b] your folds instead of simply rubbing against your cunt,
                            collecting more than a little of your arousal in the process.
                            It's not just the woman's tongue that's [b]sating[/b] your [b]need[/b].
                            The sheer fact that a [b]human[/b] is eating you out spurs your libido far more than her lapping ever could.
                        [/p]
                        [p]
                            You recommence your rocking back and forth, outright [b]humping[/b] the housewife's face,
                            slathering her pristine skin with feminine excitement.
                            You tighten your grip on her head, feeling your end rapidly approaching.
                            As her tongue accidentally slips inside of you, probably due to a miscalculation on her end, you are sent over the edge.
                            You let out a loud, satisfied sigh as you lower your entire body onto the human's face,
                            practically [b]sitting[/b] atop it as arousal [b]gushes[/b] from your slit, completely [b]soaking[/b] the woman's visage.
                        [/p]
                        [p]
                            The housewife struggles for a few moments, but you keep her pinned in place using both your scaly thighs and your hands.
                            As your orgasm fades, so too do her protests.
                            You let go of her face, and the woman instantly drops to the ground.
                            Fainted.
                            This entire ordeal must have been a little too much for her to process.
                            Heh.
                            Guess these rural humans aren't as tough as they claim to be.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.rape.continued() ? "disabled" : "";
        var isSelected = ctrl.rape.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rape.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.ORAL) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.rape.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.ORAL) {
            return;
        }

        next.registerListeners();

        if (ctrl.rape.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseForcedOral.js

// scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseTiedUpRape.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseTiedUpRape(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseSexContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var dick = ctrl.misc.dick();
        var pants = ctrl.misc.pants();
        var underwear = ctrl.misc.underwear();

        var toReturn = `[p]
                            For the first time in your life, you have a human completely at your mercy.
                            Their kind has pushed you around for as long as you can remember.
                            They treat you like garbage.
                            Like complete and utter [b]trash[/b].
                            For once, you are not subservient to humans, but domineering one of them.
                            And you intend to take full advantage of this opportunity.
                        [/p]
                        [p]
                            Lewd thoughts had crept into your mind from the moment you first laid eyes upon the housewife.
                            Her long, brown hair, her curvaceous frame,
                            her fair face and her considerable rack have been [b]taunting[/b] you ever since she entered the kitchen.
                            Watching the woman struggle and squirm in a feeble attempt to free herself from her bondage only serves to fuel your desire.
                            A wicked grin creeps onto your face.
                            You lustily lick your lips, `;

        if (pants && underwear) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(pants)} and your ${global.kobold.inventory.createItemWithTooltip(underwear)}, `;
        } else if (pants) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(pants)}, `;
        } else if (underwear) {
            toReturn += `lowering your ${global.kobold.inventory.createItemWithTooltip(underwear)}, `;
        }

        toReturn += `       your ${dick ? `fully erect cock` : `moist loins`} a clear indicator of your arousal.
                            This is going to be [b]fun[/b].
                        [/p]`;

        if (dick) {
            toReturn += `[p]
                            You kneel down next to the human, and crudely [b]grope[/b] her ass with one of your hands.
                            She freezes up for a moment, then begins to struggle, kick and scream as best as she can through the rope covering her mouth.
                            Luckily, even despite her frantic wriggling, your bindings hold.
                            She isn't going anywhere, which means you can take your sweet time feeling up her ass through her shabby leather pants.
                        [/p]
                        [p]
                            You lecherously squeeze the woman's soft, garment-clad posterior.
                            While she clearly hasn't had the easiest of lives so far,
                            her butt feels like it's been sitting idly on a comfortable throne for the most part of it.
                            Soft and squishy, truly an ass fit for an expensive [b]whore[/b] instead of a run-down housewife.
                            Your free hand finds its way to your groin, and you begin playing with yourself,
                            fantasizing about fucking her, your pelvis [b]smacking[/b] against her rear, making it [b]jiggle[/b] and [b]bounce[/b].
                        [/p]
                        [p]
                            Every touch and every thought entices you more and more.
                            You didn't tie her up [i]just[/i] to get a feel, though.
                            You [b]want[/b], no, you [b]need[/b] more.
                            Gripping the waistband of her pants, you tug them down as far as the rope will allow,
                            exposing the entirety of her behind, the waistband ending up nestling snugly just below her buttcheeks,
                            giving you an excellent rear-view of her loins.
                            Underwear must be too expensive for this family, for she is wearing none.
                            What a [b]slut[/b].
                        [/p]
                        [p]
                            You get down on the floor next to the woman, guiding your rock-hard member in between her legs,
                            pressing your tapered length into the small gap separating her thick thighs.
                            While you can't quite [i]fuck her[/i] with her legs bound this tightly together,
                            the warm, soft-skinned flesh of her thighs enveloping your manhood is [b]more[/b] than enough stimulation for you.
                            Gripping the housewife's waist to steady yourself, you slowly begin to rock back and forth,
                            grinding your cock along the length of her slit, parting her folds with the the top-side of your dick,
                            while her bare upper legs squeeze down on your shaft, milking it far better than your own hand ever could.
                        [/p]
                        [p]
                            Feeling your cock in between her thighs, the woman begins to squirm anew, desperate to free herself and throw you off of her.
                            Little does she know that the small motions of her legs only serve to [b]massage[/b] your shaft,
                            pleasuring you further instead of [b]dissuading[/b] you from toying with her body.
                            A steady stream of precum oozing from the tapered tip of your member serves to smoothen the peasant's already pristine flesh,
                            allowing you to pick up the pace, turning your slow grinding into [i]actual[/i] thighfucking,
                            the fantasies of your hips smacking against her ass becoming reality.
                        [/p]
                        [p]
                            Realizing she can't shake you off, the housewife ceases her protests,
                            resorting to laying perfectly still as you [b]use[/b] her body to get yourself off,
                            hoping her lack of resistance would speed things along.
                            Her assumption proves to be far from false.
                            Without the constant wriggling, you can up your pace one final time,
                            the loud slapping of scales against skin echoing through the decrepit house as you hump away to your heart's delight.
                        [/p]
                        [p]
                            You can feel your end rapidly approaching,
                            stimulated not only by the thighs squeezing around your length and the warm cunt your member rubs against with every thrust,
                            but also by the very situation itself.
                            You are fucking the thighs of a [b]human[/b], a human that isn't even [b]struggling[/b] anymore.
                            Like she realizes that [b]you[/b] are in command here.
                            Her unwillingness turning into emotionless reluctance excites you to no end,
                            stroking the sadistic side of your personality in [b]all[/b] the right ways.
                        [/p]
                        [p]
                            You reach up to grab one of the woman's clothed boobs, kneading it tightly as you reach your end,
                            your frantic fucking slowing down considerably, ending up in a series of firm,
                            full-length thrusts to [b]squeeze[/b] every bit of cum out of your internal balls.
                            The tip of your member protruding from the front-end of the housewife's groin,
                            you squirt rope after rope of thick, heady cum onto the wooden floor, a few of which get caught on the waistband of her pants.
                        [/p]
                        [p]
                            Your intense orgasm coming to an end,
                            you make sure to wipe a bit of oozing spunk off directly onto the human's cunt as you retract your length from in between her thighs.
                            It is only when you rise to your feet, that you notice the woman isn't even [i]conscious[/i] anymore.
                            She must have passed while you were having your way with her.
                            Fainted from emotional distress.
                            Guess these rural humans aren't as tough as they claim to be.
                        [/p]`;
        } else {
            toReturn += `[p]
                            You kneel down next to the human, and crudely [b]grope[/b] her ass with one of your hands.
                            She freezes up for a moment, then begins to struggle, kick and scream as best as she can through the rope covering her mouth.
                            Luckily, even despite her frantic wriggling, your bindings hold.
                            She isn't going anywhere, which means you can take your sweet time feeling up her ass through her shabby leather pants.
                        [/p]
                        [p]
                            You lecherously squeeze the woman's soft, garment-clad posterior.
                            While she clearly hasn't had the easiest of lives so far,
                            her butt feels like it's been sitting idly on a comfortable throne for the most part of it.
                            Soft and squishy, truly an ass fit for an expensive [b]whore[/b] instead of a run-down housewife.
                            Your free hand finds its way to your groin, and you begin playing with yourself,
                            fantasizing about pressing your face in between her near-perfect buttcheeks,
                            and lapping at her twat until she cums all over your face.
                        [/p]
                        [p]
                            Every touch and every thought entices you more and more.
                            You didn't tie her up [i]just[/i] to get a feel, though.
                            You [b]want[/b], no, you [b]need[/b] more.
                            Gripping the waistband of her pants, you tug them down as far as the rope will allow,
                            exposing the entirety of her behind, the waistband ending up nestling snugly just below her buttcheeks,
                            giving you an excellent rear-view of her loins.
                            Underwear must be too expensive for this family, for she is wearing none.
                            What a [b]slut[/b].
                        [/p]
                        [p]
                            More importantly, lowering her pants has made bare a large stretch of rope-less leg,
                            just big enough for you to...[i]maybe[/i]...
                        [/p]
                        [p]
                            You step over the human, one leg on either side of her frame, before squatting down,
                            pressing your sopping wet slit against the side of her leg.
                            Using one hand to keep the woman pinned down, and the other to stabilize yourself with,
                            you slowly begin to rub your groin alongside the smooth surface of her skin.
                            The sensation is surprisingly pleasurable, more so than you had anticipated.
                            This was a good idea.
                        [/p]
                        [p]
                            Your absolutely [b]drenched[/b] cunt leaves a trail of arousal in its wake,
                            slickening the human's skin, allowing you to up your pace.
                            You begin to roll your hips back and forth, grinding against the side of her leg,
                            innate warmth enhanced by sheer [b]friction[/b] heating up your feminine excitement.
                        [/p]
                        [p]
                            Much more than the simple act of humping against the housewife's leg,
                            the feeling of having a [b]human[/b] underneath you spurs your libido.
                            To make things even better, she's not even [b]struggling[/b] against your hold.
                            She could very easily knock you off of her, if she tried, although that would only end bad for her.
                            Still, being in control of someone instead of being the meek person [b]being[/b] controlled feels [b]oh so[/b] good, for once.
                            It strokes the sadistic side of your personality in [b]all[/b] the right ways.
                        [/p]
                        [p]
                            Your soft moans and pleasured grunts echo throughout the decrepit house.
                            You lean forwards, pressing against the side of the woman's body to make your movements a bit more natural,
                            allowing you to [b]ride[/b] her skin like a professional kobold whore would a cock.
                            You grip one of her clothed tits, kneading it to your heart's content, toying with it without so much as an [b]objection[/b] from the housewife.
                        [/p]
                        [p]
                            Your breathing intensifies.
                            You can [b]feel[/b] your end rapidly approaching, your legs clutching tightly around hers.
                            You press your groin as tightly down onto the human's leg as you possibly can,
                            rubbing back and forth just a few more times before reaching your end.
                            You can [b]barely[/b] prevent yourself from [b]screaming[/b] in pleasure as feminine fluids [b]gush[/b] from your slit,
                            completely [b]soaking[/b] the woman's legs, a small puddle forming on the wooden floor below.
                        [/p]
                        [p]
                            Rising from the housewife's leg, you give yourself a few more strokes with your hand, basking in super-sensitive, post-orgasmic bliss.
                            With a loud, pleased sigh, you slowly rise to your feet.
                            It is only then, that you notice the woman isn't even [i]conscious[/i] anymore.
                            She must have passed while you were having your way with her.
                            Fainted from emotional distress.
                            So that's why she didn't struggle.
                            Guess these rural humans aren't as tough as they claim to be.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.rape.continued() ? "disabled" : "";
        var isSelected = ctrl.rape.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rape.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.tied.get() !== FarmerHouseTiedUpChoice.RAPE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.rape.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.tied.get() !== FarmerHouseTiedUpChoice.RAPE) {
            return;
        }

        next.registerListeners();

        if (ctrl.rape.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseTiedUpRape.js

// scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseWrestleRape.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestleRape(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseSexContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var dick = ctrl.misc.dick();
        var pants = ctrl.misc.pants();
        var underwear = ctrl.misc.underwear();
        
        var toReturn = `[p]
                            This [b]stupid human[/b] thought she could keep you from picking this place clean.
                            She's probably one of those types that thinks kobolds are [i]inferior[/i].
                            The kind of person that would [b]kill[/b] one of your kind without cause and without remorse, if given the tools to do so.
                            But now, the tables have turned.
                            [b]You[/b] aren't at [i]her[/i] mercy, but [b]she[/b] is at [i]yours[/i].
                            And you intend to make her feel every bit as [b]miserable[/b] as her species makes your kind feel [b]every single day[/b].
                        [/p]
                        [p]
                            While keeping the housewife firmly in place, you lean down, pressing your muzzle into the back of her head,
                            burying your snout into her long, brown hair.
                            You take a loud [b]sniff[/b], the scent of unprocessed [b]wheat[/b] and other farm produce filling your nostrils,
                            along with a distinctly [b]feminine[/b] scent buried underneath layers upon layers of a lifetime's worth of hard work.
                        [/p]
                        [p]
                            You brush her hair to the side with the tip of your nose.
                            With the woman's neck exposed, you extend your tongue, and plant a single long, wet lick across her smooth skin,
                            which leaves a trail of saliva in its wake.
                            The human tastes exactly like she smells: [i]rough around the edges[/i], but with a subtle, unmistaken core of femininity,
                            so subtle that species with less-well-defined tastebuds probably wouldn't even [b]notice[/b] it.
                        [/p]
                        [p]
                            In that way, your kind is blessed, just like most canines are.
                            You are more [i]cognitive[/i], able to pick up and separate distinct traces of scents and of tastes.
                            This is both a blessing and a curse, depending on [b]what[/b] you end up smelling or tasting.
                            Right now, it is a [b]godsend[/b], the subtle feminine fragrance and flavour of the woman spurring your libido to new heights.
                        [/p]
                        [p]
                            The housewife seems to be starting to understand what your intentions are.
                            Her struggling intensifies, to the point of nearly throwing you off her back.
                            Using your knee to keep both of her arms in position,
                            you use your now-freed hand to take hold of her hair.
                            Cruelly twisting it seems sufficient to pacify the woman, pain reminding that [b]you[/b] are in charge here.
                            Once her squirming stops, `;
        
        if (pants && underwear) {
            toReturn += `you take the time to lower your ${global.kobold.inventory.createItemWithTooltip(pants)} 
                        and your ${global.kobold.inventory.createItemWithTooltip(underwear)}, your fully aroused loins made bare.`;
        } else if (pants) {
            toReturn += `you take the time to lower your ${global.kobold.inventory.createItemWithTooltip(pants)}, your fully aroused loins made bare.`;
        } else if (underwear) {
            toReturn += `you take the time to lower your ${global.kobold.inventory.createItemWithTooltip(underwear)}, your fully aroused loins made bare.`;
        } else {
            toReturn += `you give your loins a few much-needed strokes.`;
        }
        
        toReturn += `     Time for the real [b]fun[/b] to begin.  
                        [/p]`;
        
        if(dick) {
            toReturn += `[p]
                            You shift around on the human's back, laying yourself down atop her, using your chest to keep her arms pinned.
                            Your groin aligns perfectly with her soft rear, your hard member pressing into her shabby leather pants from behind.
                            You remind the housewife that if she struggles, you'll be forced to [b]hurt[/b] her again.
                            Then, you forcefully [b]tug[/b] her pants down as far as they'll go, her ass fully exposed,
                            the tip of your cock directly in between her buttcheeks.
                            Slut isn't even wearing any underwear.
                            Like she [b]wanted[/b] this to happen all along.
                        [/p]
                        [p]
                            Gripping your member, you guide it deeper in between the human's buttocks, her warm flesh enveloping your length.
                            She writhes around like crazy, yet you manage to keep the lower half of her body somewhat steady,
                            simply by forcing it down against the floor with your all of your might, using your entire frame to keep her in place.
                            You press your tapered tip directly against her [i]other[/i] hole, the one place she's least likely to have ever been fucked.
                            She [b]begs[/b] you not to, her pleas muffled by your palm still covering her mouth.
                            Her miserable whining only serves to spur you on further.
                        [/p]
                        [p]
                            For being such a horrible, kobold-hating [b]human[/b], and for her constant, annoying [b]struggling[/b], she doesn't deserve your mercy.
                            You mercilessly press forwards, slowly forcing your rock-hard length into her tight behind,
                            powering inch after inch of ever-thickening kobold meat into the woman's ass.
                            Her writhing instantly comes to an end.
                            She [b]screams[/b] into your hand, before slumping down onto the ground, utterly [b]broken[/b] and [b]defeated[/b].
                            But her [b]punishment[/b] has only just begun.
                        [/p]
                        [p]
                            You manage to lodge over half of your member inside of her overly-tight hole in a single thrust.
                            You pull back, only to [b]ram[/b] yourself forwards once more, shoving another couple of inches inside.
                            A lusty grunt escapes your lips.
                            [i]Almost there.[/i]
                            A third time, you slip out everything but the tip.
                            With your free hand, you take hold of the woman's hips to steady yourself.
                            Then, you give her the most [b]fiercesome[/b] of thrusts so far, your scaly pelvis [b]smacking[/b] against her rear as you bottom out,
                            sinking the entirety of your length, up to the very base, into her far-too-tight behind.
                            [b]Fuck yes.[/b]
                        [/p]
                        [p]
                            The woman's insides [b]squeeze down[/b] on your shaft like a [b]vice[/b].
                            Still, you need more.
                            You begin to roll your hips back and forth, ever-so-slightly at first, giving her tender,
                            unlubed hole a few small thrusts while the precum oozing from your tip slickens her insides.
                            As the human's behind begins to loosen somewhat, you pick up both the pace and the ferocity of your motions,
                            fucking her ass [b]deeper[/b] and [b]harder[/b].
                            It isn't long until you settle into a steady rhythm, pumping your cock in and out of her,
                            your low growls and the sound of your hips smacking against her behind echoing across the room.
                        [/p]
                        [p]
                            The thing that turns you on the most about this situation isn't the actual [i]sex[/i].
                            Sure, fucking the woman's rear end is pleasurable, but if you really wanted to have a good time, you would have just fucked her cunt instead.
                            No, the thing that arouses you most, is the fact that you're having your way with a [b]human[/b].
                            You're making up for [b]years[/b] of your kind being [b]treaded on[/b], [b]used[/b] and [b]abused[/b].
                            You're punishing her for the misdeeds of her entire [b]species[/b].
                            For the wrongdoings of [b]all[/b] of the races that think they're so much [b]better[/b] than you.
                        [/p]
                        [p]
                            The feeling of finally [b]getting back[/b] at the humans for making your life [b]miserable[/b],
                            for making the lives of [b]all[/b] kobolds a living hell, [b]that[/b] is the thought that spurs your desire the most.
                            Combined with a hole so tight your member barely [b]fits[/b], you don't manage to last very long.
                            Mere [b]minutes[/b] after you first forced yourself into the housewife's ass, you can already feel your end approaching.
                        [/p]
                        [p]
                            Your fucking become more [b]bestial[/b], even more [b]violent[/b] than before.
                            You wildly [b]pound[/b] away at the woman's rear with a series of firm, stabbing thrusts, until finally you can hold yourself no longer.
                            One final time, you [b]slam[/b] yourself inside of her, hilting your member up to the base as you reach your climax,
                            squirting your hot, sticky load deep inside of her guts, filling her up with the very seed of the race she [b]despises[/b] so.
                        [/p]
                        [p]
                            You keep your member firmly in place for the entire duration of your orgasm.
                            Only once the multitude of individual waves of spunk fade into a continuous dribble of jizz,
                            do you decide to pull yourself out of the woman's rear.
                            Retracting your cock, the human's ass is left slightly agape.
                            Mere moments after you remove yourself from her behind, a trickle of spunk begins to ooze from it,
                            running over and alongside her cunt before spilling to the floor below.
                        [/p]
                        [p]
                            Breathing heavily, you pull your focus from the housewife's ass to her visage.
                            You wanted to see the look on her face, but alas, it seems she fainted somewhere between you inserting your member and you pulling out.
                            No longer needing to hold her down, you rise to your feet.
                            Stupid human can't even take a buttfucking without passing out.
                        [/p]`;
            if(ctrl.misc.dickVirgin()) {
                toReturn += `[p][c][b]You are no longer a virgin.[/b][/c][/p]`;
            }
        } else {
            toReturn += `[p]
                            You shift around on the human's back, laying yourself down atop her, using your chest to keep her arms pinned.
                            You remind the housewife that if she struggles, you'll be forced to [b]hurt[/b] her again.
                            Then, you forcefully [b]tug[/b] her pants down as far as they'll go, her ass fully exposed.
                            Slut isn't even wearing any underwear.
                            Like she [b]wanted[/b] this to happen all along.
                        [/p]
                        [p]
                            You press your free hand in between the woman's buttcheeks, your scaly fingers rubbing against her sensitive rear hole.
                            You intend to [b]punish[/b] her for all the bad things she's done to your kind.
                            You figure that toying with her cunt would be more of a [b]reward[/b].
                            She begs you not to go through with this, yet her voice is muffled by your palm still plastered atop her mouth.
                        [/p]
                        [p]
                            Without mercy, you deftly plunge [b]two[/b] of your fingers into her overly-tight rear,
                            stretching her ass with a pair of unlubed digits.
                            The woman [b]screams[/b] into your hand, before slumping down onto the ground, utterly [b]broken[/b] and [b]defeated[/b].
                            But her [b]torment[/b] has only just begun.
                            Without delay, you begin to pump your fingers in and out of her, thoroughly [b]fucking[/b] her obviously inexperienced butt.
                        [/p]
                        [p]
                            But you aren't done just yet.
                            With a cruel smirk on your face, you force a [b]third[/b] thick finger up her rear, inserting all but your thumb,
                            testing and [b]exceeding[/b] the housewife's limits without ever halting your incessant fingerfucking.
                            You'd hoped to get a rise out of the woman, yet looking up at her face, you notice that she's completely [b]passed out[/b].
                            That's kind of a [b]buzzkill[/b].
                            You retract your fingers from her rear.
                            There's no point to fingerfucking an unconscious human.
                        [/p]`;
        }
        
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.rape.continued() ? "disabled" : "";
        var isSelected = ctrl.rape.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rape.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.wrestle.get() !== FarmerHouseWrestleSuccessChoice.RAPE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.rape.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.wrestle.get() !== FarmerHouseWrestleSuccessChoice.RAPE) {
            return;
        }
        
        next.registerListeners();

        if (ctrl.rape.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\rape\FarmerHouseWrestleRape.js

// scenes\HamletDay2\FarmerHouse\confront\scared\FarmerHouseScared.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseScared(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var oralId = arguments.callee.name + "ButtonOne";
    var tieId = arguments.callee.name + "ButtonTwo";
    var lootId = arguments.callee.name + "ButtonThree";

    var toOral = new FarmerHouseForcedOral(ctrl);
    var toTie = new FarmerHouseTiedUp(ctrl);
    var toLoot = new FarmerHousePartialLoot(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    function buildIntimidate() {
        var toReturn = `[p]
                            You rise to your tip-toes to seem as large and imposing as you possibly can,
                            before loudly [b]shouting[/b] at the woman to [b]stay put[/b] and to [b]keep quiet[/b].
                            Your booming voice and your lizard-like appearance seem to overcome your lacking [i]size[/i],
                            making you seem far more threatening than you actually are.
                            The human looks scared out of her mind, a terrified expression on her face as she cowers in place.
                        [/p]
                        ${global.ui.condSucc(`Charisma greater than 5`)}
                        [p]
                            Your intimidation successful, you command the woman to get down on her knees.
                            A twisted form of sadistic pride makes your chest swell as she actually [b]follows[/b] your order.
                            For the first time in your life, a human is doing as [b]you[/b] say, instead of it being the other way around.
                            You can't say you [b]dislike[/b] the feeling of being in charge.
                            It mentally empowers you, makes you feel like you're on top of the world.
                            Is this how humans feel when they boss your kind around?
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    function buildThreaten() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You tell the woman to keep quiet and stay put.
                            You threaten to [b]hurt[/b] and even [b]kill[/b] her, should she so much as [b]dare[/b] to make a noise.
                            You brandish your ${global.kobold.inventory.createItemWithTooltip(weapon)} in an attempt to appear more dangerous, and it works.
                            The human seems scared out of her mind, a terrified expression on her face as she eyes your weapon.
                        [/p]
                        ${global.ui.condSucc(`${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence greater than 3`)}
                        [p]
                            Your threatening successful, you command the woman to get down on her knees.
                            A twisted form of sadistic pride makes your chest swell as she actually [b]follows[/b] your order.
                            For the first time in your life, a human is doing as [b]you[/b] say, instead of it being the other way around.
                            You can't say you [b]dislike[/b] the feeling of being in charge.
                            It mentally empowers you, makes you feel like you're on top of the world.
                            Is this how humans feel when they boss your kind around?
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        if (ctrl.confront.get() === FarmerHouseConfrontChoice.INTIMIDATE) {
            return buildIntimidate();
        } else {
            return buildThreaten();
        }
    }

    /**
     * Builds and returns the choice button.
     */
    function buildOralChoice() {
        var disabled = ctrl.scared.has() ? "disabled" : "";
        var isSelected = ctrl.scared.get() === FarmerHouseScaredChoice.ORAL;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${oralId}" ${disabled}>Fuck her face</button>`;

        var tooltip = global.parser.parse(`Seeing the woman drop to her knees has awoken some unsavory thoughts. Act on them.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildTieChoice() {
        var hasRope = ctrl.hasRope();
        var disabled = ctrl.scared.has() || !hasRope ? "disabled" : "";
        var isSelected = ctrl.scared.get() === FarmerHouseScaredChoice.TIE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${tieId}" ${disabled}>Tie her up</button>`;

        var tooltip = global.parser.parse(`Requires a rope.`);
        if (hasRope) {
            tooltip = global.parser.parse(`You got this rope for a reason: to [s]hang yourself[/s] tie people up with.`);
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildLootChoice() {
        var disabled = ctrl.scared.has() ? "disabled" : "";
        var isSelected = ctrl.scared.get() === FarmerHouseScaredChoice.LOOT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${lootId}" ${disabled}>Loot</button>`;

        var tooltip = global.parser.parse(`Loot as best as you can without taking your eyes off of the human.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildOralChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildTieChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildLootChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + oralId), $("#" + lootId), $("#" + tieId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerOralListener() {
        $("#" + oralId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.scared.set(FarmerHouseScaredChoice.ORAL);
            loadNextPart(toOral);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerTieListener() {
        $("#" + tieId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.scared.set(FarmerHouseScaredChoice.TIE);
            loadNextPart(toTie);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerLootListener() {
        $("#" + lootId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.scared.set(FarmerHouseScaredChoice.LOOT);
            loadNextPart(toLoot);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toOral.build();
        part += toTie.build();
        part += toLoot.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toOral.registerListeners();
        toTie.registerListeners();
        toLoot.registerListeners();

        if (ctrl.scared.has()) {
            return;
        }

        registerOralListener();
        registerTieListener();
        registerLootListener();
    };
}

var FarmerHouseScaredChoice = {
    ORAL: 1,
    TIE: 2,
    LOOT: 3
};

// END scenes\HamletDay2\FarmerHouse\confront\scared\FarmerHouseScared.js

// scenes\HamletDay2\FarmerHouse\confront\scared\FarmerHouseTiedUp.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseTiedUp(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var lootId = arguments.callee.name + "ButtonOne";
    var rapeId = arguments.callee.name + "ButtonTwo";

    var toLoot = new FarmerHouseFullLoot(ctrl);
    var toRape = new FarmerHouseTiedUpRape(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The woman may be pacified, for now, but you still don't trust her.
                            You can [i]sense[/i] that she's just [b]waiting[/b] for you to slip up,
                            for you to turn your back so she can [b]plunge a knife[/b] into it.
                            Luckily, you brought just the right tool to deal with these kinds of situations.
                        [/p]
                        [p]
                            You produce your rope.
                            After telling the human to [b]stay down[/b], you slowly advance upon her.
                            You crudely [b]grab[/b] both her arms, and loop your rope around her wrists.
                            Forcing the meek woman to the ground, you then firmly bind her ankles,
                            looping the cord around her legs multiple times.
                            As a finishing touch, you wind the rope upwards, and spin it around her head to cover her mouth,
                            preventing her from screaming for help while completely immobilizing the housewife.
                        [/p]
                        [p]
                            After tying a firm knot, you rise to your feet and admire your handiwork.
                            The human struggles fiercely against her restraints, wriggling her body back and forth in a vain attempt to escape the well-tied rope.
                            She tries pleading with you through her improvised gag,
                            but all that escapes her lips are a bunch of muffled, high-pitched tones
                            barely even resembling the ever more frustrated and desperate implorations she had in mind.
                        [/p]
                        [p]
                            It looks like you did an excellent job.
                            She can't move, she can't scream for help, and the house is all yours now.
                            But then again: she [b]can't move[/b] and she [b]can't scream for help[/b].
                            You could very easily do something [i]other[/i] than stealing things.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildLootChoice() {
        var disabled = ctrl.tied.has() ? "disabled" : "";
        var isSelected = ctrl.tied.get() === FarmerHouseTiedUpChoice.LOOT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${lootId}" ${disabled}>Loot</button>`;

        var tooltip = global.parser.parse(`With the human restrained, you can loot in peace.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 140px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRapeChoice() {
        var disabled = ctrl.tied.has() ? "disabled" : "";
        var isSelected = ctrl.tied.get() === FarmerHouseTiedUpChoice.RAPE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${rapeId}" ${disabled}>Fuck her</button>`;

        var tooltip = global.parser.parse(`Have your way with the woman. She's completely restrained, and thus unable to fight back.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildRapeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildLootChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + lootId), $("#" + rapeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerLootListener() {
        $("#" + lootId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.tied.set(FarmerHouseTiedUpChoice.LOOT);
            loadNextPart(toLoot);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerRapeListener() {
        $("#" + rapeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.tied.set(FarmerHouseTiedUpChoice.RAPE);
            loadNextPart(toRape);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.TIE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.removeRope();
        part += buildChoice();
        part += `</div>`;
        part += toLoot.build();
        part += toRape.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.TIE) {
            return;
        }

        toLoot.registerListeners();
        toRape.registerListeners();

        if (ctrl.tied.has()) {
            return;
        }

        registerLootListener();
        registerRapeListener();
    };
}

var FarmerHouseTiedUpChoice = {
    LOOT: 1,
    RAPE: 2
};

// END scenes\HamletDay2\FarmerHouse\confront\scared\FarmerHouseTiedUp.js

// scenes\HamletDay2\FarmerHouse\confront\threaten\FarmerHouseThreatenFail.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseThreatenFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You tell the woman to keep quiet and stay put.
                            You threaten to [b]hurt[/b] and even [b]kill[/b] her, should she so much as [b]dare[/b] to make a noise.
                            ${weapon ? `You brandish your ${global.kobold.inventory.createItemWithTooltip(weapon)} in an attempt to seem more dangerous,
                                        yet you seem to achieve the opposite effect.` : `Sadly, you don't have a weapon to back your threats up with.`}
                            The human seems unimpressed.
                            She reaches behind her, grabbing a rather sharp-looking ${global.kobold.inventory.createItemWithTooltip(new KitchenKnife)} from a wooden block.
                            [i]Oh no.[/i]
                        [/p]
                        ${global.ui.condFail(weapon ? `${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence below 4` : `No weapon`)}
                        [p]
                            The housewife menacingly advances upon you, knife in hand.
                            Every step she takes towards you, you take one backwards in fear.
                            Before long, you've been pushed back into the hallway you came from.
                            Unarmed, you could have easily taken her on, but with a knife [i]that big[/i],
                            she could [b]seriously[/b] hurt you, if not [b]kill you[/b] outright.
                        [/p]
                        [p]
                            Your fight or flight instincts kick in, and fighting a crazy lady armed with a sharp cutting tool is [b]not[/b] on your mind.
                            You spot the perfect opportunity to flee when the woman nearly trips over a crooked, protruding floorboard.
                            While she's busy recovering her balance, you turn tail and [b]bolt[/b] towards the rear exit.
                            As you open the door, the enraged human hurls her knife towards you,
                            the steel blade missing you by a hair, lodging itself into the wooden doorframe.
                        [/p]
                        [p]
                            For a moment, you consider retrieving the weapon and turning the tables on the housewife,
                            but the blade is stuck fast, and the woman lets out a shriek for help.
                            You decide to keep running, sprinting into the adjacent potato field.
                            A bunch of farmers emerge from their homes, rushing to the aid of the damsel in distress.
                            Luckily, by the time she's pointed out to them that you're escaping through the acreage,
                            you've already vanished into the small stretch of woodland on the opposite end.
                        [/p]
                        [p]
                            Despite having ditched your pursuers, you continue running all the way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least here you can [b]pretend[/b] to be a traveller instead of a [b]miserable failure[/b] of a thief.
                            Exhausted, you brace yourself against a nearby tree.
                        [/p]
                        [p]
                            You're glad you managed to get out of that one alive.
                            If you had ${weapon ? "a more intimidating weapon" : "some kind of weapon"}, perhaps your threats would have been taken a bit more seriously.
                            Alas, what's done is done.
                            You can't win [i]every[/i] confrontation you get into, and at least you didn't get [b]caught[/b] or [b]cut[/b].
                            You've gained nothing, but you haven't lost much either, besides an opportunity.
                            Your newfound fear of kitchen knives is sure to vanish in due time.
                        [/p]
                        [p]
                            Your brief period of respite over, you continue on the way to Varanar.
                            Hopefully, the humans there will be less [i]ballsy[/i].
                            That's what the hamlet's inhabitants constantly claim, anyway.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.kobold.history.shared.farmerhouse.failed = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.threatenFail();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\threaten\FarmerHouseThreatenFail.js

// scenes\HamletDay2\FarmerHouse\confront\threaten\FarmerHouseThreaten.js

/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHouseThreaten(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toSuccess = new FarmerHouseScared(ctrl);
    var toFail = new FarmerHouseThreatenFail(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}


    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.THREATEN) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (!ctrl.misc.weapon() || ctrl.misc.weapon().offence() < 4) {
            part += toFail.build();
        } else {
            part += toSuccess.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.THREATEN) {
            return;
        }

        if (!ctrl.misc.weapon() || ctrl.misc.weapon().offence() < 4) {
            toFail.registerListeners();
        } else {
            toSuccess.registerListeners();
        }
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\threaten\FarmerHouseThreaten.js

// scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleSuccess.js

/**
 * Arrival section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestleSuccess(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var chokeId = arguments.callee.name + "ButtonOne";
    var rapeId = arguments.callee.name + "ButtonTwo";

    var toChoke = new FarmerHouseWrestleChoke(ctrl);
    var toRape = new FarmerHouseWrestleRape(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You have to stop her from screaming for help.
                            The only way you can do that is by covering her mouth, which you can't reach.
                            You're going to have to [b]bring her down[/b] to your level somehow.
                            And that means you need to get [i]physical[/i].
                        [/p]
                        [p]
                            You wildly charge the scared housewife, roughly grabbing her by the hips, the very centre of her mass, in an attempt to throw her to the floor.
                            She harshly [b]slaps[/b] you in the face, but that doesn't prevent you from tugging her away from the kitchen counter,
                            and [b]hurling[/b] her to the ground, the human falling face-first on top of the rotten, wooden floor with a loud thud, and a soft pained grunt.
                        [/p]
                        ${global.ui.condSucc(`Strength greater than 5`)}
                        [p]
                            You leap on top of the woman, [b]slamming[/b] your body into hers.
                            While you don't weigh that much, your move still suffices to knock the air out of the housewife's lungs,
                            preventing her from screaming for just long enough for you take hold of both of her arms, restraining them behind her back.
                            The moment she recovers her breath enough to attempt to shriek, you firmly plant the palm of your free hand on the lower half of her face,
                            her shout muffled to the point of inaudibility.
                        [/p]
                        [p]
                            The feisty human tries, but fails, to bite your palm.
                            She attempts to free her arms, to no avail.
                            She may be larger than you, but she [i]isn't[/i] by any means [b]stronger[/b].
                            Giving up on overpowering you, the woman begins to wildly kick her legs and wriggle her body around, striving to throw you off of her.
                        [/p]
                        [p]
                            Growing tired of her resistance, you cruelly press your knee into her back,
                            causing her more than a little discomfort while pinning her to the floor.
                            Still, she does not relent, both her legs still wildly flailing about.
                            Stupid human just doesn't know when to give up.
                            You press your sharp claws against her cheeks, digging into her soft and tender flesh without drawing blood,
                            while at the same time painfully twisting one of her arms.
                        [/p]
                        [p]
                            Finally, her protests die down, at least to the point where you don't have to actively [b]struggle[/b] to keep her in place.
                            While the housewife clearly hasn't [i]given up[/i] yet, you've scared her into calming down, for now.
                            Still, you doubt she'll remain [i]pacified[/i] for long.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildChokeChoice() {
        var disabled = ctrl.wrestle.has() ? "disabled" : "";
        var isSelected = ctrl.wrestle.get() === FarmerHouseWrestleSuccessChoice.CHOKE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${chokeId}" ${disabled}>Choke her</button>`;

        var tooltip = global.parser.parse(`Knock her out so you can loot in peace.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRapeChoice() {
        var disabled = ctrl.wrestle.has() ? "disabled" : "";
        var isSelected = ctrl.wrestle.get() === FarmerHouseWrestleSuccessChoice.RAPE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${rapeId}" ${disabled}>Rape her</button>`;

        var tooltip = global.parser.parse(`You've got her pinned down. Time to have some [i]fun[/i] with the human.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildChokeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildRapeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + chokeId), $("#" + rapeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerChokeListener() {
        $("#" + chokeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.wrestle.set(FarmerHouseWrestleSuccessChoice.CHOKE);
            loadNextPart(toChoke);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerRapeListener() {
        $("#" + rapeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.wrestle.set(FarmerHouseWrestleSuccessChoice.RAPE);
            loadNextPart(toRape);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.wrestle.change();
        part += buildChoice();
        part += `</div>`;
        part += toChoke.build();
        part += toRape.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toChoke.registerListeners();
        toRape.registerListeners();

        if (ctrl.wrestle.has()) {
            return;
        }

        registerChokeListener();
        registerRapeListener();
    };
}

var FarmerHouseWrestleSuccessChoice = {
    CHOKE: 1,
    RAPE: 2
};

// END scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleSuccess.js

// scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleChokeContinue.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestleChokeContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You exit the bedroom, returning to the living room.
                            Just going out the front door would be risky.
                            [b]Too[/b] risky.
                            You make your way back through the dining room into the kitchen.
                            The woman is gradually starting to wake up, her groans inspiring a sense of fear in you.
                            Looks like your time here is running out, and if you don't escape before she regains her senses fully,
                            you'll be [b]hunted down[/b] by whoever comes to her aid.
                            You bolt past her downed frame into the adjacent hallway, and exit the building the same way you entered it: quietly, and through the back.
                        [/p]
                        [p]
                            The moment you close the door behind you, a cry for help resonates from within the building.
                            Looks like the human woke up.
                            You weren't planning on sticking around anyway.
                            You sprint into the adjacent potato field, and after crossing it with some difficulty,
                            you escape into a small patch of woodland with nobody but the housewife having seen you.
                        [/p]
                        [p]
                            Still, just because you [i]may[/i] not be getting chased, doesn't mean you don't expeditiously make your way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least while on the road you can [b]pretend[/b] to be a traveller instead of a [b]woman-choking bandit[/b].
                            You're glad you didn't leave the farmer's house empty-handed.
                            Getting your loot might have been a little harder to achieve than what you'd hoped for,
                            but regardless of the amount of extra effort put in, the results are there.
                        [/p]
                        [p]
                            Reaching the road, you continue on your way to Varanar.
                            Hopefully, your streak of good luck will continue in the city.
                            And maybe you won't even have to [b]strangle[/b] anyone, the next time you attempt to break in somewhere.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }
    
    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.choke.continued()) {
            return "";
        }

        global.kobold.history.shared.farmerhouse.stole = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.choke.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleChokeContinue.js

// scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleChoke.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestleChoke(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseWrestleChokeContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You remove your hand from the housewife's mouth, instantly wrapping your arm around her neck before she manages to utter a shout for help,
                            her attempts at a desperate plea reduced to a bunch of surprised gurgling noises as you squeeze her throat shut.
                            With her air supply cut off, the woman resumes her frantic struggling, even more intense than before.
                            While you've no intention of [i]killing[/i] her, [b]she[/b] doesn't know that.
                            Fearing for her very life, the human wildly writhes about, kicking her legs,
                            fighting against your grasp on her arms and wriggling her body from side to side in an effort to throw you off of her.
                        [/p]
                        [p]
                            While threatening violence doesn't seem to dissuade her this time,
                            you manage to keep the housewife firmly pinned down and in place throughout the entirety of her protests.
                            Merely half a minute after you began to choke her out, her floundering slowly begins to die down,
                            until her body finally slumps down to the ground, unconsciousness setting in.
                            You could very well [b]kill[/b] her, but you're a [b]thief[/b], and not a murderer.
                            You'd have preferred if things didn't get this [i]physical[/i] to begin with.
                        [/p]
                        [p]
                            The woman knocked out, you rise to your feet and take a quick breather.
                            You doubt she'll be out for long, however.
                            You need to get to looting this place, and fast.
                        [/p]
                        [p]
                            You step into the dining room.
                            Nothing worth a damn in here, at first glance.
                            You keep moving, entering the living room, or at least what's supposed to pass for it.
                            The room is dominated by a single padded chair, the most comfortable piece of furniture you've come across in the house, so far.
                            It stands before a small fireplace, probably the only source of [i]warmth[/i] apart from the oven in the kitchen.
                            Still, nothing interesting to take.
                        [/p]
                        [p]
                            There's only one room left, which you assume to be the bedroom.
                            Opening the shabby wooden door, your suspicions are confirmed.
                            Two beds.
                            A big one, and a small one.
                            A dresser, and a nightstand.
                            You decide to go for the latter.
                        [/p]
                        [p]
                            Opening the small drawer, you are surprised to find that it [b]isn't[/b] full of underwear.
                            Instead, the nightstand contains a pair of worn books,
                            atop which lay some out-of-place looking ${global.kobold.inventory.createItemWithTooltip(new FancyGlasses())}.
                            Your eyes open wide, avarice surging to a whole new level.
                            This find more than makes up for having to deal with the housewife.
                            If you knew these farmers had something [b]this[/b] expensive laying around,
                            you wouldn't have stopped attempting to steal from them after your first attempt.
                        [/p]
                        [p]
                            You figure they must be a family heirloom or something.
                            Regardless, they're [b]yours[/b] now.
                            You pocket the valuable item, and start thinking of a way out of this place.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.choke.continued() ? "disabled" : "";
        var isSelected = ctrl.choke.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.choke.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.wrestle.get() !== FarmerHouseWrestleSuccessChoice.CHOKE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.choke.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.wrestle.get() !== FarmerHouseWrestleSuccessChoice.CHOKE) {
            return;
        }
        
        next.registerListeners();

        if (ctrl.choke.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleChoke.js

// scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleFail.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestleFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You have to stop her from screaming for help.
                            The only way you can do that is by covering her mouth, which you can't reach.
                            You're going to have to [b]bring her down[/b] to your level somehow.
                            And that means you need to get [i]physical[/i].
                        [/p]
                        [p]
                            You wildly charge the scared housewife, roughly grabbing her by the hips, the very centre of her mass, in an attempt to throw her to the floor.
                            While you succeed in throwing off her balance, the woman manages to grab the countertop behind her,
                            preventing herself from falling to the floor despite the most [b]ferocious[/b] pulling, tugging and throwing you manage to muster.
                            Realizing your initial plan has failed, you reach for the human's mouth in an attempt to cover it,
                            but the housewife seems far more [i]feisty[/i] than you'd have thought.
                        [/p]
                        ${global.ui.condFail(`Strength below 6`)}
                        [p]
                            She crudely [b]bashes[/b] your face with her elbow, 
                            her linen shirt doing little to soften the heavy blow to your snout, which sends you recoiling backwards.
                            You let out a pained grunt, clutching your muzzle.
                            Blood flows liberally from your nostrils.
                            The woman, no longer being held by you, flees into adjacent dining room.
                            While you're still reeling from her blow, trying to figure out whether your nose is broken or merely bleeding,
                            the human starts [b]screaming[/b] for help, just as you feared would happen.
                        [/p]
                        [p]
                            Her cries resonate throughout the entire decrepit house, and are undoubtably heard all over the hamlet.
                            Dreading the [i]retribution[/i] of whoever may be coming to [i]help[/i],
                            you collect your senses enough to [b]bolt[/b] out of the back door from whence you came.
                            A small mob of pitchfork-wielding farmers has already formed at the front of the house,
                            being instructed by the lady-in-distress to go after the kobold that spooked her.
                            With nowhere to hide that [i]wouldn't[/i] lead to certain death, you sprint into the nearby potato field, which is luckily unmanned.
                        [/p]
                        [p]
                            As the peasants comb through the house and the back yard,
                            one of them notices you vanishing into a small gathering of trees at the far end of the farmland.
                            Luckily, the group decides [b]not[/b] to give chase.
                            Your lead is too great for them to have any hopes of catching up,
                            and the men consider having chased you off a victory on its own.
                        [/p]
                        [p]
                            Despite having ditched your pursuers, you continue running all the way back to the eastern Varanar road.
                            The villagers might call the guards to investigate, and should anyone ask,
                            at least here you can [b]pretend[/b] to be a traveller instead of a [b]miserable failure[/b] of a thief.
                            Exhausted, you brace yourself against a nearby tree.
                            Your nose still hurts like hell, but the bleeding must have stopped somewhere between the hamlet and the road.
                            You're fairly sure nothing is [i]broken[/i], although it might be wise to see a medician once you're in town.
                        [/p]
                        [p]
                            If you were just a [i]little bit[/i] [b]stronger[/b], you [i]might[/i] have been able to overpower the woman.
                            Alas, what's done is done.
                            You can't win [i]every[/i] fight you get into, and at least you didn't get [b]caught[/b].
                            You've gained nothing, but you haven't lost much either, besides an opportunity and the ability to breathe through your nose.
                            The latter is bound to return once the rest of the blood clears out.
                        [/p]
                        [p]
                            Your brief period of respite over, you continue on the way to Varanar.
                            Hopefully, the humans there will be less [i]hardy[/i].
                            That's what the hamlet's inhabitants constantly claim, anyway.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.kobold.history.shared.farmerhouse.raped = true;
        global.kobold.history.shared.farmerhouse.failed = true;
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.wrestleFail();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestleFail.js

// scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestle.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseWrestle(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toSuccess = new FarmerHouseWrestleSuccess(ctrl);
    var toFail = new FarmerHouseWrestleFail(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}


    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.WRESTLE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (ctrl.misc.strength() < 6) {
            part += toFail.build();
        } else {
            part += toSuccess.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.confront.get() !== FarmerHouseConfrontChoice.WRESTLE) {
            return;
        }

        if (ctrl.misc.strength() < 6) {
            toFail.registerListeners();
        } else {
            toSuccess.registerListeners();
        }
    };
}

// END scenes\HamletDay2\FarmerHouse\confront\wrestle\FarmerHouseWrestle.js

// scenes\HamletDay2\FarmerHouse\confront\FarmerHouseConfront.js

/**
 * Confront section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseConfront(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var threatenId = arguments.callee.name + "ButtonOne";
    var intimidateId = arguments.callee.name + "ButtonTwo";
    var wrestleId = arguments.callee.name + "ButtonThree";

    var toThreaten = new FarmerHouseThreaten(ctrl);
    var toIntimidate = new FarmerHouseIntimidate(ctrl);
    var toWrestle = new FarmerHouseWrestle(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Your mind races to come up with a solution to this [i]problem[/i].
                            You can't just leave the human be.
                            She's blocking your way into the rest of the house,
                            and if she keeps wandering about, she's bound to stumble upon you sooner or later.
                            If that happens, you're toast.
                            You decide it's best to take matters into your own hands.
                            That way, you might have a chance to stifle the woman's startled shouts instead of [i]you[/i] being the one getting surprised by [i]her[/i].
                        [/p]
                        [p]
                            You step out into the kitchen, approaching the housewife from behind.
                            She seems to be making ham sandwiches, likely the family's dinner.
                            Most human women you encounter wear dresses, but this one seems to be clothed a bit more [i]practically[/i].
                            She wears a plain, ragged linen top with torn sleeves, combined with what appears to be a home-made pair of leather pants.
                            Despite visibly being stitched together from various different pieces of animal hide,
                            the peasant-lady's trousers accentuate her form rather nicely.
                            While by no means young, she's far from [i]old[/i].
                            You'd wager her to be in her late thirties, perhaps early fourties.
                        [/p]
                        [p]
                            Suddenly, a floorboard creaks underneath your foot, alerting the woman to your presence.
                            She turns around, at first looking [i]over[/i] you,
                            seemingly on the verge of returning to her duties before lowering her eyes,
                            her gaze falling directly on top of the kobold standing in the middle of her kitchen.
                        [/p]
                        [p]
                            The woman's mouth drops agape.
                            She rubs her eyes to make sure she isn't dreaming.
                            Time itself seems to slow down as adrenaline begins to flood your body.
                            Your brain shoots into overdrive to try and formulate a plan of action that [i]doesn't[/i] entail you [b]running away[/b],
                            a primal urge to [i]flee[/i] building at the back of your head.
                        [/p]
                        [p]
                            On one hand, she's a human.
                            She's [i]much[/i] taller than you, and probably smarter, too.
                            Plus she lives here, which means she knows the layout of the house.
                            You don't.
                            A disadvantage, should this end in a chase.
                            On the other hand, she seems almost as scared as you were when she first entered the kitchen.
                            You [i]could[/i] try to threaten violence to keep her in line, or you could intimidate her in some other way.
                            A final possibility would be to [i]tackle[/i] the woman.
                            She may be twice your size, but that won't matter if you can force her to the floor.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThreatenChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.THREATEN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${threatenId}" ${disabled}>Threaten her</button>`;

        var tooltip = global.parser.parse(`Threaten the woman's life to keep her in check.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildIntimidateChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.INTIMIDATE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${intimidateId}" ${disabled}>Intimidate her</button>`;

        var tooltip = global.parser.parse(`Shout loudly to inspire fear.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 130px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildWrestleChoice() {
        var disabled = ctrl.confront.has() ? "disabled" : "";
        var isSelected = ctrl.confront.get() === FarmerHouseConfrontChoice.WRESTLE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${wrestleId}" ${disabled}>Wrestle</button>`;

        var tooltip = global.parser.parse(`Tackle the human and wrestle her to the ground.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You loot: </div>`;
        choice += buildThreatenChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildIntimidateChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildWrestleChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + threatenId), $("#" + intimidateId), $("#" + wrestleId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerThreatenListener() {
        $("#" + threatenId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.THREATEN);
            loadNextPart(toThreaten);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerIntimidateListener() {
        $("#" + intimidateId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.INTIMIDATE);
            loadNextPart(toIntimidate);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerWrestleListener() {
        $("#" + wrestleId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.confront.set(FarmerHouseConfrontChoice.WRESTLE);
            loadNextPart(toWrestle);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.arrival.get() !== FarmerHouseArrivalChoice.CONFRONT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toThreaten.build();
        part += toIntimidate.build();
        part += toWrestle.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.arrival.get() !== FarmerHouseArrivalChoice.CONFRONT) {
            return;
        }

        toThreaten.registerListeners();
        toIntimidate.registerListeners();
        toWrestle.registerListeners();

        if (ctrl.confront.has()) {
            return;
        }

        registerThreatenListener();
        registerIntimidateListener();
        registerWrestleListener();
    };
}

var FarmerHouseConfrontChoice = {
    THREATEN: 1,
    INTIMIDATE: 2,
    WRESTLE: 3
};

// END scenes\HamletDay2\FarmerHouse\confront\FarmerHouseConfront.js

// scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseCabinetRun.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseCabinetRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You silently close the door behind you, and turn around to face what you hope will be an easy getaway.
                            Instead, you are greeted with the sight of a broad-postured, scythe-wielding human, presumably the farmer who lives here.
                            He glares down at you.
                            You stare back at him.
                            For a moment, neither of you move.
                            He grips his scythe with both hands.
                            You can almost [b]feel[/b] the tension in the air.
                        [/p]
                        [p]
                            [big]"[color=red][b]Thief![/b][/color]"[/big]
                            the man bellows out in a deep, bass voice, breaking the small stalemate between the two of you, rallying comrades to his aid.
                            The human advances upon you.
                            He attacks you with an overhead scythe-swipe, which you narrowly manage to avoid.
                            The blade of his weapon ends up being stuck in the wooden backdoor, giving you just enough time to run past him, into the adjacent potato field.
                            The farmer struggles to recover his lethal tool, although not for long.
                            He gives chase, wildly swings his scythe at you, only [b]barely[/b] missing your tail with every cut.
                            To make matters worse, he's [b]gaining[/b] on you.
                        [/p]
                        [p]
                            Being almost twice your size, he can take far bigger steps, thus traversing the rough farmland with greater ease.
                            Halfway across the field, you can almost [b]feel[/b] his weapon cutting through the air, nearly slicing off the very tip of your hind appendage.
                            Right as he was about to land a definitive strike, the clumsy human trips over a pile of dirt, landing face-first on the hard, tilled ground.
                            You glance back at him just to gloat, but the sight of a veritable [b]mob[/b] of armed farmers rushing out of the hamlet towards you 
                            inspires you to keep running instead of rubbing the man's defeat in his face.
                        [/p]
                        [p]
                            For the second time in recent history, you find yourself running from a group of angry peasants.
                            This time, luckily, you have somewhat of a headstart.
                            That, and the farmers chasing you seem more interested in helping the man that originally chased you off to his feet,
                            rather than pursuing you as you escape through a sparse set of trees.
                            Still, you keep running for quite a while, all the way back to the eastern road to Varanar.
                        [/p]
                        [p]
                            Confident that you've lost your pursuers, you take a moment to catch your breath.
                            That was close.
                            If that stupid farmer hadn't tripped, you'd be minced meat by now.
                            Luckily, you made it out alive, with some goodies to show for it.
                            The cupboard was annoying to loot, to say the least, but in the end, you feel like you at at least got something out of this entire ordeal.
                            It may not be much, but it's more than nothing.
                            And more than nothing is all you need to survive in Varanar.
                            [small]Or so you hope.[/small]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.cabinetSteal.continued()) {
            return "";
        }

        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.cabinetSteal.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseCabinetRun.js

// scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseDrawer.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseDrawer(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseCabinetRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You can't open the cupboard, and you can't reach the top shelf.
                            That leaves the drawer.
                            You grab the metallic handle and tug on it, half-expecting the tray to be jammed much like the bottom compartiment is.
                            To your surprise, the drawer opens rather smoothly.
                            Overestimating the amount of force needed, you almost pull it out completely, your yank luckily stopping just short of disaster.
                        [/p]
                        [p]
                            The drawer seems to be mostly filled with worthless stuff.
                            Letters, a ball of yarn, an old, moldy cookie, some religious book you don't recognize and a spare broom head.
                            Disappointed, you begin to rummage through the stuff.
                            Surely there's something better underneath all of this [b]crap[/b], right?
                        [/p]
                        [p]
                            Nothing.
                            You start to get annoyed.
                            You toss the letters to the floor.
                            You put the well-worn book on the cabinet.
                            You throw the ball of yarn across the hallway in aggravation.
                            And then, hiding in the corner of the drawer, beyond the now-discarded thread, your prize is revealed to you:
                            a single ${global.kobold.inventory.createItemWithTooltip(new CopperRing())}.
                        [/p]
                        [p]
                            It probably slipped off of someone's finger, or maybe it was stored in there for safekeeping.
                            Regardless, you pocket it as your just due for having to stand through the harrowing sensation of almost being caught.
                            It isn't nearly as much as you'd hoped to gain from this venture, but it's more than nothing.
                            Honestly, you shouldn't have expected anything more valuable coming from a farmer's home.
                            You're glad you got at least [b]something[/b] out of this ordeal.
                        [/p]
                        [p]
                            You don't bother cleaning up after yourself.
                            Honestly, you're doing these dumb, disorganized humans a favor by emptying out their drawer full of trash.
                            Maybe when they pick up the stuff you threw to the floor, they'll store it in a proper fashion.
                            [i]And get rid of that disgusting cookie in the process.[/i]
                            Still somewhat frustrated, you silently slip out through the back door from whence you came.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.cabinetSteal.continued() ? "disabled" : "";
        var isSelected = ctrl.cabinetSteal.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinetSteal.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.DRAWER) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.cabinetSteal.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.DRAWER) {
            return;
        }

        next.registerListeners();

        if (ctrl.cabinetSteal.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseDrawer.js

// scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHousePry.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHousePry(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseCabinetRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The compartiment at the bottom of the cabinet probably contains the best goodies.
                            It makes sense, right?
                            Biggest storage, best stuff.
                            You take hold of the metallic handle attached to the wooden door, and firmly [b]yank[/b] it backwards,
                            but it absolutely [b]refuses[/b] to budge.
                            You don't know whether there's some kind of lock you're not seeing, if the hinges are rusted, or if the wood is somehow blocked.
                        [/p]
                        [p]
                            Annoyed, you begin to tug at the jammed door, briskly rattling it back and forth to little effect.
                            You stop once you become aware of the noise you're making.
                            Luckily, the woman doesn't seem to notice.
                            The walls of this house, while rotten, must be pretty thick.
                        [/p]
                        [p]
                            You go for a new approach.
                            Applying pressure to the winch-side of the door with one hand, you start to steadily pull on the handle with the other.
                            You begin to exert ever-intensifying force.
                            The wood begins to creak and crack.
                            Right as you think you've [b]nearly[/b] got it, the entire thing gives way!
                            While you had meant to force the door open, you [b]tear[/b] it from the cupboard as a whole,
                            the decaying wood no longer suffice to hold the winches in place.
                        [/p]
                        [p]
                            The cabinet breaks open surprisingly quietly.
                            You place the torn-off door on the floor.
                            The contents of the cupboard are revealed to you:
                            a single, tiny bag of gold, which you presume to be the farmer's life savings.
                            A small amount of coins stashed away for a rainy day.
                        [/p]
                        [p]
                            While you do feel kind of bad for the peasant family, you probably need this money more than they do.
                            They may be pretty low on the social ladder, but they're still [b]humans[/b].
                            They have rights, and laws protecting them and whatnot.
                            Compared to you, they'll have an easy life even without this coin purse.
                            Besides, you're a thief.
                            Stealing is just what you [i]do[/i].
                            It's in your nature.
                            In your blood.
                        [/p]
                        [p]
                            This isn't the time to grow remorseful.
                            You pocket the bag of coins.
                            There's no point in trying to [i]fix[/i] the cabinet.
                            You leave the broken part on the floor, and slip out the back door from whence you came.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.cabinetSteal.continued() ? "disabled" : "";
        var isSelected = ctrl.cabinetSteal.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinetSteal.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.PRY) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.cabinetSteal.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.PRY) {
            return;
        }

        next.registerListeners();

        if (ctrl.cabinetSteal.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHousePry.js

// scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseTopShelf.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseTopShelf(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseCabinetRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The box on top of the cabinet has piqued your attention.
                            You can't quite see what it contains,
                            but you're pretty certain a poor family like this would have scrapped the crate for firewood long ago if it weren't something [i]good[/i].
                            Why put it all the way [i]up there[/i], though?
                            These humans have to make [b]everything[/b] harder than it needs to be, don't they?
                        [/p]
                        [p]
                            With a soft sigh and a rolling of your eyes, you hoist yourself onto the bottom half of the cupboard.
                            The wooden piece of furniture creaks menacingly as you place your entire weight upon it,
                            almost as though it's [b]threatening[/b] to crack should you proceed to scale its rotten shelves.
                            Even standing upright on the lower half, you still can't reach the top.
                        [/p]
                        [p]
                            You consider your options.
                            You [i]could[/i] jump up and try to snag the crate, but that would probably cause the entire cabinet to break upon your landing.
                            Besides the noise attracting unwanted attention, you might fall,
                            and if there's something fragile in the box, all of your efforts may have been for nothing.
                        [/p]
                        [p]
                            Jumping ruled out, you begin to test the individual [i]racks[/i].
                            Most of them are rotten through and through, a few barely even withstanding your gentle, testing pats.
                            Only [b]one[/b] of the planks seems stable enough to hold any sort of weight, at least for a little while.
                            It's the bottom-most one, a few inches above the surface you're standing on,
                            perfect for giving you a small boost, which may be just enough for you to reach the top.
                        [/p]
                        [p]
                            You gingerly place one of your feet atop of the non-rotten plank, testing its limits.
                            You aren't too sure about this, but it's the only way you're going to get to the crate that [i]doesn't[/i] involve making a ruckus.
                            You decide to go for it.
                            Hoping for the best, you lift yourself upwards, hoisting your entire body up onto the tips of your toes.
                            The box coming into reach, you grab onto it, slide it off the top of the cabinet, and in a single, smooth motion,
                            you drop back down onto the lower half of the cupboard, your descent aided by the sheer [b]weight[/b] of the crate.
                        [/p]
                        [p]
                            The wooden piece of furniture creaks even loader than before, as though it's on the [b]verge[/b] of breaking.
                            You decide to hop off of it [i]before[/i] you ruin your slick move.
                            You quietly hop to the ground.
                            It is only when you land for a second time, that the heaviness of the open-topped crate begins to burden you.
                            You quickly set it down on the floor, careful not to break any of the glass bottles contained within.
                        [/p]
                        [p]
                            You let out a soft, relieved sigh.
                            Those were a couple of intense moments, but all things considered, you made it out just fine.
                            Time to inspect the loot.
                            You take a bottle out of the box.
                            Empty.
                            No worries, you probably just picked the wrong one to begin with.
                            You pull out another.
                            Also empty?!
                            Did you just risk [b]all of that[/b] for a crate filled with [b]empty bottles[/b]?!
                        [/p]
                        [p]
                            You frantically remove container after container from the box, the most filled one thus far containing naught but a few [b]droplets[/b].
                            This can't be happening.
                            Why would they put [b]empty bottles[/b] on top of the darned cupboard?!
                            Your hope of scoring a good haul having all but sunk, you pull out the very last of the glass flasks.
                            You can immediately feel that it's different.
                            Heavier.
                            A smile forms on your face.
                            It's full.
                        [/p]
                        [p]
                            "${global.kobold.inventory.createItemWithTooltip(new CouragePotion(), true)}", it says on the label.
                            You'll have to try it out later.
                            Or you could sell it at Varanar.
                            Either way, now is not the time to think about what you're going to do with the bottle.
                            You pocket it, and silently slip through the back door from which you came.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.cabinetSteal.continued() ? "disabled" : "";
        var isSelected = ctrl.cabinetSteal.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinetSteal.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.TOPSHELF) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.cabinetSteal.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.cabinet.get() !== FarmerHouseCabinetChoice.TOPSHELF) {
            return;
        }

        next.registerListeners();

        if (ctrl.cabinetSteal.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseTopShelf.js

// scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseCabinet.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseCabinet(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var drawerId = arguments.callee.name + "ButtonOne";
    var pryId = arguments.callee.name + "ButtonTwo";
    var topShelfId = arguments.callee.name + "ButtonThree";

    var toDrawer = new FarmerHouseDrawer(ctrl);
    var toPry = new FarmerHousePry(ctrl);
    var toTopShelf = new FarmerHouseTopShelf(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The cupboard is your best bet.
                            You doubt the woman'll be returning to the kitchen any time soon,
                            and even if she does, she might just [i]overlook[/i] you like she did the first time.
                            Besides, what better place to store a bunch of valuables than a sole piece of furniture at the very rear end of the house?
                        [/p]
                        [p]
                            You step out from behind your cover, and turn to face the cabinet properly.
                            It is a decently-sized, rickety construct, parts of which are rotten to the point where you could probably punch right through the wood.
                            The bottom half consists of a single, rather large compartiment, which is topped off by a drawer.
                            Further upwards are a bunch of empty shelves, but you can see a crate of some sort on the very topmost one, far out of your reach.
                            You try to open the compartiment instead, but it is jammed shut.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildDrawerChoice() {
        var disabled = ctrl.cabinet.has() ? "disabled" : "";
        var isSelected = ctrl.cabinet.get() === FarmerHouseCabinetChoice.DRAWER;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${drawerId}" ${disabled} style="width:190px;">Check the drawer</button>`;

        var tooltip = global.parser.parse(`The easiest part to get to. Bound to contain some goodies.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width:190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildPryChoice() {
        var strongEnough = ctrl.misc.strength() >= 6;
        var disabled = ctrl.cabinet.has() || !strongEnough ? "disabled" : "";
        var isSelected = ctrl.cabinet.get() === FarmerHouseCabinetChoice.PRY;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${pryId}" ${disabled} style="width:190px;">Force it open</button>`;

        var tooltip = global.parser.parse(`Requires [b]6 strength[/b].`);
        if (strongEnough) {
            tooltip = global.parser.parse(`The small wooden door is jammed shut, but you can pry it open with some elbow grease.`);
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width:190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildTopShelfChoice() {
        var statCheck = ctrl.misc.agility() >= 6;
        var disabled = ctrl.cabinet.has() || !statCheck ? "disabled" : "";
        var isSelected = ctrl.cabinet.get() === FarmerHouseCabinetChoice.TOPSHELF;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${topShelfId}" ${disabled} style="width:190px;">Climb up</button>`;

        var tooltip = global.parser.parse(`Requires [b]6 agility[/b].`);
        if (statCheck) {
            tooltip = global.parser.parse(`Climb on top of the cabinet to reach the top shelf.`);
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width:190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildDrawerChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildPryChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildTopShelfChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + drawerId), $("#" + pryId), $("#" + topShelfId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerDrawerListener() {
        $("#" + drawerId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinet.set(FarmerHouseCabinetChoice.DRAWER);
            loadNextPart(toDrawer);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerPryListener() {
        $("#" + pryId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinet.set(FarmerHouseCabinetChoice.PRY);
            loadNextPart(toPry);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerTopShelfListener() {
        $("#" + topShelfId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.cabinet.set(FarmerHouseCabinetChoice.TOPSHELF);
            loadNextPart(toTopShelf);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.inside.get() !== FarmerHouseInsideChoice.CABINET) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toDrawer.build();
        part += toPry.build();
        part += toTopShelf.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.inside.get() !== FarmerHouseInsideChoice.CABINET) {
            return;
        }

        toDrawer.registerListeners();
        toPry.registerListeners();
        toTopShelf.registerListeners();

        if (ctrl.cabinet.has()) {
            return;
        }

        registerDrawerListener();
        registerPryListener();
        registerTopShelfListener();
    };
}

var FarmerHouseCabinetChoice = {
    DRAWER: 1,
    PRY: 2,
    TOPSHELF: 3
};

// END scenes\HamletDay2\FarmerHouse\steal\inside\cabinet\FarmerHouseCabinet.js

// scenes\HamletDay2\FarmerHouse\steal\inside\follow\FarmerHouseFollowRun.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseFollowRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You silently close the door behind you, and turn around to face what you hope will be an easy getaway.
                            Instead, you are greeted with the sight of a broad-postured, scythe-wielding human, presumably the farmer who lives here.
                            He glares down at you.
                            You stare back at him.
                            For a moment, neither of you move.
                            He grips his scythe with both hands.
                            You can almost [b]feel[/b] the tension in the air.
                        [/p]
                        [p]
                            [big]"[color=red][b]Thief![/b][/color]"[/big]
                            the man bellows out in a deep, bass voice, breaking the small stalemate between the two of you, rallying comrades to his aid.
                            The human advances upon you.
                            He attacks you with an overhead scythe-swipe, which you narrowly manage to avoid.
                            The blade of his weapon ends up being stuck in the wooden backdoor, giving you just enough time to run past him, into the adjacent potato field.
                            The farmer struggles to recover his lethal tool, although not for long.
                            He gives chase, wildly swings his scythe at you, only [b]barely[/b] missing your tail with every cut.
                            To make matters worse, he's [b]gaining[/b] on you.
                        [/p]
                        [p]
                            Being almost twice your size, he can take far bigger steps, thus traversing the rough farmland with greater ease.
                            Halfway across the field, you can almost [b]feel[/b] his weapon cutting through the air, nearly slicing off the very tip of your hind appendage.
                            Right as he was about to land a definitive strike, the clumsy human trips over a pile of dirt, landing face-first on the hard, tilled ground.
                            You glance back at him just to gloat, but the sight of a veritable [b]mob[/b] of armed farmers rushing out of the hamlet towards you 
                            inspires you to keep running instead of rubbing the man's defeat in his face.
                        [/p]
                        [p]
                            For the second time in recent history, you find yourself running from a group of angry peasants.
                            This time, luckily, you have somewhat of a headstart.
                            That, and the farmers chasing you seem more interested in helping the man that originally chased you off to his feet,
                            rather than pursuing you as you escape through a sparse set of trees.
                            Still, you keep running for quite a while, all the way back to the eastern road to Varanar.
                        [/p]
                        [p]
                            Confident that you've lost your pursuers, you take a moment to catch your breath.
                            That was close.
                            If that stupid farmer hadn't tripped, you'd be minced meat by now.
                            Luckily, you made it out alive, with some yummy sandwiches to show for it.
                            The housewife was annoying to deal with, and you could have wrung her neck in frustration when she closed the door to the den behind her,
                            but in the end, you feel like you at at least got something out of this entire ordeal.
                            It may not be much, but it's more than nothing.
                            And more than nothing is all you need to survive in Varanar.
                            [small]Or so you hope.[/small]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.follow.continued()) {
            return "";
        }

        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.follow.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\follow\FarmerHouseFollowRun.js

// scenes\HamletDay2\FarmerHouse\steal\inside\follow\FarmerHouseFollow.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseFollow(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var nextId = arguments.callee.name + "ButtonOne";

    var next = new FarmerHouseFollowRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You doubt the cupboard'll contain much of interest.
                            The best stuff is probably scattered throughout the house.
                            Who puts all their eggs in one basket, right?
                            Hopefully not these humans.
                        [/p]
                        [p]
                            You sneak into the kitchen and peek around the corner.
                            The housewife, as you expected, has just finished setting up the small, round table for dinner.
                            Three metallic plates and two sandwiches, one of which is cut in half and distributed over two plates.
                            The family's meagre meal set up, the human steps out into the living room, closing the loudly creaking,
                            half-crooked dining room door behind her.
                            Must not be dinnertime yet.
                        [/p]
                        [p]
                            Darnit.
                            While you [i]do[/i] like having a solid surface in between you and the housewife,
                            the wooden door she shut bars your way to the rest of the house.
                            Once again, your plans are foiled and your progress is hampered.
                            You figure it's too late to back out now.
                            Best to grab what you can and get the hell out of this place.
                        [/p]
                        [p]
                            You scour the kitchen, looking for anything of value to take.
                            The counter is barren, apart from some basic, well-worn tools.
                            The small coal-oven underneath looks like it hasn't been used in [b]months[/b].
                            The coal-prod is too rusted to be of any use, and there aren't even any spices like salt or pepper around.
                            The woman took the bread along with her when she left this part of the house, leaving absolutely [b]nothing[/b] for you to steal.
                        [/p]
                        [p]
                            Annoyed, you step into the dining room.
                            Through the cracks of the door, you can see the woman brushing the floor on the other side.
                            This room contains naught but a single, kobold-sized cupboard and the table set for dinner.
                            You quietly rummage through the cupboard.
                            Nothing but worthless metal dishes, only the fanciest of which are made of iron.
                            Not even a blacksmith would pay a single coin for low-grade copper and bronze.
                        [/p]
                        [p]
                            The cutlery on the table, too, is exceedingly supbar.
                            That leaves just one option: the food itself.
                            The full-sized sandwich and its two smaller siblings are smeared with some kind of butter, upon which rests a thick slice of ham.
                            Your mouth waters just [b]thinking[/b] about the taste of an actual [b]meal[/b].
                            While you might not be able to sell the food for much, at least you'll have gained [i]something[/i] out this adventure.
                        [/p]
                        [p]
                            The moral implications of stealing from an obviously poor family, 
                            who may or may not have a child, as indicated by the third plate, come to mind for a moment.
                            You figure that if they had enough food to prepare [b]one[/b] meal, you're certain they'll have plenty to make a second one.
                            Plus, you've gone for [b]way[/b] longer than a single day without eating in the past.
                            You're certain it won't kill them to miss out on a couple of slices of bread and ham.
                        [/p]
                        [p]
                            While the allure of digging in and having a feast right here and now is great, you know better than to linger when trespassing.
                            You'll have to save these for later.
                            You grab the large sandwich, and both of the smaller ones, and pocket them.
                            While you'd have liked to get a lot more loot out of this place, you'll take what you can get.
                            You silently slip out through the back door from whence you came.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.follow.continued() ? "disabled" : "";
        var isSelected = ctrl.follow.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.follow.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.inside.get() !== FarmerHouseInsideChoice.FOLLOW) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.follow.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.inside.get() !== FarmerHouseInsideChoice.FOLLOW) {
            return;
        }
        
        next.registerListeners();

        if (ctrl.follow.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\inside\follow\FarmerHouseFollow.js

// scenes\HamletDay2\FarmerHouse\steal\inside\FarmerHouseInside.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseInside(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var cabinetId = arguments.callee.name + "ButtonOne";
    var followId = arguments.callee.name + "ButtonTwo";

    var toCabinet = new FarmerHouseCabinet(ctrl);
    var toFollow = new FarmerHouseFollow(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Going to the shed would be a dumb idea.
                            There's probably [b]nothing[/b] of value in there.
                            At least in here, there might be [b]something[/b] worth stealing.
                            Still, it's not going to be easy with the human in the kitchen.
                        [/p]
                        [p]
                            Then, as though your prayers were heard, a miracle happens.
                            Footsteps, once more.
                            You peek out from behind the cabinet, just in time to spot the woman leaving the kitchen, a pair of sandwiches in hand.
                            You wait for a few moments to see if she might return, but she doesn't.
                            You hear her putting down what you presume to be a pair of metallic plates in an adjacent room, probably the dining room or the den.
                        [/p]
                        [p]
                            Your way further into the house is cleared.
                            You [b]knew[/b] staying indoors was the correct way to go about things!
                            But still, this isn't the time to get cocky.
                            The woman may have left the kitchen, but she's still around.
                            It'd be a shame if you [i]bumped[/i] into her after all this stressful waiting and careful planning.
                            If you want to play it safe, you could simply see what the cabinet you've been hiding behind contains.
                            Following the woman as she goes about her business is a more [i]risky[/i] strategy,
                            but it still beats randomly roaming about the house in both security and stealing potential.
                            The human can't see you if you're behind her back, after all.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildCabinetChoice() {
        var disabled = ctrl.inside.has() ? "disabled" : "";
        var isSelected = ctrl.inside.get() === FarmerHouseInsideChoice.CABINET;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${cabinetId}" ${disabled} style="width:180px;">Rob the cabinet</button>`;

        var tooltip = global.parser.parse(`Steal from the cabinet.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width:180px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFollowChoice() {
        var disabled = ctrl.inside.has() ? "disabled" : "";
        var isSelected = ctrl.inside.get() === FarmerHouseInsideChoice.FOLLOW;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${followId}" ${disabled}>Follow her</button>`;

        var tooltip = global.parser.parse(`Follow the woman around and take what you can along the way.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildFollowChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildCabinetChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + cabinetId), $("#" + followId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerCabinetListener() {
        $("#" + cabinetId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.inside.set(FarmerHouseInsideChoice.CABINET);
            loadNextPart(toCabinet);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFollowListener() {
        $("#" + followId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.inside.set(FarmerHouseInsideChoice.FOLLOW);
            loadNextPart(toFollow);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.INSIDE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toCabinet.build();
        part += toFollow.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.INSIDE) {
            return;
        }

        toCabinet.registerListeners();
        toFollow.registerListeners();

        if (ctrl.inside.has()) {
            return;
        }

        registerCabinetListener();
        registerFollowListener();
    };
}

var FarmerHouseInsideChoice = {
    CABINET: 1,
    FOLLOW: 2
};

// END scenes\HamletDay2\FarmerHouse\steal\inside\FarmerHouseInside.js

// scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedTrident.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShedTrident(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";

    var next = new FarmerHouseShedRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You grab the pitchfork leaning against the wall.
                            It's [b]way[/b] too big to be of much use to you.
                            You place it atop the tiny table that holds most of the tools, the wooden handle sticking out over the edge.
                            You need to make this thing smaller somehow, so you can use it as a weapon.
                            An idea forms within your mind.
                        [/p]
                        [p]
                            You take the large, well-used, two-handed saw.
                            Another tool not cut for someone of your stature.
                            You can barely even [b]lift[/b] it due to the two heavy weights attached to either end.
                            You imagine this saw is used for cutting down trees with, but it's the best thing around to shorten the pitchfork with.
                        [/p]
                        [p]
                            Through some fidgeting and a lot of physical exertion to the point of annoyance,
                            you finally manage to get the saw to stand upright near the edge of the table, with the pitchfork's handle underneath its dull blade.
                            You gingerly begin to roll the heavy contraption back and forth as best you can, slowly carving it into the wooden grip below.
                            You need to stop your sawing ever so often, either to adjust the pitchfork to ensure it doesn't fall down, or just to take a breather.
                        [/p]
                        [p]
                            This tool was [b]not[/b] meant to be used  by a kobold.
                            You don't even think it was made for a [i]single[/i] human to employ.
                            The fact that it's more dull than your claws isn't helping.
                            Still, you perservere, deftly sawing away millimetre after millimetre, until finally, you reach the end.
                            As the saw carves slightly into the table, a large part of what was once the pitchfork's handle tumbles down to the ground.
                        [/p]
                        [p]
                            You did it!
                            With one final burst of effort, you quietly lay the saw back down onto table,
                            before taking hold of your brand new ${global.kobold.inventory.createItemWithTooltip(new KoboldTrident())}.
                            You raise the former tool up to inspect it.
                            Far smaller than before.
                            Lighter, and easier for you to use.
                            Perfect.
                            You stab the air a few times to get a feeling for your new weapon.
                            ${weapon ? `You ditch your old ${global.kobold.inventory.createItemWithTooltip(weapon)}.
                            You have found something better.` : `You feel like a spear-wielding knight.`}
                        [/p]
                        [p]
                            You [b]love[/b] it.
                            And you made it yourself!
                            Well, sort of, anyway.
                            The important part is that it's yours now.
                            With the pitchfork-turned-trident in hand, you excitedly leave the shed, ready to face whatever the world has to throw at you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.postshed.continued() ? "disabled" : "";
        var isSelected = ctrl.postshed.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postshed.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.shed.get() !== FarmerHouseShedChoice.TRIDENT) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.postshed.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.shed.get() !== FarmerHouseShedChoice.TRIDENT) {
            return;
        }

        next.registerListeners();

        if (ctrl.postshed.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedTrident.js

// scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedRun.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShedRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var gotWeapon = ctrl.shed.get() === FarmerHouseShedChoice.TRIDENT;
        var toReturn = `[p]
                            The moment you exit the shed, you are greeted with the sight of a broad-postured, scythe-wielding human,
                            presumably the farmer who lives here.
                            He glares down at you.
                            You stare back at him.
                            For a moment, neither of you move.
                            He grips his scythe with both hands.
                            You can almost [b]feel[/b] the tension in the air.
                        [/p]
                        [p]
                            [big]"[color=red][b]Thief![/b][/color]"[/big]
                            the man bellows out in a deep, bass voice, breaking the small stalemate between the two of you, rallying comrades to his aid.
                            ${gotWeapon ? `You consider attacking him with your new weapon, 
                            but if you stick around for much longer, the entire hamlet'll be up your ass. Instead, you` : `You`}
                            turn tail and run, bolting past the wooden shed into the potato field.
                            The farmer gives chase.
                            He wildly swings his scythe at you, only [b]barely[/b] missing your tail with every swipe.
                            To make matters worse, he's [b]gaining[/b] on you.
                        [/p]
                        [p]
                            Being almost twice your size, he can take far bigger steps, thus traversing the rough farmland with greater ease.
                            Halfway across the field, you can almost [b]feel[/b] his weapon cutting through the air, nearly slicing off the very tip of your hind appendage.
                            Right as he was about to land a definitive strike, the clumsy human trips over a pile of dirt, landing face-first on the hard, tilled ground.
                            You glance back at him just to gloat, but the sight of a veritable [b]mob[/b] of armed farmers rushing out of the hamlet towards you 
                            inspires you to keep running instead of rubbing the man's defeat in his face.
                        [/p]
                        [p]
                            For the second time in recent history, you find yourself running from a group of angry peasants.
                            This time, luckily, you have somewhat of a headstart.
                            That, and the farmers chasing you seem more interested in helping the man that originally chased you off to his feet,
                            rather than pursuing you as you escape through a sparse set of trees.
                            Still, you keep running for quite a while, all the way back to the eastern road to Varanar.
                        [/p]
                        [p]
                            Confident that you've lost your pursuers, you take a moment to catch your breath.
                            That was close.
                            If that stupid farmer hadn't tripped, you'd be minced meat by now.
                            ${gotWeapon ? `Luckily, you made it out alive, with a fancy new weapon to boot.` : `Luckily, the only thing that got sliced was your hand.`}
                            You consider your trip to the farmer's house a ${gotWeapon ? "relative success" : "horrible failure"}.
                            You can only hope your luck ${gotWeapon ? "won't change" : "will improve"} when you reach Varanar.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     *  @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.varanarSceneName, ctrl.varanarFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.postshed.continued()) {
            return "";
        }

        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runChange();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.postshed.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedRun.js

// scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedNoWeapon.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShedNoWeapon(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseShedRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You decide against taking a pitchfork as your weapon.
                            ${weapon ? `You already have a perfectly fine ${global.kobold.inventory.createItemWithTooltip(weapon)} anyway.` : ``}
                            Still, you aren't going to walk away from this entire ordeal empty-handed.
                            You don't want to have risked trespassing with absolutely [b]nothing[/b] to show for it.
                            You need [i]something[i], [b]anything[/b] to make it seem like you [b]didn't[/b] waste your time coming here.
                        [/p]
                        [p]
                            While most of the tools are neatly arrayed on a small table or hanging off the wall on half-crooked nails,
                            there's a metallic pile of rusty [b]rubbish[/b] cluttering a corner of the shed that you had neglected at first.
                            Seeing as everything else is practically worthless, you may as well look through it and see if there's anything of any value.
                        [/p]
                        [p]
                            You drop to your knees, and begin to rummage through the pile of metallic scrap.
                            After removing a few rusted pieces, you see a small glint of something shiny at the very bottom of the heap.
                            Excited, you eagerly reach for what you [b]hope[/b] to be a pile of coins buried underneath the rest of the trash,
                            yet long before you manage to make it to the object in question,
                            your hand brushes alongside an exceptionally [b]sharp[/b] piece of metal, which [b]rakes[/b] open your tender palm.
                        [/p]
                        [p]
                            You barely refrain from [b]screaming[/b] in pain.
                            You use your free hand to cover your muzzle, silencing the few pathetic whimpers that escape your lips.
                            You slowly retract the bleeding appendage from the pile of metal refuse, careful not to bump into any other jagged edges.
                            Blood leaks liberally from it, dripping down on top of the rusted heap, obscuring the glint that had caught your attention,
                            burying it, along with your self-esteem.
                        [/p]
                        [p]
                            Once fully removed from the scrapheap, you inspect the damage done to your palm, clutching your wrist with your free hand.
                            The cut is luckily not very [b]deep[/b], but it [b]is[/b] pretty long, 
                            running from the top of your wrist along the entire length of your palm, stopping just short of your index finger.
                            Despite it being shallow, your wound [b]hurts[/b] like [b]hell[/b], and it's bleeding quite a lot.
                        [/p]
                        [p]
                            You sit yourself down on the dusty wooden floor.
                            A few tears spill from your eyes.
                            You try to remain stoic, yet the pain mixed with the humiliation of utter [b]failure[/b] are too much.
                            That shiny thing you saw probably wasn't even a coin.
                            You quietly begin to sob.
                        [/p]
                        [p]
                            You're a thief, yet that's [b]twice[/b] now that you've completely [b]fucked up[/b] stealing from these simple farmers,
                            some of which have a life worse than your own.
                            Although right now, you find that a bit hard to believe.
                            As the bleeding slowly comes to a halt, you painfully wriggle your fingers around to make sure nothing critical was damaged.
                            Through teary eyes, you observe that all four of them are still in working condition,
                            although it'll be a while still before you can move them without hurting yourself.
                        [/p]
                        [p]
                            This is all that stupid farmer's fault.
                            If he had some [b]decent[/b] tools, you wouldn't have had to go rummaging in his pile of [b]crap[/b].
                            Why does he keep rusted metal around anyway?!
                            Stupid human.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            You wipe your eyes with your non-hurting hand.
                            Your tears have stopped, but your dignity remains dented.
                            You have been bested by a pile of scrap.
                            You [i]want[/i] to kick it, to get some [i]revenge[/i] on the inanimate objects that hurt you,
                            but you'd rather not draw attention, let alone cut your foot as well.
                            With a hurting hand and wounded pride, you decide to leave the shed.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.postshed.continued() ? "disabled" : "";
        var isSelected = ctrl.postshed.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postshed.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.shed.get() !== FarmerHouseShedChoice.NOWEAPON) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.postshed.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.shed.get() !== FarmerHouseShedChoice.NOWEAPON) {
            return;
        }
        
        next.registerListeners();

        if (ctrl.postshed.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShedNoWeapon.js

// scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShed.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var tridentId = arguments.callee.name + "ButtonOne";
    var noWeaponId = arguments.callee.name + "ButtonTwo";

    var toTrident = new FarmerHouseShedTrident(ctrl);
    var toNoWeapon = new FarmerHouseShedNoWeapon(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You make the wise decision of [b]not[/b] taking any unnecessary risks.
                            You creep towards the door, quietly open it once more, and slip out of the dilapidated homestead.
                            A second wave of relief is your body's way of thanking you for [b]not[/b] making it have to weather more uncertainty and stress.
                        [/p]
                        [p]
                            You walk towards the shed.
                            Like the back door, it is unlocked, and like the house, it is rickety to the point of barely being able to stand upright.
                            You reckon a firm gust of wind could break it down, but you're no engineer.
                            Like your home, it might be more stable than it appears.
                        [/p]
                        [p]
                            It looks like you were right.
                            Tools, but not just farming tools.
                            A near-blunt saw, a chipped hammer, a bunch of nails, a plow that's seen better days,
                            a spade made out of wood, a hoe, a rake and a pitchfork.
                            Those are the ones you recognize.
                            The rest, you imagine, are various farming and carpenting tools that a dishonest kobold like yourself has no use for.
                        [/p]
                        [p]
                            Darnit.
                            Where's all the good stuff at?!
                            You were hoping to find a scythe or a sickle or at least a pickaxe.
                            The farmer must have taken those.
                            Or maybe he just doesn't own them.
                            Seeing as he doesn't seem too keen on [b]maintaining[/b] what little he [i]does[/i] have,
                            you doubt the [i]quality[/i] of the tools would have been all that great anyway.
                        [/p]
                        [p]
                            The only thing serviceable in this entire shed would be the pitchfork.
                            It's far too big for someone of your [i]posture[/i] to use,
                            but maybe you can [i]trim[/i] the handle somehow.
                            ${weapon ? `You [b]would[/b] have to leave behind your ${global.kobold.inventory.createItemWithTooltip(weapon)} behind, though.
                            No point in carrying two weapons.` : ``}
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildTridentChoice() {
        var disabled = ctrl.shed.has() ? "disabled" : "";
        var isSelected = ctrl.shed.get() === FarmerHouseShedChoice.TRIDENT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${tridentId}" ${disabled}>Take it</button>`;

        var tooltip = global.parser.parse(`With some slight alterations, the pitchfork could make a formidable weapon.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNoWeaponChoice() {
        var disabled = ctrl.shed.has() ? "disabled" : "";
        var isSelected = ctrl.shed.get() === FarmerHouseShedChoice.NOWEAPON;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${noWeaponId}" ${disabled}>Do not</button>`;

        var tooltip = global.parser.parse(`A pitchfork? Hardly a weapon for a thief. What are you, a farmer?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildTridentChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildNoWeaponChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + tridentId), $("#" + noWeaponId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerTridentListener() {
        $("#" + tridentId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.shed.set(FarmerHouseShedChoice.TRIDENT);
            loadNextPart(toTrident);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerNoWeaponListener() {
        $("#" + noWeaponId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.shed.set(FarmerHouseShedChoice.NOWEAPON);
            loadNextPart(toNoWeapon);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.SHED) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toTrident.build();
        part += toNoWeapon.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.steal.get() !== FarmerHouseStealChoice.SHED) {
            return;
        }

        toTrident.registerListeners();
        toNoWeapon.registerListeners();

        if (ctrl.shed.has()) {
            return;
        }

        registerTridentListener();
        registerNoWeaponListener();
    };
}

var FarmerHouseShedChoice = {
    TRIDENT: 1,
    NOWEAPON: 2
};

// END scenes\HamletDay2\FarmerHouse\steal\shed\FarmerHouseShed.js

// scenes\HamletDay2\FarmerHouse\steal\FarmerHouseSteal.js

/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseSteal(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var toolshedId = arguments.callee.name + "ButtonOne";
    var insideId = arguments.callee.name + "ButtonTwo";

    var toToolshed = new FarmerHouseShed(ctrl);
    var toInside = new FarmerHouseInside(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You press yourself against the wall, hiding behind a tall cabinet to ensure you won't be spotted while you think of a plan.
                            The woman softly hums to herself while she goes about her business, making sandwiches for dinner.
                            While the hallway you are in somewhat separates the two of you, the only way further into the house is via the kitchen,
                            and you doubt you can sneak by the human without her noticing.
                        [/p]
                        [p]
                            You struggle to come up with an alternate route that won't lead to your detection.
                            You [i]could[/i] leave out the back and go through the front door, or maybe try a different house,
                            but you'd essentially be trading one roll of the dice for another.
                            Who's to say there won't be a guard dog or another housewife or an inquisitive child next door?
                            Heck, you got [b]lucky[/b] this time around.
                            The door was unlocked, and you managed to enter without making much noise.
                            Besides, if the woman is making dinner, you doubt it'll be long before the men of the village return.
                        [/p]
                        [p]
                            A thought bubbles up in your brain.
                            The shed outside.
                            You [i]could[/i] go take a look in there.
                            It'd be [b]safer[/b] than staying in the house,
                            and you might find something useful, although probably nothing [i]too[/i] valuable.
                            But then again, this place doesn't look too promising either.
                            [b]If[/b] you can get anything at all from it without the woman in the kitchen spotting you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildToolshedChoice() {
        var disabled = ctrl.steal.has() ? "disabled" : "";
        var isSelected = ctrl.steal.get() === FarmerHouseStealChoice.SHED;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${toolshedId}" ${disabled}>Go to the shed</button>`;

        var tooltip = global.parser.parse(`At least the toolshed outside won't have a housewife running around in it.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 195px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildInsideChoice() {
        var disabled = ctrl.steal.has() ? "disabled" : "";
        var isSelected = ctrl.steal.get() === FarmerHouseStealChoice.INSIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${insideId}" ${disabled}>Stay indoors</button>`;

        var tooltip = global.parser.parse(`Profit trumps risk. Find a way to steal things without being noticed.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 150px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildToolshedChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildInsideChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + toolshedId), $("#" + insideId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerToolshedListener() {
        $("#" + toolshedId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.steal.set(FarmerHouseStealChoice.SHED);
            loadNextPart(toToolshed);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerInsideListener() {
        $("#" + insideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.steal.set(FarmerHouseStealChoice.INSIDE);
            loadNextPart(toInside);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if(ctrl.arrival.get() !== FarmerHouseArrivalChoice.STEAL) {
            return "";
        }
        
        global.kobold.history.shared.farmerhouse.stole = true;
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toToolshed.build();
        part += toInside.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if(ctrl.arrival.get() !== FarmerHouseArrivalChoice.STEAL) {
            return;
        }
        
        toToolshed.registerListeners();
        toInside.registerListeners();

        if (ctrl.steal.has()) {
            return;
        }

        registerToolshedListener();
        registerInsideListener();
    };
}

var FarmerHouseStealChoice = {
    SHED: 1,
    INSIDE: 2
};

// END scenes\HamletDay2\FarmerHouse\steal\FarmerHouseSteal.js

// scenes\HamletDay2\FarmerHouse\arrival\FarmerHouseArrival.js

/**
 * Arrival section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    
    var confrontId = arguments.callee.name + "ButtonOne";
    var stealId = arguments.callee.name + "ButtonTwo";

    var toConfront = new FarmerHouseConfront(ctrl);
    var toSteal = new FarmerHouseSteal(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You decide some good old thievery would be the best way to get over the loss of your favorite person in the entire world.
                            Your target: one of the dozen or so homes that make up most of the little village.
                            The farmers around these parts aren't [i]rich[/i] by any means, but the more well off amongst them aren't [i]poor[/i] either.
                            They eat what they grow, and with low rural taxes, their hard work manages to net them a decent profit.
                            Hopefully, some of that profit will soon be [i]yours[/i].
                        [/p]
                        [p]
                            You walk down the dirt road that runs through the middle of the hamlet.
                            You've stolen from here before,
                            although you hope this time [i]won't[/i] end in you being chased off by a mob of angry humans with pitchforks and torches.
                            It was late at night, so you don't think anyone'll [i]recognize[/i] you.
                            Not that there [b]is[/b] anyone present [b]to[/b] remember your face.
                            This place is a ghost town.
                            Everyone's probably off working the fields or selling their wares.
                            And that makes [b]your[/b] [i]job[/i] a whole lot easier.
                        [/p]
                        [p]
                            You take one final look around to make sure absolutely [b]no one[/b] is spying on you,
                            before casually wandering off the main road, strolling into a random farmer's back yard as though you own the place.
                            Reaching the rear-end, you turn the corner and lower your posture, dropping into a sneaking stance.
                            You take note of a small shed a bit further out back, one not unlike your [i]home[/i].
                            It doesn't look like there's anyone in it.
                            It's probably used as a storage place for farming tools.
                            Luckily enough, it blocks the view of anyone working the neighbouring potato field,
                            hiding you from the prying eyes of honest, hard-working men.
                        [/p]
                        [p]
                            The house you chose is not nearly the biggest, nor is it the most fancy.
                            In fact, it's one of the less [i]prestigious[/i] buildings in town.
                            You doubt whoever lives here is expecting to get [b]robbed[/b], which means security will be lax.
                            And indeed, testing the back door, you find the shabby, wooden thing completely unlocked!
                            The only well maintained parts of it seem to be the hinges, which function noiselessly and smoothly,
                            the door opening in complete [b]silence[/b], allowing you to slip inside in an equally quiet fashion.
                        [/p]
                        [p]
                            You find yourself in a small hallway leading into the humble household's kitchen.
                            The inside of the house is every bit as rundown as the exterior is.
                            The undecorated wooden walls look nigh-rotten.
                            You spot a [i]nasty[/i] discoloration in a particularly [i]damp[/i] ceiling-corner.
                            [b]Mold[/b].
                            Disgusting.
                            You briefly wonder how people can [i]live[/i] like this,
                            but then you remind yourself of the fact that this place is [b]several times[/b] the size of your own home.
                            Still, [i]your[/i] shack [b]doesn't[/b] have mold growing in it, despite it being in the middle of the forest.
                        [/p]
                        [p]
                            The moment you were about to head deeper into the house, [b]footsteps[/b] resonate from within.
                            You freeze in place.
                            Your heart skips a beat.
                            Someone's home.
                            Flashbacks of your last [i]heist[/i] in this very hamlet shoot through your brain.
                            Your mind is torn between hiding and running, but your body ends up doing neither.
                        [/p]
                        [p]
                            While you're stuck halfway between [i]concealing yourself[/i] and [i]getting the hell out of there[/i],
                            a human saunters into the kitchen.
                            A woman.
                            You mindlessly stare at her as she turns to the small countertop dominating the cramped room,
                            and leisurely begins to slice a loaf of bread using a rusty-looking butter knife.
                            She's too engrossed in her day-to-day activities to notice you.
                            For now, at least.
                        [/p]
                        [p]
                            A mixture of relief and anxiety washes over you.
                            You haven't been caught just yet, but the presence of a housewife [i]complicates[/i] things, to say the least.
                            You need to do [i]something[/i], and you need to do it fast.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildConfrontChoice() {
        var disabled = ctrl.arrival.has() ? "disabled" : "";
        var isSelected = ctrl.arrival.get() === FarmerHouseArrivalChoice.CONFRONT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${confrontId}" ${disabled}>Confront her</button>`;

        var tooltip = global.parser.parse(`Approach the woman. Maybe you can prevent her from screaming somehow.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 150px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildStealChoice() {
        var disabled = ctrl.arrival.has() ? "disabled" : "";
        var isSelected = ctrl.arrival.get() === FarmerHouseArrivalChoice.STEAL;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${stealId}" ${disabled}>Steal stuff</button>`;

        var tooltip = global.parser.parse(`Ignore the housewife. Do what you came here to do. You can run if she sees you. That's what you did last time.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 185px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildConfrontChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildStealChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + confrontId), $("#" + stealId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerConfrontListener() {
        $("#" + confrontId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.arrival.set(FarmerHouseArrivalChoice.CONFRONT);
            loadNextPart(toConfront);
        });
    }
    
    /**
     * Adds a listener to the button.
     */
    function registerStealListener() {
        $("#" + stealId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.arrival.set(FarmerHouseArrivalChoice.STEAL);
            loadNextPart(toSteal);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.kobold.history.shared.farmerhouse.went = true;
        global.ui.applyTheme("hamlet_barn");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toConfront.build();
        part += toSteal.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toConfront.registerListeners();
        toSteal.registerListeners();

        if (ctrl.arrival.has()) {
            return;
        }

        registerConfrontListener();
        registerStealListener();
    };
}

var FarmerHouseArrivalChoice = {
    CONFRONT: 1,
    STEAL: 2
};

// END scenes\HamletDay2\FarmerHouse\arrival\FarmerHouseArrival.js

// scenes\HamletDay2\FarmerHouse\FarmerHouseController.js

/**
 * Controller for the Farmer's House scene.
 * @param {string} sceneId The id of the scene.
 */
function FarmerHouseController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    function postShedChange() {
        if (that.shed.get() === FarmerHouseShedChoice.NOWEAPON) {
            global.kobold.status.physical.decrease(10, "Cut by rusty metal");
            global.kobold.status.mental.decrease(10, "Beaten by garbage");
        } else {
            global.kobold.inventory.equipment.wear(new KoboldTrident());
            global.kobold.status.mental.increase(5, "Crafted weapon");
            global.kobold.status.stamina.decrease(10, "Hard work");
        }
    }

    function postShedChangeNote() {
        if (that.shed.get() === FarmerHouseShedChoice.NOWEAPON) {
            return global.ui.change.buildNotification([
                new Change("Rust-cut", "Health", -10),
                new Change("Beaten by garbage", "Sanity", -10)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Crafted", global.kobold.inventory.createItemWithTooltip(new KoboldTrident(), true), +1),
                new Change("Made something", "Sanity", +5),
                new Change("Hard work", "Stamina", -10)
            ]);
        }
    }

    function followChange() {
        global.kobold.inventory.bag.addConsumable(new Sandwich(), 2);
    }

    function followChangeNote() {
        return global.ui.change.buildNotification([
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new Sandwich(), true), +2)
        ]);
    }

    function cabinetChange() {
        if (that.cabinet.get() === FarmerHouseCabinetChoice.DRAWER) {
            global.kobold.status.mental.decrease(5, "Frustration");
            global.kobold.inventory.bag.addMisc(new CopperRing());
        } else if (that.cabinet.get() === FarmerHouseCabinetChoice.PRY) {
            global.kobold.inventory.coins += 20;
        } else {
            global.kobold.inventory.bag.addConsumable(new CouragePotion());
        }
    }

    function cabinetChangeNote() {
        if (that.cabinet.get() === FarmerHouseCabinetChoice.DRAWER) {
            return global.ui.change.buildNotification([
                new Change("Frustration", "Sanity", -5),
                new Change("Loot", global.kobold.inventory.createItemWithTooltip(new CopperRing(), true), +1)
            ]);
        } else if (that.cabinet.get() === FarmerHouseCabinetChoice.PRY) {
            return global.ui.change.buildNotification([
                new Change("Loot", "Gold", +20)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Loot", global.kobold.inventory.createItemWithTooltip(new CouragePotion(), true), +1)
            ]);
        }
    }

    function wrestleChange() {
        global.kobold.status.physical.decrease(5, "Slapped");
        global.kobold.status.stamina.decrease(15, "Wrestling");
    }

    function wrestleChangeNote() {
        return global.ui.change.buildNotification([
            new Change("Slapped", "Health", -5),
            new Change("Wrestling", "Stamina", -15)
        ]);
    }

    function wrestleLootChange() {
        global.kobold.inventory.addItem(new FancyGlasses());
        global.kobold.status.stamina.decrease(10, "Choking a housewife");
    }

    function wrestleLootChangeNote() {
        return global.ui.change.buildNotification([
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new FancyGlasses(), true), +1),
            new Change("Further wrestling", "Stamina", -10)
        ]);
    }

    function rapeChange() {
        if (that.wrestle.has()) {
            // Wrestle rape change
            if (that.misc.dick()) {
                global.kobold.body.dickVirgin = false;
                global.kobold.status.mental.increase(10, "Sadistic pleasure");
                global.kobold.status.stamina.decrease(15, "Forceful buttsex");
                global.kobold.status.exertion.increase(25, "Forceful buttsex");
            } else {
                global.kobold.status.mental.decrease(5, "Sadism denied");
                global.kobold.status.exertion.increase(5, "Fingered human ass");
            }
        } else if (that.tied.has()) {
            // Tied rape change
            global.kobold.status.mental.increase(10, "Sadistic pleasure");
            global.kobold.status.stamina.decrease(10, "Non-penetrative sex");
            global.kobold.status.exertion.increase(20, "Non-penetrative sex");
        } else {
            // Forced oral change
            if (that.misc.dick()) {
                global.kobold.body.dickVirgin = false;
            }
            global.kobold.status.mental.increase(10, "Sadistic pleasure");
            global.kobold.status.stamina.decrease(10, "Facefucking housewife");
            global.kobold.status.exertion.increase(20, "Facefucking housewife");
        }
    }

    function rapeChangeNote() {
        if (that.wrestle.has()) {
            // Wrestle rape note
            if (that.misc.dick()) {
                return global.ui.change.buildNotification([
                    new Change("Dominated human", "Sanity", +10),
                    new Change("Buttsex", "Stamina", -15),
                    new Change("Buttsex", "Sexual exhaustion", +25, true)
                ]);
            } else {
                return global.ui.change.buildNotification([
                    new Change("Dominated human", "Sanity", +10),
                    new Change("Pleasure denied", "Sanity", -15),
                    new Change("Pleasure denied", "Sexual exhaustion", +5, true)
                ]);
            }
        } else if (that.tied.has()) {
            // Tied rape change note
            return global.ui.change.buildNotification([
                new Change("Dominated human", "Sanity", +10),
                new Change("Non-penetrative sex", "Stamina", -10),
                new Change("Non-penetrative sex", "Sexual exhaustion", +20, true)
            ]);
        } else {
            // Forced oral change note
            return global.ui.change.buildNotification([
                new Change("Dominated human", "Sanity", +10),
                new Change("Facefucking", "Stamina", -10),
                new Change("Facefucking", "Sexual exhaustion", +20, true)
            ]);
        }
    }

    function partialLootChange() {
        global.kobold.inventory.bag.addConsumable(new Bread());
        global.kobold.inventory.bag.addConsumable(new Butter());
        global.kobold.inventory.bag.addConsumable(new Ham());
    }

    function partialLootChangeNote() {
        return global.ui.change.buildNotification([
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new Bread(), true), +1),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new Butter(), true), +1),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new Ham(), true), +1)
        ]);
    }

    function fullLootChange() {
        global.kobold.inventory.addItem(new FancyGlasses());
    }

    function fullLootChangeNote() {
        return global.ui.change.buildNotification([
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new FancyGlasses(), true), +1)
        ]);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.varanarSceneName = "NightTwo";
    this.varanarFolderName = "Varanar";
    
    this.misc = new MiscController(id);
    this.arrival = new ChoiceController(id, "arrival");

    this.steal = new ChoiceController(id, "steal");
    this.shed = new ChoiceController(id, "shed");
    this.postshed = new ChangeController(id, "postshed", postShedChange, postShedChangeNote);
    this.inside = new ChoiceController(id, "inside");
    this.follow = new ChangeController(id, "follow", followChange, followChangeNote);
    this.cabinet = new ChoiceController(id, "cabinet");
    this.cabinetSteal = new ChangeController(id, "cabinetSteal", cabinetChange, cabinetChangeNote);

    this.confront = new ChoiceController(id, "confront");
    this.wrestle = new ChangeController(id, "wrestle", wrestleChange, wrestleChangeNote);
    this.choke = new ChangeController(id, "choke", wrestleLootChange, wrestleLootChangeNote);
    this.rape = new ChangeController(id, "rape", rapeChange, rapeChangeNote);
    this.scared = new ChoiceController(id, "scared");
    this.tied = new ChoiceController(id, "tied");
    this.partialLoot = new ChangeController(id, "partialLoot", partialLootChange, partialLootChangeNote);
    this.fullLoot = new ChangeController(id, "fullLoot", fullLootChange, fullLootChangeNote);


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    this.hasRope = function () {
        global.kobold.history.setIfNotSet(id, "hasRope", global.kobold.inventory.bag.hasMisc(ItemEnum.ROPE));
        return global.kobold.history.getKey(id, "hasRope");
    };

    this.removeRope = function () {
        global.kobold.history.doIfNotSet(id, "removeRope", true, function () {
            global.kobold.inventory.bag.removeMisc(ItemEnum.ROPE);
        });
        return "";
    };

    this.intimidateFail = function () {
        global.kobold.history.doIfNotSet(id, "wrestleFail", true, function () {
            global.kobold.status.stamina.decrease(15, "Ran from farmer");
        });
        return global.ui.change.buildNotification([
            new Change("Ran", "Stamina", -15)
        ]);
    };

    this.threatenFail = function () {
        global.kobold.history.doIfNotSet(id, "wrestleFail", true, function () {
            global.kobold.status.stamina.decrease(15, "Ran from farmer");
        });
        return global.ui.change.buildNotification([
            new Change("Ran", "Stamina", -15)
        ]);
    };

    this.wrestleFail = function () {
        global.kobold.history.doIfNotSet(id, "wrestleFail", true, function () {
            global.kobold.status.physical.decrease(15, "Elbow-bashed");
            global.kobold.status.mental.decrease(10, "Beaten by a woman");
            global.kobold.status.stamina.decrease(15, "Ran from angry mob");
        });
        return global.ui.change.buildNotification([
            new Change("Elbow-bashed", "Health", -15),
            new Change("Beaten by a woman", "Stamina", -15),
            new Change("Ran from angry mob", "Stamina", -15)
        ]);
    };

    this.runChange = function () {
        global.kobold.history.doIfNotSet(id, "shedRun", true, function () {
            global.kobold.status.stamina.decrease(15, "Ran from farmer");
        });
        return global.ui.change.buildNotification([
            new Change("Ran", "Stamina", -15)
        ]);
    };
}

// END scenes\HamletDay2\FarmerHouse\FarmerHouseController.js

// scenes\HamletDay2\FarmerHouse\FarmerHouse.js

/**
 * Farmer's house. Followup scene of the Hamlet.
 */
function FarmerHouse() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var sceneId = "FarmerHouse";
    var sceneName = "Farmer's House";
    var sceneSubfolder = "HamletDay2";
    var ctrl = new FarmerHouseController(sceneId);
    var arrival = new FarmerHouseArrival(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets all the required global variables such as scene id, scene name,
     * scene stack and theme.
     */
    function setGlobals() {
        global.currentScene = that.id;
        global.currentSceneName = that.name;
        global.currentSceneFolder = that.subfolder ? that.subfolder : "";
        global.kobold.history.addToSceneStack(that.id);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = global.ui.buildGenericHeader();
        tab += global.parser.parse(`[t]farmer's house[/t]`);
        tab += arrival.build();
        tab += global.ui.buildGenericFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = sceneId;
    this.name = sceneName;
    this.subfolder = sceneSubfolder;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this scene.
     */
    this.load = function () {
        setGlobals();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        arrival.registerListeners();
        ctrl.autosave();
    };
}

global.sceneLoader.queueRegister(function () {
    var scene = new FarmerHouse();
    global.sceneLoader.registerScene(scene.id, scene);
});

// END scenes\HamletDay2\FarmerHouse\FarmerHouse.js



// END SCENE FarmerHouse

