

// SCENE NightTwo

// scenes\Varanar\NightTwo\confront\crude\appease\eager\NightTwoCrudeEagerContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeEagerContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Crude eager continue.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.crudeEager.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.crudeEager.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\eager\NightTwoCrudeEagerContinue.js

// scenes\Varanar\NightTwo\confront\crude\appease\eager\NightTwoCrudeEager.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeEager(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoCrudeEagerContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Crude eager.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.crudeEager.continued() ? "disabled" : "";
        var isSelected = ctrl.crudeEager.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.crudeEager.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.EAGER) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.crudeEager.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.EAGER) {
            return;
        }

        next.registerListeners();

        if (ctrl.crudeEager.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\eager\NightTwoCrudeEager.js

// scenes\Varanar\NightTwo\confront\crude\appease\reluctant\NightTwoCrudeReluctantContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeReluctantContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You spend a good half hour stuck in between wanting to vomit, and not being able to.
                            While you wisely held your breath while down on your knees,
                            the small bit of sweat still staining your snout carries an unpleasant scent along with it.
                            The smell is probably not [i]that[/i] intense, but being smeared around your nostrils, it taints every breath you take.
                        [/p]
                        [p]
                            As you finally manage to compose yourself a little, the sun starts to rise above the rooftops.
                            You'd rather have spent the remainder of the night in bed, as opposed to on your knees and bent over trying to throw up, but alas.
                            You head back towards the town square, where your [i]urban adventure[/i] started.
                            Your first night in Varanar is over, and your first day is just about to begin.
                            If there's anything you've learned, it's that you need to keep your guard up at all times.
                            The next group of humans to corner you might not be sated with a small peck on their buttcheeks.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.crudeReluctant.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.crudeReluctant.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\reluctant\NightTwoCrudeReluctantContinue.js

// scenes\Varanar\NightTwo\confront\crude\appease\reluctant\NightTwoCrudeReluctant.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeReluctant(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoCrudeReluctantContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.buttKiss = true;
        var toReturn = `[p]
                            You try your hardest to look for a way out of this mess you've created, but your brain draws a blank.
                            It seems as though you've talked yourself into a corner.
                            You can simply [i]refuse[/i] to play along, but judging by the group's intense behavior,
                            you can make a pretty decent guess that things [b]won't[/b] turn out nicely if you back out now.
                            The best course of action, for now, would be to just [i]play along[/i].
                            If you do ask they ask of you, this'll all be over soon...you hope.
                        [/p]
                        [p]
                            "[color=bred]Well? Don't keep me waiting.[/color]" the half-naked human's voice snaps you back to reality.
                            He bends over slightly, his unsavory ass jutting out towards you.
                            You can feel your stomach churning.
                            It's just so [b]hairy[/b] and [b]disgusting[/b].
                            You'd rather kiss a [b]horse[/b]'s rear end.
                            You [b]really[/b] don't want to do this, but the men have left you with little choice.
                        [/p]
                        [p]
                            You sigh loudly and take a deep breath of cool, late-night air.
                            Your lungs filled with clean air, you drop to your knees, and hold your breath.
                            Closing your eyes, you lean forwards, and plant a single small kiss on the human's right buttcheek,
                            your snout pressing against his damp skin.
                            As you pull back from the simple peck, the group bursts out into laughter.
                        [/p]
                        [p]
                            You are completely disgusted.
                            You wipe off the sweat clinging to the front of your muzzle.
                            Rising to your feet once more, an intense feeling of [b]sickness[/b] washes over you.
                            You heave, retch and gag.
                            It's like you're on the verge of vomiting, without being able to hurl and get it over with already.
                            All the while, the humans are taunting you.
                            "[color=green]The lizard actually did it![/color]"
                            "[color=brown]You kiss your mother with that mouth?![/color]"
                            "[color=purple]Ass-kissing kobold![/color]"
                            "[color=dred]Bet it tastes better than what you're used to eating![/color]"
                        [/p]
                        [p]
                            The exposed man pulls up his pants.
                            While the crowd slowly simmers down, he addresses his comrade.
                            "[color=bred]Told you they were good at something, Jacoby.
                            You owe us a round at the tavern![/color]"
                            The group bursts into cheer.
                            Pushing you aside, they break their encircling formation, and wander off towards the brothel district you came from,
                            leaving you alone in the street, struggling to overcome your sheer revulsion.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.crudeReluctant.continued() ? "disabled" : "";
        var isSelected = ctrl.crudeReluctant.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.crudeReluctant.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.RELUCTANT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.crudeReluctant.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.RELUCTANT) {
            return;
        }

        next.registerListeners();

        if (ctrl.crudeReluctant.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\reluctant\NightTwoCrudeReluctant.js

// scenes\Varanar\NightTwo\confront\crude\appease\unfit\NightTwoCrudeUnfitContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeUnfitContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the humans leaving, you decide that you, too, should probably get moving, lest they change their minds.
                            You begin to make your way back towards the city square, where your nighttime adventure began.
                            While you walk, you ponder the events that just transpired.
                            Were it not for your clever way of handling the group of drunkards, things could have gone [b]very[/b] bad for you.
                            You're glad you made it out unharmed, with your dignity mostly intact.
                        [/p]
                        [p]
                            As you near the marketplace, the sun slowly rises above the rooftops.
                            You guess you won't be getting any sleep tonight.
                            Oh well.
                            Taking a deep breath of fresh morning air, you steel yourself for your first day in Varanar.
                            You'll need to be on your guard if it's to be anything like your first [i]night[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.crudeUnfit.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.crudeUnfit.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\unfit\NightTwoCrudeUnfitContinue.js

// scenes\Varanar\NightTwo\confront\crude\appease\unfit\NightTwoCrudeUnfit.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeUnfit(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoCrudeUnfitContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You say you are [b]flattered[/b] that the men would even [b]consider[/b] letting a [b]lowly[/b] kobold as yourself kiss their behinds,
                            but you are afraid that you simply [b]can't[/b].
                            You see, you have a [b]rare[/b] and [b]dangerous[/b] disease called [b]tongorrhea[/b],
                            which pretty much means that if your tongue so much as [b]comes close[/b] to their skin, they'll get [b]infected[/b],
                            and their [b]dicks[/b] will [b]shrivel[/b] and [b]fall off[/b] in a couple of months.
                            Not to mention the horrible [b]itching[/b].
                            Their wives won't be too pleased if they go home with a [b]nasty[/b] kobold disease.
                            Surely they can find a non-infected member of your kind to settle this [i]dispute[/i], right?
                        [/p]
                        [p]
                            The group bursts into disarray.
                            "[color=purple]Mangy kobold![/color]"
                            "[color=bred]Jacoby picked this one because he knew he was going to lose the bet![/color]"
                            "[color=green]Hey now, [b]you're[/b] the one with your bare ass close to the diseased vermin's snout![/color]"
                            Realizing his pants are still down, the exposed human in front of you quickly re-dresses, before turning around to face you once more.
                            "[color=bred]Lizard.[/color]" he speaks, drawing your attention.
                            "[color=bred]Thanks for telling us.
                            Wouldn't want to catch any infections from your kind.[/color]"
                            You wave away his thanks, and mention that you're glad things worked out for the better, for [b]both[/b] of you.
                        [/p]
                        [p]
                            Their interest in you vanishing, but their argument still going strong, the drunkards slowly break their encirclement.
                            "[color=brown]Let's go find another one![/color]"
                            "[color=purple]At this hour? Where?[/color]"
                            "[color=dred]To the lizard district![/color]"
                            Like they've completely forgotten you even exist,
                            the group begins to head for the kobold district, leaving you all alone in the middle of the street.
                            Just the way you like it.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.crudeUnfit.continued() ? "disabled" : "";
        var isSelected = ctrl.crudeUnfit.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.crudeUnfit.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.UNFIT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.crudeUnfit.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoCrudeAppeaseChoice.UNFIT) {
            return;
        }

        next.registerListeners();

        if (ctrl.crudeUnfit.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\appease\unfit\NightTwoCrudeUnfit.js

// scenes\Varanar\NightTwo\confront\crude\appease\NightTwoCrudeAppease.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeAppease(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reluctantId = arguments.callee.name + "ButtonOne";
    var eagerId = arguments.callee.name + "ButtonTwo";
    var unfitId = arguments.callee.name + "ButtonThree";

    var toReluctant = new NightTwoCrudeReluctant(ctrl);
    var toEager = new NightTwoCrudeEager(ctrl);
    var toUnfit = new NightTwoCrudeUnfit(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            No matter what you say, they'll find a way to turn it against you,
                            but if you don't play along, you doubt things would end well.
                            If you just give them the laugh they're after, you're certain they'll leave you alone.
                            Besides, if you show the group that you can take a [i]joke[/i] just as well as any human can,
                            perhaps they'll get bored and find a different target to pick on.
                        [/p]
                        [p]
                            Forsaking your pride, you half-heartedly stammer out that yes, kobolds are [i]indeed[/i] good at [b]kissing ass[/b].
                            You can barely even get those final two words past your lips, your mind screaming at you [b]not[/b] to put yourself down like this.
                            But, in a way, the statement isn't even [i]wrong[/i].
                            While most kobolds are [i]scoundrels[/i] of some form or another, the ones that aren't tend to be [b]suck-ups[/b],
                            relying on innate [b]cuteness[/b] and [b]flattering words[/b] in the hopes of being tossed a morsel or a coin every now and again.
                            In that sense: yes. Your kind [i]is[/i] good at [b]kissing ass[/b].
                        [/p]
                        [p]
                            The entire group bursts out into laughter at the sight of a kobold humiliating itself.
                            The drunken men begin to taunt you anew, although the mood is slightly more cheerful now than it was before,
                            their insults clearly less meant to [b]hurt[/b] you this time around.
                            It seems as though your gamble has paid off.
                        [/p]
                        [p]
                            One of the men still seems displeased.
                            As the posse begins to quiet down, he raises his voice:
                            "[color=green]Kobolds are [b]liars[/b].
                            Who knows if this one is telling the truth?!
                            Make'em [b]prove[/b] it![/color]"
                            At first, the group pays the dissenter no mind.
                            Slowly, but steadily, however, cogs begin to turn inside of their drunken heads.
                            As they come to a communal conclusion, some of the men begin to voice their response in unison:
                            "[color=purple]Yeah. Make'em prove it![/color]"
                            "[color=brown]The kobold lies![/color]"
                            "[color=dred]Put on a show![/color]"
                        [/p]
                        [p]
                            The man in front of you realizes he's losing the crowd's favour.
                            A grin forms atop his visage.
                            "[color=bred]Jacoby's right.[/color]" he speaks, his hands travelling down to his waist, resting atop his belt buckle.
                            "[color=bred]You lizards are a bunch of liars.[/color]"
                            He unfastens the clasp sealing the leather cord, his pants visible loosening.
                            "[color=bred]And we don't take kindly to liars around these parts.[/color]"
                            That being said, the drunkard turns around, facing away from you before promptly lowering his pants and his underwear,
                            his sweaty and hairy, undoubtedly [b]masculine[/b] rear end coming into view.
                            "[color=bred]Let's see if you were telling the truth or not.[/color]"
                        [/p]
                        [p]
                            The entire group seems riled up.
                            They [b]really[/b] want this to happen, and judging by the exposed human's tone,
                            you're in too deep to back out now.
                            Unless you can present a strong case for yourself, you fear this [b]is[/b] happening.
                            That doesn't mean you need to like it, though.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildReluctantChoice() {
        var disabled = ctrl.appease.has() ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoCrudeAppeaseChoice.RELUCTANT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${reluctantId}" ${disabled}>Reluctant</button>`;

        var tooltip = global.parser.parse(`You don't really want to humiliate yourself for these guys' entertainment, but you don't have much of a choice.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildEagerChoice() {
        var disabled = ctrl.appease.has() ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoCrudeAppeaseChoice.EAGER;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${eagerId}" ${disabled}>Eager</button>`;

        var tooltip = global.parser.parse(`Give the men a show they'll never forget.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildUnfitChoice() {
        var test = ctrl.misc.intelligence() > 5 && ctrl.misc.charisma() > 5;
        var disabled = (ctrl.appease.has() || !test) ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoCrudeAppeaseChoice.UNFIT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${unfitId}" ${disabled}>Unfit</button>`;

        var tooltip = global.parser.parse(`Surely these men can find a more fitting kobold to abuse, right?`);
        if (!test) {
            tooltip = `Requires 6 intelligence and 6 charisma.`;
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You are: </div>`;
        choice += buildReluctantChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildUnfitChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + reluctantId), $("#" + eagerId), $("#" + unfitId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerReluctantListener() {
        $("#" + reluctantId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoCrudeAppeaseChoice.RELUCTANT);
            loadNextPart(toReluctant);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerEagerListener() {
        $("#" + eagerId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoCrudeAppeaseChoice.EAGER);
            loadNextPart(toEager);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerUnfitListener() {
        $("#" + unfitId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoCrudeAppeaseChoice.UNFIT);
            loadNextPart(toUnfit);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.lewdCrude.get() !== NightTwoCrudeChoice.APPEASE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toReluctant.build();
        part += toEager.build();
        part += toUnfit.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.lewdCrude.get() !== NightTwoCrudeChoice.APPEASE) {
            return;
        }

        toReluctant.registerListeners();
        toEager.registerListeners();
        toUnfit.registerListeners();

        if (ctrl.appease.has()) {
            return;
        }

        registerReluctantListener();
        registerEagerListener();
        registerUnfitListener();
    };
}

var NightTwoCrudeAppeaseChoice = {
    RELUCTANT: 1,
    EAGER: 2,
    UNFIT: 3
};

// END scenes\Varanar\NightTwo\confront\crude\appease\NightTwoCrudeAppease.js

// scenes\Varanar\NightTwo\confront\crude\rude\fight\lose\NightTwoCrudeLose.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeLose(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    var idsToRemove = [
        "NightTwoCrudeLose",
        "NightTwoCrudeFight",
        "NightTwoCrudeRude",
        "NightTwoCrude",
        "NightTwoConfront",
        "NightTwoRunIn",
        "NightTwoStage"
    ];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            No.
                            You aren't going to let them do this.
                            You can't and you [b]won't[/b] get [b]anywhere[/b] near that filthy human's disgusting rear, let alone pressing your lips against it!
                            [b]Over your dead body.[/b]
                            They may be stronger than you, they may outnumber you, they may have you in an arm-grip,
                            but all of that just means they [b]won't[/b] be [i]expecting[/i] you to go on the offensive.
                        [/p]
                        [p]
                            They think they have you scared shitless, that you're just another kobold for them to [b]bully[/b] and [b]abuse[/b],
                            that you won't put up a fight simply because this is [b]their[/b] town.
                            But you aren't any ordinary run-of-the-mill lizard.
                            The other members of your kind may be content being thieves, or bandits or prostitutes or beggars, but not [b]you[/b].
                            You may be a thief right now, but you [b]aspire[/b] to be a [b]knight[/b].
                            And knights don't back down in the face of adversity.
                            They charge head-forward into danger.
                            Just like you're about to do.
                        [/p]`;

        if (weapon) {
            toReturn += `[p]
                            While one of your arms is being restrained, you reach for your ${global.kobold.inventory.createItemWithTooltip(weapon)} with the other.
                            With your head being forced ever-closer to the drunkard's butt, you decide to strike before your snout brushes against his body hair.
                            Swiftly and silently, you [b]surge[/b] your weapon forwards, intent on striking the oaf's rear,
                            yet before your strike manages to connect, your free arm is grabbed and painfully twisted, much like the other.
                            With a pained yelp, you drop your ${global.kobold.inventory.createItemWithTooltip(weapon)} to the ground,
                            where it is quickly picked up by one of the humans, who promptly [b]bashes[/b] you on the head with it.
                        [/p]`;
        } else {
            toReturn += `[p]
                            While one of your arms is being restrained, you lash out with the other, intent on striking the oaf's rear,
                            yet before your strike manages to connect, your free arm is grabbed and painfully twisted, much like the other.
                            You let out a pained yelp.
                            Your attempted aggression is soon punished, one of the humans brutally [b]bashing[/b] the top of your head with his balled up hand.
                        [/p]`;
        }

        toReturn += `   ${global.ui.condFail(weapon ? `${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence below 4` : `No weapon`)}
                        [p]
                            Your vision fades to black.
                            You lose consciousness.
                            You dream about a ceremony.
                            One in your honor.
                            You are in a castle.
                            There's princes and princesses, barons and dukes, royals from all corners of the world.
                            You are kneeling.
                            The Dragon King himself puts a gilded blade atop your shoulder.
                            Then suddenly, the dream turns into a nightmare, the castle replaced by a terrible, all-consuming void that sucks you into its core.
                            Once the void has consumed all, you are left with nothing, not even yourself.
                            The nightmare ends, and so do you.
                            In the real world, the men kicked you to death for passing out instead of doing their bidding.
                            Luckily, you were not awake to witness the horrible pain inflicted upon you.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Nightmare ending.
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400, function () {
            var main = $("#main");
            main.scrollTop(main[0].scrollHeight - main.height() - 5); // Fix scrolling
        });
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.misc.unsetAll();
            ctrl.rude.undo();
            ctrl.rude.unset();
            ctrl.lewdCrude.undo();
            ctrl.lewdCrude.unset();
            ctrl.runin.unset();
            ctrl.stage.unset();
            for (var i = 0; i < idsToRemove.length; i++) {
                $("#" + idsToRemove[i]).remove();
            }
            loadNextPart(new NightTwoStage(ctrl));
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        registerContinueListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\fight\lose\NightTwoCrudeLose.js

// scenes\Varanar\NightTwo\confront\crude\rude\fight\win\NightTwoCrudeWinContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeWinContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            As your adrenaline rush fades, the harsh reality of things comes crashing down on you.
                            You just [b]attacked[/b] and seriously [b]hurt[/b] a bunch of humans.
                            They might have been bothering you, but the guards won't care.
                            To them, you're a stupid, unruly lizard that crossed the line.
                            The men you assaulted had rights.
                            You don't.
                        [/p]
                        [p]
                            You make the smart decision to vacate the scene of the crime.
                            After jogging a few streets over, you lower your pace to a brisk walk.
                            You think you're in the clear.
                            If the famous guards of Varanar were called, you wouldn't have made it this far.
                            You'd have been captured or killed before you even left the street.
                            Still, you probably shouldn't linger in this particular area for too long.
                        [/p]
                        [p]
                            As you begin to make your way back to the town square, where your late-night adventure began, the sun slowly begins to rise above the rooftops.
                            It seems as though you've managed to retain your dignity throughout the night, but at what cost?
                            You don't want to think about it.
                            It's too late, or rather [i]too early[/i] to go to bed.
                            Who needs sleep?
                            You'd just have bad dreams about the things you did anyway.
                            You know you acted in self-defence, but your conscience doesn't agree. 
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.fightWin.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.fightWin.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\fight\win\NightTwoCrudeWinContinue.js

// scenes\Varanar\NightTwo\confront\crude\rude\fight\win\NightTwoCrudeWin.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeWin(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoCrudeWinContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        global.kobold.history.shared.varanar.nightTwo.fought = true;
        var toReturn = `[p]
                            No.
                            You aren't going to let them do this.
                            You can't and you [b]won't[/b] get [b]anywhere[/b] near that filthy human's disgusting rear, let alone pressing your lips against it!
                            [b]Over your dead body.[/b]
                            They may be stronger than you, they may outnumber you, they may have you in an arm-grip,
                            but all of that just means they [b]won't[/b] be [i]expecting[/i] you to go on the offensive.
                        [/p]
                        [p]
                            They think they have you scared shitless, that you're just another kobold for them to [b]bully[/b] and [b]abuse[/b],
                            that you won't put up a fight simply because this is [b]their[/b] town.
                            But you aren't any ordinary run-of-the-mill lizard.
                            The other members of your kind may be content being thieves, or bandits or prostitutes or beggars, but not [b]you[/b].
                            You may be a thief right now, but you [b]aspire[/b] to be a [b]knight[/b].
                            And knights don't back down in the face of adversity.
                            They charge head-forward into danger.
                            Just like you're about to do.
                        [/p]
                        [p]
                            While one of your arms is being restrained, you reach for your ${global.kobold.inventory.createItemWithTooltip(weapon)} with the other.
                            With your head being forced ever-closer to the drunkard's butt, you decide to strike before your snout brushes against his body hair.
                            Swiftly and silently, you [b]surge[/b] your weapon forwards,
                            shoving the pointy-end directly into the ruffian's exposed ass, stabbing him in his buttcheek.
                            The man screams in pain.
                            His companions laugh out loud, thinking you bit him.
                            Their laughter comes to a halt, however, as the group slowly realizes what has transpired.
                        [/p]
                        [p]
                            You pull your weapon from the pantless delinquent's rear, and in one swift move, you bring it to bear on the person holding your arm.
                            Blindly stabbing backwards, you land a seemingly solid strike on the human's abdomen.
                            He loses his hold on your arm nigh-instantly as your ${global.kobold.inventory.createItemWithTooltip(weapon)} sinks into his gut.
                            The man whose butt you perforated tumbles to the ground.
                            Blood oozes liberally from his mutilated ass.
                            The guy behind you stumbles backwards, clutching his wounded belly.
                            Communal panic settles in.
                            [i]Perfect[/i].
                        [/p]
                        ${global.ui.condSucc(`${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence greater than 3`)}
                        [p]
                            Your freedom regained, you drop into a combat stance and begin to wildly swing your weapon about in all directions,
                            attempting and succeeding in pushing back the humans surrounding you.
                            Their encircling formation broken, one of their comrades downed, and the other vomiting blood, the frightened men turn tail and run away.
                            By growing hysteria, fostered by your ballsy outburst, you have managed to drive the overwhelming odds to flee,
                            causing an entire group of intoxicated humans to sprint away from a single kobold with a weapon,
                            not even bothering to pick up their wounded in the process.
                        [/p]
                        [p]
                            As the dust settles, the only people left in the street are you, and the guy you stabbed in the butt.
                            Only one of you is standing, and it isn't him.
                            You take a few deep breaths, slowly calming down from the adrenaline rush that took over you.
                            You did it.
                            You scared them off.
                            You showed them that they can't just push kobolds around without any repercussions,
                            that you and your kind alike [b]will[/b] fight back if cornered.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.fightWin.continued() ? "disabled" : "";
        var isSelected = ctrl.fightWin.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.fightWin.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.fightWin.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.fightWin.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\fight\win\NightTwoCrudeWin.js

// scenes\Varanar\NightTwo\confront\crude\rude\fight\NightTwoCrudeFight.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeFight(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoCrudeLose(ctrl);
    var toSuccess = new NightTwoCrudeWin(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var weapon = ctrl.misc.weapon();
        return weapon && weapon.offence() >= 3 && weapon.stab();
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.rude.get() !== NightTwoCrudeRudeChoice.FIGHT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.rude.get() !== NightTwoCrudeRudeChoice.FIGHT) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\fight\NightTwoCrudeFight.js

// scenes\Varanar\NightTwo\confront\crude\rude\struggle\NightTwoCrudeStruggleContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeStruggleContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You are absolutely [b]disgusted[/b].
                            Sickened, not only by what you were made to do, but by yourself, and by society as a whole.
                            [b]Nobody[/b] should be forced to do something so [b]foul[/b] and [b]revolting[/b] just to settle some stupid drunken quarrel.
                            You cry for what seems like an eternity.
                            The ground is as cold as the hearts of the humans that humiliated you.
                        [/p]
                        [p]
                            The sun slowly rises above the building-rooftops, and as you feel its first warming rays shine atop your back,
                            you pick yourself off the ground, along with the broken pieces of your shattered self-image.
                            While staying on the ground until you [b]die[/b] sounds like a good plan right now,
                            you decide it'd be best if you made your way back to the city square.
                            You won't be getting any sleep tonight.
                            At least you learned a valuable lesson: humans are [b]not[/b] nice people.
                            They [b]hate[/b] your kind, and you shouldn't trust them any further than you can throw them,
                            which isn't very far at all.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.crudeStruggle.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.crudeStruggle.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\struggle\NightTwoCrudeStruggleContinue.js

// scenes\Varanar\NightTwo\confront\crude\rude\struggle\NightTwoCrudeStruggle.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeStruggle(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoCrudeStruggleContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.forced = true;
        global.kobold.history.shared.varanar.nightTwo.buttKiss = true;
        var toReturn = `[p]
                            You wriggle around, attempting to escape the human's grasp,
                            but all he has to do is apply some extra [b]force[/b] to halt your squirming.
                            Your arm feels like it's about to [b]break[/b].
                            The pain almost makes you burst into tears, yet you manage to save face for just a little while longer.
                            Your mind [b]screams[/b] for you to beg him to let go, but you don't want to humiliate yourself any further.
                        [/p]
                        [p]
                            Your head gets gradually pushed lower and lower, closer to the hulking brute's bare behind.
                            "[color=bred]Hurry up back there, I haven't got all night.[/color]" he complains,
                            slightly bending over to make his ass just out further, its unsavory fragrance reaching your nose.
                            "[color=purple]You heard him, lizard.
                            Here you go.[/color]" the man holding your arm speaks.
                            Then, the slow, constant lowering is replaced by a single fierce [b]shove[/b],
                            your face forcibly pressed against the exposed delinquent's rear.
                        [/p]
                        [p]
                            Whether by accident, or just as planned, your muzzle ends up lodged directly in between the drunkard's buttcheeks,
                            warm, sweaty, hairy flesh enveloping your scales from all sides.
                            Your lips end up pressing directly against his [i]least sanitary[/i] part.
                            You make the mistake of gasping in surprise,
                            inadvertently taking in a mouthful and a noseful of the foul, concentrated stench of the human's butthole,
                            an intense feeling of [b]sickness[/b] building inside of you.
                        [/p]
                        [p]
                            The group goes wild.
                            Half of them burst out into laughter, the other half start to ridicule you,
                            hurling every insult they know your way in an attempt to make your life even more [b]miserable[/b] than it already is.
                            Your free arm gets grabbed, and held in place much like the other.
                            Your tail gets harshly yanked as a prank.
                            You don't know what brings more tears to your eyes:
                            the foul act you're forced to perform, or the oh-so-casual cruelty and complete disregard of your well-being.
                            You're nothing but an [b]animal[/b] to these people.
                        [/p]
                        [p]
                            You struggle with all of your might, yet the grip on your arms and on your head is unyielding.
                            You thought they just wanted you to give a peck on his buttcheek, not [b]this[/b]!
                            If you knew they wanted you to kiss his [b]asshole[/b], you'd have [b]fought them[/b] with a bit more [b]conviction[/b]!
                            But it's too late for that now.
                            You're completely trapped.
                            And as the humans so kindly remind you:
                            "[color=brown]You ain't gettin' out 'till you kiss it![/color]"
                        [/p]
                        [p]
                            Crying out loud, you swallow what little remains of your pride.
                            Puckering your lips, you plant a single, small kiss atop the half-naked human's hole,
                            and begin to [b]retch[/b] and [b]gag[/b] immediately afterwards.
                            "[color=bred]The lizard did it![/color]" the drunkard exclaims, and immediately you are yanked from his rear,
                            tossed to the ground like a used piece of toilet paper.
                            The moment your body hits the cold, hard dirt, you curl up into a ball and begin to cry your eyes out.
                        [/p]
                        [p]
                            Some of the humans taunt and humiliate you, but luckily,
                            most of them are more interested in the fact that the [i]bet[/i] between Jacoby and Jehova has been settled.
                            You tune out for a little while, not interested in what they, or anyone, for that matter has to say.
                            After a bit of rowdy discussion, the group sets off towards the brothel district, leaving you to wallow in self-pity.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.crudeStruggle.continued() ? "disabled" : "";
        var isSelected = ctrl.crudeStruggle.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.crudeStruggle.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.rude.get() !== NightTwoCrudeRudeChoice.STRUGGLE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.crudeStruggle.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.rude.get() !== NightTwoCrudeRudeChoice.STRUGGLE) {
            return;
        }

        next.registerListeners();

        if (ctrl.crudeStruggle.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\crude\rude\struggle\NightTwoCrudeStruggle.js

// scenes\Varanar\NightTwo\confront\crude\rude\NightTwoCrudeRude.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrudeRude(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var fightId = arguments.callee.name + "ButtonOne";
    var struggleId = arguments.callee.name + "ButtonTwo";

    var toFight = new NightTwoCrudeFight(ctrl);
    var toStruggle = new NightTwoCrudeStruggle(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            No matter what you say, they'll find a way to turn it against you.
                            Might as well not even [b]bother[/b] to play along.
                            Besides, if you show the group that you don't take kindly to them messing with you,
                            perhaps they'll leave you alone and find a different target to pick on.
                            Heck, given their [i]rough[/i] appearance, they might even [i]appreciate[/i] you not being a [b]total pushover[/b].
                        [/p]
                        [p]
                            Raising your voice, you loudly tell the man to [b]fuck off[/b].
                            You're not solving any disputes for a bunch of rude, inbred [b]humans[/b]!
                            Kobolds are ass kissers, huh?!
                            That's funny, coming from a race of [b]bootlickers[/b] that'd [b]suck their lord off[/b] for an extra acre of land,
                            or some food to make it through the winter!
                            If anything, [b]humans[/b] are the ass kissers, and you can't [b]wait[/b] for your kind to replace theirs as the dominant party around here!
                        [/p]
                        [p]
                            Wew.
                            You, uhh, didn't mean to go on a [i]rant[/i] like that.
                            You just wanted to throw some cuss words around to impress the drunkards, so they'd leave you alone.
                            A lifetime of [b]taking shit[/b] from their kind left you with a [b]lot[/b] of pent up frustration,
                            and you couldn't really stop yourself after you started venting some of that budding resentment.
                            Alas, the deed is done.
                            There's no going back now.
                            All you can do is [b]pray[/b] that the humans'll take your little [i]tirade[/i] as a [i]joke[/i].
                        [/p]
                        [p]
                            Sadly, that doesn't seem to be the case.
                            The group goes eerily quiet, angry eyes glaring down at you from all directions.
                            "[color=bred]I don't think you understand where you are.[/color]" the man that approached you breaks the silence,
                            his tone far more [b]hostile[/b] than before.
                            His hands travel down to his waist, and he deftly unbuckles the heavy belt keeping his pants up.
                            "[color=bred]This is a [b]human[/b] city, [b]lizard[/b].[/color]"
                        [/p]
                        [p]
                            From behind, one of the men roughly grabs your arm, cruelly and painfully twisting it.
                            You barely refrain from voicing your anguish, still attempting to keep up the [i]tough[/i] facade.
                            "[color=bred]And in Varanar, when [b]humans[/b] ask you a question, you [b]answer them[/b].[/color]"
                            Your head is pushed down, [b]forcing[/b] you to bend over forwards.
                            "[color=bred]But since you don't want to [b]talk[/b] to us...[/color]"
                            The hulking brute turns around as he speaks, lowering his trousers and his underwear simultaneously,
                            his sweat-ridden, hairy behind coming into view, buttcheeks mere inches away from your face.
                            "[color=bred]...you're just going to have to [b]show[/b] us how good you are at [b]kissing ass[/b].[/color]"
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFightChoice() {
        var disabled = ctrl.rude.has() ? "disabled" : "";
        var isSelected = ctrl.rude.get() === NightTwoCrudeRudeChoice.FIGHT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fightId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`You're not going to go down without a fight. Come out swinging.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildStruggleChoice() {
        var disabled = ctrl.rude.has() ? "disabled" : "";
        var isSelected = ctrl.rude.get() === NightTwoCrudeRudeChoice.STRUGGLE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${struggleId}" ${disabled}>Struggle</button>`;

        var tooltip = global.parser.parse(`Struggle against the humans' grasp, but don't outright attack them.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildFightChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildStruggleChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + fightId), $("#" + struggleId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFightListener() {
        $("#" + fightId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rude.set(NightTwoCrudeRudeChoice.FIGHT);
            loadNextPart(toFight);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerStruggleListener() {
        $("#" + struggleId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rude.set(NightTwoCrudeRudeChoice.STRUGGLE);
            loadNextPart(toStruggle);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.lewdCrude.get() !== NightTwoCrudeChoice.RUDE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.rude.change();
        part += buildChoice();
        part += `</div>`;
        part += toFight.build();
        part += toStruggle.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.lewdCrude.get() !== NightTwoCrudeChoice.RUDE) {
            return;
        }

        toFight.registerListeners();
        toStruggle.registerListeners();

        if (ctrl.rude.has()) {
            return;
        }

        registerFightListener();
        registerStruggleListener();
    };
}

var NightTwoCrudeRudeChoice = {
    FIGHT: 1,
    STRUGGLE: 2
};

// END scenes\Varanar\NightTwo\confront\crude\rude\NightTwoCrudeRude.js

// scenes\Varanar\NightTwo\confront\crude\NightTwoCrude.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoCrude(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var appeaseId = arguments.callee.name + "ButtonOne";
    var rudeId = arguments.callee.name + "ButtonTwo";

    var toAppease = new NightTwoCrudeAppease(ctrl);
    var toRude = new NightTwoCrudeRude(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You decide [b]not[/b] to empower the group's belligerent behavior.
                            Instead of giving them attention, or even the [b]satisfaction[/b] of chasing after you,
                            you're going to walk right past the hoodlums without so much as [b]looking[/b] at them.
                            Tilting your head the other way,
                            staring at the front-faces of the buildings you pass as though you're inspecting them,
                            you begin to stride down the road.
                            As you approach, the group appears to simmer down a bit, your tactic of not paying them any mind seemingly effective.
                        [/p]
                        [p]
                            Looking away to the side, however, quickly proves to have been a mistake.
                            The moment you cross the delinquents' party, you suddenly [b]bump[/b] into something,
                            the unexpected collision almost making you fall back onto your butt.
                            Your gaze snapping forwards, you become aware that the [i]thing[/i] you walked into is none other than one of the men.
                            Taking a few steps backwards to regain your balance, you find your rear end pressing against something as well.
                            Another human.
                            It looks like, while you were [i]pretending[/i] not to notice them, the drunkards somehow managed to surround you!
                        [/p]
                        [p]
                            The man you bumped into last pushes you forwards, into the middle of the little [i]circle[/i] they've formed.
                            While they were quiet enough to sneak up on you, now that they have you surrounded, the group soon returns to its rowdy nature.
                            Cruel jokes and a myriad of insults are hurled your way.
                            "[color=purple]Who left the cage open?![/color]"
                            "[color=dred]It lost its collar![/color]"
                            "[color=brown]My wife cooked two of you for dinner last night![/color]"
                            Growing concerned, you desperately look around for a way out, yet they have you [b]completely[/b] encircled.
                            Any time you take so much as a step towards them, you get roughly [b]shoved[/b] back into the very centre of the group.
                        [/p]
                        [p]
                            One of the humans breaks formation, stepping towards you with a swaying gait that [b]screams[/b] drunkenness.
                            You fear for the worst, but instead of the [b]brutal pummeling[/b] you half-expected,
                            the ruffian comes to a halt a few feet in front of you.
                            It seems he is sober enough to slur out a few coherent words:
                            "[color=bred]Me and...[/color]" *hic* "[color=bred]...my buddy Jacoby back there was arguin' 'bout you [i]lizards[/i] earlier.[/color]"
                            The brute takes a lengthy pause to consider his next words, his inebriated brain clearly working overtime.
                            "[color=bred]He says your kind is good for [b]nothing[/b].
                            I disagree.
                            You kobolds are good at [b]kissing ass[/b], especially when it comes to [b]civilized people[/b] like [b]us[/b].
                            I think you kiss ass better than [b]any[/b] other species!
                            So, which one of us is right?[/color]"
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildAppeaseChoice() {
        var disabled = ctrl.lewdCrude.has() ? "disabled" : "";
        var isSelected = ctrl.lewdCrude.get() === NightTwoCrudeChoice.APPEASE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${appeaseId}" ${disabled}>Appeasing</button>`;

        var tooltip = global.parser.parse(`Attempt to appease the man. If you play along, the group'll have no reason to hurt you, right?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 230px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRudeChoice() {
        var disabled = ctrl.lewdCrude.has() ? "disabled" : "";
        var isSelected = ctrl.lewdCrude.get() === NightTwoCrudeChoice.RUDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${rudeId}" ${disabled}>Rude</button>`;

        var tooltip = global.parser.parse(`Don't give them an inch. Refuse to play along. Give them a piece of your mind, and send them on their way.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You are: </div>`;
        choice += buildRudeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAppeaseChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + appeaseId), $("#" + rudeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerAppeaseListener() {
        $("#" + appeaseId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdCrude.set(NightTwoCrudeChoice.APPEASE);
            loadNextPart(toAppease);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerRudeListener() {
        $("#" + rudeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdCrude.set(NightTwoCrudeChoice.RUDE);
            loadNextPart(toRude);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdCrude.change();
        part += buildChoice();
        part += `</div>`;
        part += toAppease.build();
        part += toRude.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toAppease.registerListeners();
        toRude.registerListeners();

        if (ctrl.lewdCrude.has()) {
            return;
        }

        registerAppeaseListener();
        registerRudeListener();
    };
}

var NightTwoCrudeChoice = {
    RUDE: 1,
    APPEASE: 2
};

// END scenes\Varanar\NightTwo\confront\crude\NightTwoCrude.js

// scenes\Varanar\NightTwo\confront\lewd\appease\eager\NightTwoLewdEagerContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdEagerContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Lewd eager continue.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.lewdEager.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.lewdEager.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\eager\NightTwoLewdEagerContinue.js

// scenes\Varanar\NightTwo\confront\lewd\appease\eager\NightTwoLewdEager.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdEager(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoLewdEagerContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Lewd eager.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.lewdEager.continued() ? "disabled" : "";
        var isSelected = ctrl.lewdEager.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdEager.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.EAGER) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdEager.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.EAGER) {
            return;
        }

        next.registerListeners();

        if (ctrl.lewdEager.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\eager\NightTwoLewdEager.js

// scenes\Varanar\NightTwo\confront\lewd\appease\reluctant\NightTwoLewdReluctantContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdReluctantContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You rise to your feet, before spending a good half hour stuck in between wanting to vomit, and not being able to.
                            You should have wiped your tongue before retracting it,
                            but your mind was too occupied with figuring out that the human you [i]licked[/i] just couldn't get it up,
                            which meant the other ruffian won their little [i]bet[/i], or whatever.
                        [/p]
                        [p]
                            As you finally manage to compose yourself a little, the sun starts to rise above the rooftops.
                            You'd rather have spent the remainder of the night in bed, as opposed to on your knees and bent over trying to throw up, but alas.
                            You head back towards the town square, where your [i]urban adventure[/i] started.
                            Your first night in Varanar is over, and your first day is just about to begin.
                            If there's anything you've learned, it's that you need to keep your guard up at all times.
                            The next group of humans to corner you might not be limp-dicked drunkards.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.lewdReluctant.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.lewdReluctant.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\reluctant\NightTwoLewdReluctantContinue.js

// scenes\Varanar\NightTwo\confront\lewd\appease\reluctant\NightTwoLewdReluctant.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdReluctant(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoLewdReluctantContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.flaccidOral = true;
        var toReturn = `[p]
                            You try your hardest to look for a way out of this mess you've created, but your brain draws a blank.
                            It seems as though you've talked yourself into a corner.
                            You can simply [i]refuse[/i] to play along, but judging by the group's intense behavior,
                            you can make a pretty decent guess that things [b]won't[/b] turn out nicely if you back out now.
                            The best course of action, for now, would be to just [i]play along[/i].
                            If you do ask they ask of you, this'll all be over soon...you hope.
                        [/p]
                        [p]
                            "[color=bred]Well? Don't keep me waiting.[/color]" the half-naked human's voice snaps you back to reality.
                            He rocks his hips forwards, his unsavory groin jutting out towards you.
                            You can feel your stomach churning.
                            It's just so [b]hairy[/b] and [b]disgusting[/b].
                            You'd rather suck off a [b]horse[/b] than him.
                            You [b]really[/b] don't want to do this, but the men have left you with little choice.
                        [/p]
                        [p]
                            You sigh loudly and take a deep breath of cool, late-night air.
                            Your lungs filled with clean air, you drop to your knees, and hold your breath.
                            Closing your eyes, you lean forwards, and plant a single tiny kiss on the human's flaccid length,
                            your lips pressing against his foreskin, his pubic hair brushing against your snout.
                            "[color=bred]That's it. Keep going.[/color]"
                        [/p]
                        [p]
                            While utterly revolted by the unwashed hairs rubbing against your scales,
                            you deftly plant a second kiss atop his shrunken shaft, followed by a small lick,
                            the [b]horrid[/b] taste of the human's member only now revealed to you.
                            You barely refrain from gagging, but your wincing face clearly reveals your aversion.
                            Biting through your own reluctance, you give the man a second, a third and a fourth lick without so much as retracting your tongue,
                            your tastebuds luckily growing a little numb, sparing you from the worst of the appalling taste of the filth clinging to your tongue.
                        [/p]
                        [p]
                            Despite your best efforts, the human's member remains completely flaccid,
                            growing not even a single inch since the start of your [i]reluctant admiration[/i].
                            You pull back slightly, and cast a quizzical gaze upwards at the human.
                            "[color=bred]I...I...[/color]" he stammers, clearly lost for words.
                            "[color=bred]It's the booze, I swear![/color]"
                            The entire group bursts out into laughter.
                        [/p]
                        [p]
                            "[color=green]Jehova can't get his dick up![/color]"
                            "[color=brown]I bet your wife could use a [b]real[/b] man![/color]"
                            "[color=purple]What's the matter? Kobold not so good after all?[/color]"
                            "[color=dred]Bet he'd get hard if it were a wolfess instead![/color]"
                            Embarrassed, the exposed man quickly pulls up his pants.
                            It seems the group's attention has shifted from you to him.
                        [/p]
                        [p]
                            They bicker amongst eachother for a while, before reaching a conclusion:
                            "[color=purple]Audric wins the bet. Drinks are on Jehovah![/color]"
                            "[color=brown]To the tavern![/color]"
                            And with that, the group of drunken humans rushes off towards the brothel district,
                            leaving you on your knees in the middle of the street with your tongue still lolling out,
                            somewhat wondering what the hell just happened.
                        [/p]
                        [p]
                            You absent-mindedly retract your tongue, the dry layer of grime on top of it mixing with your saliva,
                            which seemingly rejuvinates the filth's nauseating taste, while simultaneously spreading all across your mouth.
                            You heave, retch and gag.
                            It's like you're on the verge of vomiting, without being able to hurl and get it over with already.
                            At least the humans aren't around to witness this.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.lewdReluctant.continued() ? "disabled" : "";
        var isSelected = ctrl.lewdReluctant.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdReluctant.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.RELUCTANT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdReluctant.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.RELUCTANT) {
            return;
        }

        next.registerListeners();

        if (ctrl.lewdReluctant.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\reluctant\NightTwoLewdReluctant.js

// scenes\Varanar\NightTwo\confront\lewd\appease\unfit\NightTwoLewdUnfitContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdUnfitContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the humans leaving, you decide that you, too, should probably get moving, lest they change their minds.
                            You begin to make your way back towards the city square, where your nighttime adventure began.
                            While you walk, you ponder the events that just transpired.
                            Were it not for your clever way of handling the group of drunkards, things could have gone [b]very[/b] bad for you.
                            You're glad you made it out unharmed, with your dignity mostly intact.
                        [/p]
                        [p]
                            As you near the marketplace, the sun slowly rises above the rooftops.
                            You guess you won't be getting any sleep tonight.
                            Oh well.
                            Taking a deep breath of fresh morning air, you steel yourself for your first day in Varanar.
                            You'll need to be on your guard if it's to be anything like your first [i]night[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.lewdUnfit.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.lewdUnfit.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\unfit\NightTwoLewdUnfitContinue.js

// scenes\Varanar\NightTwo\confront\lewd\appease\unfit\NightTwoLewdUnfit.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdUnfit(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoLewdUnfitContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var gender = ctrl.misc.baseGender();
        var genderName = GenderEnum.properties[gender].name.toLowerCase();
        var toReturn = `[p]
                            You say you are [b]flattered[/b] that the men would even [b]consider[/b] letting such an [b]ugly[/b] kobold as yourself near their loins,
                            but surely they can find a [b]prettier[/b] one than you to quell their need, right?
                            You mean, you've got some well-defined muscles, and some very visible scars.
                            Unless they [i]like[/i] that sort of stuff, that is.
                            But wouldn't that make them attracted to [b]masculine[/b] features?
                            You don't know much about humans and what they like, but that sounds kind of [i]gay[/i] to you.
                        [/p]
                        [p]
                            The group bursts into disarray.
                            "[color=purple]The lizard's right![/color]"
                            "[color=dred]Audric's a queer![/color]"
                            "[color=green]Hey now, it was [b]Jehova[/b] that pulled down his pants![/color]"
                            They begin to argue about the merits of having sex with butch-looking ${genderName} kobolds.
                            After a bit of back and forth bickering, the human in front of you pulls up his pants, and buckles his belt once more.
                            "[color=bred]Can't even get my dick sucked without a fight these days.[/color]" he mumbles.
                        [/p]
                        [p]
                            While you aren't keen on interrupting the men while they're mid-[b]discussion[/b],
                            you quietly suggest that they should perhaps go visit a whorehouse,
                            where there are plenty of more [b]feminine[/b] people eager to give them [b]all[/b] blowjobs.
                            Heck, perhaps the whores can give further insights on their little [i]disagreement[/i].
                            They [i]are[/i] experts at having sex, after all.
                            Surely they must know plenty about whether or not it's gay to face-fuck a somewhat masculine kobold.
                        [/p]
                        [p]
                            "[color=brown]Right again.
                            You're pretty smart, for a lizard![/color]"
                            One of the most attentive humans commends.
                            You quietly whisper that he has no idea just [i]how[/i] smart you are.
                            Luckily, he doesn't hear you.
                            "[color=green]Let's go settle this at the whorehouse, then![/color]"
                            "[color=purple]Yeah, let's go prove Audric's gay![/color]"
                            "[color=green]You wish, Cedric![/color]"
                            "[color=brown][b]To the brothel![/b][/color]"
                            "[color=dred]God damnit.[/color]"
                        [/p]
                        [p]
                            With that, the drunkards break their encirclement.
                            Like they've completely forgotten you even exist, they head for the brothel district, leaving you all alone in the middle of the street.
                            Just the way you like it.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.lewdUnfit.continued() ? "disabled" : "";
        var isSelected = ctrl.lewdUnfit.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdUnfit.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.UNFIT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdUnfit.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.appease.get() !== NightTwoLewdAppeaseChoice.UNFIT) {
            return;
        }

        next.registerListeners();

        if (ctrl.lewdUnfit.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\appease\unfit\NightTwoLewdUnfit.js

// scenes\Varanar\NightTwo\confront\lewd\appease\NightTwoLewdAppease.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdAppease(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reluctantId = arguments.callee.name + "ButtonOne";
    var eagerId = arguments.callee.name + "ButtonTwo";
    var unfitId = arguments.callee.name + "ButtonThree";

    var toReluctant = new NightTwoLewdReluctant(ctrl);
    var toEager = new NightTwoLewdEager(ctrl);
    var toUnfit = new NightTwoLewdUnfit(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            No matter what you say, they'll find a way to turn it against you,
                            but if you don't play along, you doubt things would end well.
                            If you just give them the laugh they're after, you're certain they'll leave you alone.
                            Besides, if you show the group that you can take a [i]joke[/i] just as well as any human can,
                            perhaps they'll get bored and find a different target to pick on.
                        [/p]
                        [p]
                            Forsaking your pride, you half-heartedly stammer out that yes, kobolds are [i]indeed[/i] good at [b]sucking human cock[/b].
                            You can barely even get those final three words past your lips, your mind screaming at you [b]not[/b] to put yourself down like this.
                            But, in a way, the statement isn't even [i]wrong[/i].
                            While most kobolds are [i]scoundrels[/i] of some form or another, the ones that aren't tend to be [b]prostitutes[/b],
                            relying on innate [b]tightness[/b] due to size difference, and a knack for shoving things down their throats without gagging or retching.
                            In that sense: yes. Your kind [i]is[/i] good at [b]giving blowjobs[/b].
                        [/p]
                        [p]
                            The entire group bursts out into laughter at the sight of a kobold humiliating itself.
                            The drunken men begin to taunt you anew, although the mood is slightly more cheerful now than it was before,
                            their insults clearly less meant to [b]hurt[/b] you this time around.
                            It seems as though your gamble has paid off.
                        [/p]
                        [p]
                            One of the men still seems displeased.
                            As the posse begins to quiet down, he raises his voice:
                            "[color=green]Kobolds are [b]liars[/b].
                            Who knows if this one is telling the truth?!
                            Make'em [b]prove[/b] it![/color]"
                            At first, the group pays the dissenter no mind.
                            Slowly, but steadily, however, cogs begin to turn inside of their drunken heads.
                            As they come to a communal conclusion, some of the men begin to voice their response in unison:
                            "[color=purple]Yeah. Make'em prove it![/color]"
                            "[color=brown]The kobold lies![/color]"
                            "[color=dred]Put on a show![/color]"
                        [/p]
                        [p]
                            The man in front of you realizes he's losing the crowd's favour.
                            A grin forms atop his visage.
                            "[color=bred]Audric's right.[/color]" he speaks, his hands travelling down to his waist, resting atop his belt buckle.
                            "[color=bred]You lizards are a bunch of liars.[/color]"
                            He unfastens the clasp sealing the leather cord, his pants visible loosening.
                            "[color=bred]And we don't take kindly to liars around these parts.[/color]"
                            That being said, the drunkard promptly lowers his pants and his underwear,
                            his flaccid, uncut member surrounded with a hefty amount of pubic hair
                            "[color=bred]Let's see if you were telling the truth or not.[/color]"
                        [/p]
                        [p]
                            The entire group seems riled up.
                            They [b]really[/b] want this to happen, and judging by the exposed human's tone,
                            you're in too deep to back out now.
                            Unless you can present a strong case for yourself, you fear this [b]is[/b] happening.
                            That doesn't mean you need to like it, though.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildReluctantChoice() {
        var disabled = ctrl.appease.has() ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoLewdAppeaseChoice.RELUCTANT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${reluctantId}" ${disabled}>Reluctant</button>`;

        var tooltip = global.parser.parse(`You don't really want to participate in these guys' debauchery, but you don't have much of a choice.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 185px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildEagerChoice() {
        var disabled = ctrl.appease.has() ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoLewdAppeaseChoice.EAGER;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${eagerId}" ${disabled}>Eager</button>`;

        var tooltip = global.parser.parse(`Give the men a show they'll never forget.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildUnfitChoice() {
        var test = ctrl.misc.strength() > 5 && ctrl.misc.endurance() > 5;
        var disabled = (ctrl.appease.has() || !test) ? "disabled" : "";
        var isSelected = ctrl.appease.get() === NightTwoLewdAppeaseChoice.UNFIT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${unfitId}" ${disabled}>Ugly</button>`;

        var tooltip = global.parser.parse(`Surely these men can find a prettier kobold to abuse, right?`);
        if (!test) {
            tooltip = `Requires 6 strength and 6 endurance.`;
        }

        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You are: </div>`;
        choice += buildReluctantChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildUnfitChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + reluctantId), $("#" + eagerId), $("#" + unfitId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerReluctantListener() {
        $("#" + reluctantId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoLewdAppeaseChoice.RELUCTANT);
            loadNextPart(toReluctant);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerEagerListener() {
        $("#" + eagerId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoLewdAppeaseChoice.EAGER);
            loadNextPart(toEager);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerUnfitListener() {
        $("#" + unfitId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.appease.set(NightTwoLewdAppeaseChoice.UNFIT);
            loadNextPart(toUnfit);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.lewdCrude.get() !== NightTwoCrudeChoice.APPEASE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toReluctant.build();
        part += toEager.build();
        part += toUnfit.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.lewdCrude.get() !== NightTwoLewdChoice.APPEASE) {
            return;
        }

        toReluctant.registerListeners();
        toEager.registerListeners();
        toUnfit.registerListeners();

        if (ctrl.appease.has()) {
            return;
        }

        registerReluctantListener();
        registerEagerListener();
        registerUnfitListener();
    };
}

var NightTwoLewdAppeaseChoice = {
    RELUCTANT: 1,
    EAGER: 2,
    UNFIT: 3
};

// END scenes\Varanar\NightTwo\confront\lewd\appease\NightTwoLewdAppease.js

// scenes\Varanar\NightTwo\confront\lewd\rude\fight\lose\NightTwoLewdLose.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdLose(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    var idsToRemove = [
        "NightTwoLewdLose",
        "NightTwoLewdFight",
        "NightTwoLewdRude",
        "NightTwoLewd",
        "NightTwoConfront",
        "NightTwoRunIn",
        "NightTwoStage"
    ];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            No.
                            You aren't going to let them do this.
                            You can't and you [b]won't[/b] get [b]anywhere[/b] near that filthy human's disgusting groin, let alone taking his [i]thing[/i] into your mouth!
                            [b]Over your dead body.[/b]
                            They may be stronger than you, they may outnumber you, they may have you in an arm-grip,
                            but all of that just means they [b]won't[/b] be [i]expecting[/i] you to go on the offensive.
                        [/p]
                        [p]
                            They think they have you scared shitless, that you're just another kobold for them to [b]bully[/b] and [b]abuse[/b],
                            that you won't put up a fight simply because this is [b]their[/b] town.
                            But you aren't any ordinary run-of-the-mill lizard.
                            The other members of your kind may be content being thieves, or bandits or prostitutes or beggars, but not [b]you[/b].
                            You may be a thief right now, but you [b]aspire[/b] to be a [b]knight[/b].
                            And knights don't back down in the face of adversity.
                            They charge head-forward into danger.
                            Just like you're about to do.
                        [/p]`;

        if (weapon) {
            toReturn += `[p]
                            While one of your arms is being restrained, you reach for your ${global.kobold.inventory.createItemWithTooltip(weapon)} with the other.
                            With your head being forced ever-closer to the drunkard's dick, you decide to strike before your snout is pushed into his unwashed pubic hair.
                            Swiftly and silently, you [b]surge[/b] your weapon forwards, intent on striking the ruffian's exposed groin,
                            yet before your strike manages to connect, your free arm is grabbed and painfully twisted, much like the other.
                            With a pained yelp, you drop your ${global.kobold.inventory.createItemWithTooltip(weapon)} to the ground,
                            where it is quickly picked up by one of the humans, who promptly [b]bashes[/b] you on the head with it.
                        [/p]`;
        } else {
            toReturn += `[p]
                            While one of your arms is being restrained, you lash out with the other, intent on striking the oaf's exposed groin,
                            yet before your strike manages to connect, your free arm is grabbed and painfully twisted, much like the other.
                            You let out a pained yelp.
                            Your attempted aggression is soon punished, one of the humans brutally [b]bashing[/b] the top of your head with his balled up hand.
                        [/p]`;
        }

        toReturn += `   ${global.ui.condFail(weapon ? `${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence below 4` : `No weapon`)}
                        [p]
                            Your vision fades to black.
                            You lose consciousness.
                            You dream about a ceremony.
                            One in your honor.
                            You are in a castle.
                            There's princes and princesses, barons and dukes, royals from all corners of the world.
                            You are kneeling.
                            The Dragon King himself puts a gilded blade atop your shoulder.
                            Then suddenly, the dream turns into a nightmare, the castle replaced by a terrible, all-consuming void that sucks you into its core.
                            Once the void has consumed all, you are left with nothing, not even yourself.
                            The nightmare ends, and so do you.
                            In the real world, the men kicked you to death for passing out instead of doing their bidding.
                            Luckily, you were not awake to witness the horrible pain inflicted upon you.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Nightmare ending.
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400, function () {
            var main = $("#main");
            main.scrollTop(main[0].scrollHeight - main.height() - 5); // Fix scrolling
        });
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.misc.unsetAll();
            ctrl.rude.undo();
            ctrl.rude.unset();
            ctrl.lewdCrude.undo();
            ctrl.lewdCrude.unset();
            ctrl.runin.unset();
            ctrl.stage.unset();
            for (var i = 0; i < idsToRemove.length; i++) {
                $("#" + idsToRemove[i]).remove();
            }
            loadNextPart(new NightTwoStage(ctrl));
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        registerContinueListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\fight\lose\NightTwoLewdLose.js

// scenes\Varanar\NightTwo\confront\lewd\rude\fight\win\NightTwoLewdWinContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdWinContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            As your adrenaline rush fades, the harsh reality of things comes crashing down on you.
                            You just [b]attacked[/b] and seriously [b]hurt[/b] a bunch of humans.
                            They might have been bothering you, but the guards won't care.
                            To them, you're a stupid, unruly lizard that crossed the line.
                            The men you assaulted had rights.
                            You don't.
                        [/p]
                        [p]
                            You make the smart decision to vacate the scene of the crime.
                            After jogging a few streets over, you lower your pace to a brisk walk.
                            You think you're in the clear.
                            If the famous guards of Varanar were called, you wouldn't have made it this far.
                            You'd have been captured or killed before you even left the street.
                            Still, you probably shouldn't linger in this particular area for too long.
                        [/p]
                        [p]
                            As you begin to make your way back to the town square, where your late-night adventure began, the sun slowly begins to rise above the rooftops.
                            It seems as though you've managed to retain your dignity throughout the night, but at what cost?
                            You don't want to think about it.
                            It's too late, or rather [i]too early[/i] to go to bed.
                            Who needs sleep?
                            You'd just have bad dreams about the things you did anyway.
                            You know you acted in self-defence, but your conscience doesn't agree. 
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.fightWin.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.fightWin.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\fight\win\NightTwoLewdWinContinue.js

// scenes\Varanar\NightTwo\confront\lewd\rude\fight\win\NightTwoLewdWin.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdWin(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoLewdWinContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        global.kobold.history.shared.varanar.nightTwo.fought = true;
        var toReturn = `[p]
                            No.
                            You aren't going to let them do this.
                            You can't and you [b]won't[/b] get [b]anywhere[/b] near that filthy human's disgusting groin, let alone put his [i]thing[/i] in your mouth!
                            [b]Over your dead body.[/b]
                            They may be stronger than you, they may outnumber you, they may have you in an arm-grip,
                            but all of that just means they [b]won't[/b] be [i]expecting[/i] you to go on the offensive.
                        [/p]
                        [p]
                            They think they have you scared shitless, that you're just another kobold for them to [b]bully[/b] and [b]abuse[/b],
                            that you won't put up a fight simply because this is [b]their[/b] town.
                            But you aren't any ordinary run-of-the-mill lizard.
                            The other members of your kind may be content being thieves, or bandits or prostitutes or beggars, but not [b]you[/b].
                            You may be a thief right now, but you [b]aspire[/b] to be a [b]knight[/b].
                            And knights don't back down in the face of adversity.
                            They charge head-forward into danger.
                            Just like you're about to do.
                        [/p]
                        [p]
                            While one of your arms is being restrained, you reach for your ${global.kobold.inventory.createItemWithTooltip(weapon)} with the other.
                            With your head being forced ever-closer to the drunkard's dick, you decide to strike before your snout is pushed into his unwashed pubic hair.
                            Swiftly and silently, you [b]surge[/b] your weapon forwards,
                            shoving the pointy-end directly into the ruffian's exposed groin, stabbing him in his flaccid length.
                            The man screams in pain.
                            His companions laugh out loud, thinking you bit him.
                            Their laughter comes to a halt, however, as the group slowly realizes what has transpired.
                        [/p]
                        [p]
                            You pull your weapon from the pantless delinquent's loins, and in one swift move, you bring it to bear on the person holding your arm.
                            Blindly stabbing backwards, you land a seemingly solid strike on the human's abdomen.
                            He loses his hold on your arm nigh-instantly as your ${global.kobold.inventory.createItemWithTooltip(weapon)} sinks into his gut.
                            The man whose cock you perforated tumbles backwards to the ground.
                            Blood oozes liberally from his mutilated groin.
                            The guy behind you stumbles backwards, clutching his wounded belly.
                            Communal panic settles in.
                            [i]Perfect[/i].
                        [/p]
                        ${global.ui.condSucc(`${global.kobold.inventory.createItemWithTooltip(weapon, true)} offence greater than 3`)}
                        [p]
                            Your freedom regained, you drop into a combat stance and begin to wildly swing your weapon about in all directions,
                            attempting and succeeding in pushing back the humans surrounding you.
                            Their encircling formation broken, one of their comrades downed, and the other vomiting blood, the frightened men turn tail and run away.
                            By growing hysteria, fostered by your ballsy outburst, you have managed to drive the overwhelming odds to flee,
                            causing an entire group of intoxicated humans to sprint away from a single kobold with a weapon,
                            not even bothering to pick up their wounded in the process.
                        [/p]
                        [p]
                            As the dust settles, the only people left in the street are you, and the guy you stabbed in the dick.
                            Only one of you is standing, and it isn't him.
                            You take a few deep breaths, slowly calming down from the adrenaline rush that took over you.
                            You did it.
                            You scared them off.
                            You showed them that they can't just push kobolds around without any repercussions,
                            that you and your kind alike [b]will[/b] fight back if cornered.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.fightWin.continued() ? "disabled" : "";
        var isSelected = ctrl.fightWin.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.fightWin.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.fightWin.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.fightWin.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\fight\win\NightTwoLewdWin.js

// scenes\Varanar\NightTwo\confront\lewd\rude\fight\NightTwoLewdFight.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdFight(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoLewdLose(ctrl);
    var toSuccess = new NightTwoLewdWin(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var weapon = ctrl.misc.weapon();
        return weapon && weapon.offence() >= 3 && weapon.stab();
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.rude.get() !== NightTwoLewdRudeChoice.FIGHT) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.rude.get() !== NightTwoLewdRudeChoice.FIGHT) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\fight\NightTwoLewdFight.js

// scenes\Varanar\NightTwo\confront\lewd\rude\struggle\NightTwoLewdStruggleContinue.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdStruggleContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You are absolutely [b]disgusted[/b].
                            Sickened, not only by what you were made to do, but by yourself, and by society as a whole.
                            [b]Nobody[/b] should be forced to do something so [b]foul[/b] and [b]revolting[/b] just to settle some stupid drunken quarrel.
                            You cry for what seems like an eternity, stuck in between wanting to throw up, and not being able to.
                            The ground is as cold as the hearts of the humans that humiliated you.
                        [/p]
                        [p]
                            The sun slowly rises above the building-rooftops, and as you feel its first warming rays shine atop your back,
                            you pick yourself off the ground, along with the broken pieces of your shattered self-image.
                            While staying on the ground until you [b]die[/b] sounds like a good plan right now,
                            you decide it'd be best if you made your way back to the city square.
                            You won't be getting any sleep tonight.
                            At least you learned a valuable lesson: humans are [b]not[/b] nice people.
                            They [b]hate[/b] your kind, and you shouldn't trust them any further than you can throw them,
                            which isn't very far at all.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.lewdStruggle.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.lewdStruggle.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\struggle\NightTwoLewdStruggleContinue.js

// scenes\Varanar\NightTwo\confront\lewd\rude\struggle\NightTwoLewdStruggle.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdStruggle(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoLewdStruggleContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.forced = true;
        global.kobold.history.shared.varanar.nightTwo.flaccidOral = true;
        var toReturn = `[p]
                            You wriggle around, attempting to escape the human's grasp,
                            but all he has to do is apply some extra [b]force[/b] to halt your squirming.
                            Your arm feels like it's about to [b]break[/b].
                            The pain almost makes you burst into tears, yet you manage to save face for just a little while longer.
                            Your mind [b]screams[/b] for you to beg him to let go, but you don't want to humiliate yourself any further.
                        [/p]
                        [p]
                            Your head gets gradually pushed lower and lower, closer to the hulking brute's bare loins.
                            "[color=bred]Hurry up, I haven't got all night.[/color]" he complains,
                            slightly rocking his hips forwards to jut his groin out further, its unsavory fragrance reaching your nose.
                            "[color=purple]You heard him, lizard.
                            Here you go.[/color]" the man holding your arm speaks.
                            Then, the slow, constant lowering is replaced by a single fierce [b]shove[/b],
                            your face forcibly pressed against the exposed delinquent's hefty sack, his member rubbing against your snout.
                        [/p]
                        [p]
                            Your lips end up pressing directly against the drunkard's balls.
                            You make the mistake of gasping in surprise,
                            inadvertently taking in a mouthful and a noseful of the foul, concentrated stench of the man's unwashed malehood,
                            an intense feeling of [b]sickness[/b] building inside of you.
                            The thought of [b]biting down[/b], of [b]tearing[/b] apart his most [b]sensitive[/b] bodypart crosses your mind,
                            but the rest of the louts would undoubtedly [b]kill you[/b] for such an act of defiance.
                            You really don't want to die with human scrotum stuck to your teeth.
                        [/p]
                        [p]
                            The group goes wild.
                            Half of them burst out into laughter, the other half start to ridicule you,
                            hurling every insult they know your way in an attempt to make your life even more [b]miserable[/b] than it already is.
                            Your free arm gets grabbed, and held in place much like the other.
                            Your tail gets harshly yanked as a prank.
                            You don't know what brings more tears to your eyes:
                            the foul act you're forced to perform, or the oh-so-casual cruelty and complete disregard of your well-being.
                            You're nothing but an [b]animal[/b] to these people.
                        [/p]
                        [p]
                            You struggle with all of your might, yet the grip on your arms and on your head is unyielding.
                            You thought they just wanted you to play with his dick a little, not lap at his filthy nuts!
                            If you knew they wanted you to do [b]this[/b], you'd have [b]fought them[/b] with a bit more [b]conviction[/b]!
                            But it's too late for that now.
                            You're completely trapped.
                            "[color=brown]Get to work or we'll [b]bash your brains in[/b] and find a different lizard that [b]will[/b] cooperate![/color]"
                        [/p]
                        [p]
                            Crying out loud, you swallow what little remains of your pride.
                            Puckering your lips, you plant a single, small kiss atop the half-naked human's nuts.
                            "[color=bred]That's it. Keep going.[/color]"
                            Despite every fiber of your being begging you not to give in, your self-preservation instinct takes control.
                            You give the man's balls another kiss, and then a small lick, your tongue collecting more than a little ballsack-grime.
                            The taste is [b]abhorrent[/b].
                            Without retracting the flexible appendage, you give the drunkard's nuts a second, and then a third lick,
                            your tastebuds luckily going numb, sparing you from the worst of the flavour.
                        [/p]
                        [p]
                            Slowly, but steadily, the flow of insults directed at [b]you[/b] comes to an end.
                            Instead, the ruffians seem to be picking on the man whose balls are in your face.
                            "[color=green]Jehova can't get his dick up![/color]" one of the group's members comments.
                            "[color=bred]What're you lookin' at my dick for, Audric?![/color]" comes the angry reply.
                            "[color=brown]He's right! It's as soft as a virgin dragoness' ass![/color]"
                            "[color=purple]What's the matter? Kobold not so good after all?[/color]"
                            "[color=dred]Bet he'd get hard if it were a wolfess instead![/color]"
                        [/p]
                        [p]
                            "[color=bred]It's the booze, alright?![/color]"
                            Embarrassed, the exposed man pushes you to the ground, before quickly pulling up his pants.
                            It seems the group's attention has shifted from you to him.
                            The moment your body hits the cold, hard dirt, you curl up into a ball and begin to cry your eyes out.
                            You accidentally retract your lolling tongue, scrotum-filth spreading throughout your mouth,
                            your saliva seemingly reinvigorating its horrid taste.
                        [/p]
                        [p]
                            Some of the humans taunt and humiliate you, but luckily,
                            most of them are more interested in the fact that the [i]bet[/i] between Audric and Jehova has been settled.
                            You tune out for a little while, not interested in what they, or anyone, for that matter has to say.
                            After a bit of rowdy discussion, the group sets off towards the brothel district, leaving you to wallow in self-pity,
                            while retching from the taste of unwashed human balls.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.lewdStruggle.continued() ? "disabled" : "";
        var isSelected = ctrl.lewdStruggle.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdStruggle.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.rude.get() !== NightTwoLewdRudeChoice.STRUGGLE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdStruggle.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.rude.get() !== NightTwoLewdRudeChoice.STRUGGLE) {
            return;
        }

        next.registerListeners();

        if (ctrl.lewdStruggle.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\confront\lewd\rude\struggle\NightTwoLewdStruggle.js

// scenes\Varanar\NightTwo\confront\lewd\rude\NightTwoLewdRude.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewdRude(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var fightId = arguments.callee.name + "ButtonOne";
    var struggleId = arguments.callee.name + "ButtonTwo";

    var toFight = new NightTwoLewdFight(ctrl);
    var toStruggle = new NightTwoLewdStruggle(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            No matter what you say, they'll find a way to turn it against you.
                            Might as well not even [b]bother[/b] to play along.
                            Besides, if you show the group that you don't take kindly to them messing with you,
                            perhaps they'll leave you alone and find a different target to pick on.
                            Heck, given their [i]rough[/i] appearance, they might even [i]appreciate[/i] you not being a [b]total pushover[/b].
                        [/p]
                        [p]
                            Raising your voice, you loudly tell the man to [b]fuck off[/b].
                            You're not solving any disputes for a bunch of rude, inbred [b]humans[/b]!
                            Kobolds suck dick, huh?!
                            That's funny, coming from a race of [b]bootlickers[/b] that'd [b]jerk their lord off[/b] for an extra acre of land,
                            or some food to make it through the winter!
                            If anything, [b]humans[/b] are the cocksuckers,
                            and you can't [b]wait[/b] for your kind to replace theirs as the dominant party around these parts!
                        [/p]
                        [p]
                            Wew.
                            You, uhh, didn't mean to go on a [i]rant[/i] like that.
                            You just wanted to throw some cuss words around to impress the drunkards, so they'd leave you alone.
                            A lifetime of [b]taking shit[/b] from their kind left you with a [b]lot[/b] of pent up frustration,
                            and you couldn't really stop yourself after you started venting some of that budding resentment.
                            Alas, the deed is done.
                            There's no going back now.
                            All you can do is [b]pray[/b] that the humans'll take your little [i]tirade[/i] as a [i]joke[/i].
                        [/p]
                        [p]
                            Sadly, that doesn't seem to be the case.
                            The group goes eerily quiet, angry eyes glaring down at you from all directions.
                            "[color=bred]I don't think you understand where you are.[/color]" the man that approached you breaks the silence,
                            his tone far more [b]hostile[/b] than before.
                            His hands travel down to his waist, and he deftly unbuckles the heavy belt keeping his pants up.
                            "[color=bred]This is a [b]human[/b] city, [b]lizard[/b].[/color]"
                        [/p]
                        [p]
                            From behind, one of the men roughly grabs your arm, cruelly and painfully twisting it.
                            You barely refrain from voicing your anguish, still attempting to keep up the [i]tough[/i] facade.
                            "[color=bred]And in Varanar, when [b]humans[/b] ask you a question, you [b]answer them[/b].[/color]"
                            Your head is pushed down, [b]forcing[/b] you to bend over forwards.
                            "[color=bred]But since you don't want to [b]talk[/b] to us...[/color]"
                            The hulking brute lowers his trousers and his underwear as he speaks,
                            his pubic hair-engulfed, flaccid, uncut member mere inches away from your face.
                            "[color=bred]...you're just going to have to [b]show[/b] us how good you are at [b]sucking dick[/b].[/color]"
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFightChoice() {
        var disabled = ctrl.rude.has() ? "disabled" : "";
        var isSelected = ctrl.rude.get() === NightTwoLewdRudeChoice.FIGHT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fightId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`You're not going to go down without a fight. Come out swinging.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildStruggleChoice() {
        var disabled = ctrl.rude.has() ? "disabled" : "";
        var isSelected = ctrl.rude.get() === NightTwoLewdRudeChoice.STRUGGLE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${struggleId}" ${disabled}>Struggle</button>`;

        var tooltip = global.parser.parse(`Struggle against the humans' grasp, but don't outright attack them.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildFightChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildStruggleChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + fightId), $("#" + struggleId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFightListener() {
        $("#" + fightId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rude.set(NightTwoLewdRudeChoice.FIGHT);
            loadNextPart(toFight);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerStruggleListener() {
        $("#" + struggleId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.rude.set(NightTwoLewdRudeChoice.STRUGGLE);
            loadNextPart(toStruggle);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.lewdCrude.get() !== NightTwoLewdChoice.RUDE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.rude.change();
        part += buildChoice();
        part += `</div>`;
        part += toFight.build();
        part += toStruggle.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.lewdCrude.get() !== NightTwoLewdChoice.RUDE) {
            return;
        }

        toFight.registerListeners();
        toStruggle.registerListeners();

        if (ctrl.rude.has()) {
            return;
        }

        registerFightListener();
        registerStruggleListener();
    };
}

var NightTwoLewdRudeChoice = {
    FIGHT: 1,
    STRUGGLE: 2
};

// END scenes\Varanar\NightTwo\confront\lewd\rude\NightTwoLewdRude.js

// scenes\Varanar\NightTwo\confront\lewd\NightTwoLewd.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoLewd(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var appeaseId = arguments.callee.name + "ButtonOne";
    var rudeId = arguments.callee.name + "ButtonTwo";

    var toAppease = new NightTwoLewdAppease(ctrl);
    var toRude = new NightTwoLewdRude(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var male = ctrl.misc.baseGender() === GenderEnum.MALE;
        var cd = ctrl.misc.crossdressing();
        var toReturn = `[p]
                            You decide [b]not[/b] to empower the group's belligerent behavior.
                            Instead of giving them attention, or even the [b]satisfaction[/b] of chasing after you,
                            you're going to walk right past the hoodlums without so much as [b]looking[/b] at them.
                            Tilting your head the other way,
                            staring at the front-faces of the buildings you pass as though you're inspecting them,
                            you begin to stride down the road.
                            As you approach, the group appears to simmer down a bit, your tactic of not paying them any mind seemingly effective.
                        [/p]
                        [p]
                            Looking away to the side, however, quickly proves to have been a mistake.
                            The moment you cross the delinquents' party, you suddenly [b]bump[/b] into something,
                            the unexpected collision almost making you fall back onto your butt.
                            Your gaze snapping forwards, you become aware that the [i]thing[/i] you walked into is none other than one of the men.
                            Taking a few steps backwards to regain your balance, you find your rear end pressing against something as well.
                            Another human.
                            It looks like, while you were [i]pretending[/i] not to notice them, the drunkards somehow managed to surround you!
                        [/p]
                        [p]
                            The man you bumped into last pushes you forwards, into the middle of the little [i]circle[/i] they've formed.
                            While they were quiet enough to sneak up on you, now that they have you surrounded, the group soon returns to its rowdy nature.
                            Cruel jokes and a myriad of lewd insults are hurled your way.
                            "[color=purple]${cd && male ? "It's dressed like a girl!" : "Nude whore!"}[/color]"
                            "[color=dred]What's your rate? Two coins?![/color]"
                            "[color=brown]Tighter than my wife, I bet![/color]"
                            Growing concerned, you desperately look around for a way out, yet they have you [b]completely[/b] encircled.
                            Any time you take so much as a step towards them, you get roughly [b]shoved[/b] back into the very centre of the group.
                        [/p]
                        [p]
                            One of the humans breaks formation, stepping towards you with a swaying gait that [b]screams[/b] drunkenness.
                            You fear for the worst, but instead of the [b]brutal pummeling[/b] you half-expected,
                            the ruffian comes to a halt a few feet in front of you.
                            It seems he is sober enough to slur out a few coherent words:
                            "[color=bred]Me and...[/color]" *hic* "[color=bred]...my buddy Audric back there was arguin' 'bout you [i]lizards[/i] earlier.[/color]"
                            The brute takes a lengthy pause to consider his next words, his inebriated brain clearly working overtime.
                            "[color=bred]He says your kind is good for [b]nothing[/b].
                            I disagree.
                            You kobolds are good at [b]suckin' dick[/b], especially when it comes to [b]bigger species[/b] like [b]us[/b].
                            I think you give better blowjobs than [b]any[/b] other race!
                            So, which one of us is right?[/color]"
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildAppeaseChoice() {
        var disabled = ctrl.lewdCrude.has() ? "disabled" : "";
        var isSelected = ctrl.lewdCrude.get() === NightTwoLewdChoice.APPEASE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${appeaseId}" ${disabled}>Appeasing</button>`;

        var tooltip = global.parser.parse(`Attempt to appease the man. If you play along, the group'll have no reason to hurt you, right?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 230px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRudeChoice() {
        var disabled = ctrl.lewdCrude.has() ? "disabled" : "";
        var isSelected = ctrl.lewdCrude.get() === NightTwoLewdChoice.RUDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${rudeId}" ${disabled}>Rude</button>`;

        var tooltip = global.parser.parse(`Don't give them an inch. Refuse to play along. Give them a piece of your mind, and send them on their way.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You are: </div>`;
        choice += buildRudeChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAppeaseChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + appeaseId), $("#" + rudeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerAppeaseListener() {
        $("#" + appeaseId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdCrude.set(NightTwoLewdChoice.APPEASE);
            loadNextPart(toAppease);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerRudeListener() {
        $("#" + rudeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.lewdCrude.set(NightTwoLewdChoice.RUDE);
            loadNextPart(toRude);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.lewdCrude.change();
        part += buildChoice();
        part += `</div>`;
        part += toAppease.build();
        part += toRude.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toAppease.registerListeners();
        toRude.registerListeners();

        if (ctrl.lewdCrude.has()) {
            return;
        }

        registerAppeaseListener();
        registerRudeListener();
    };
}

var NightTwoLewdChoice = {
    RUDE: 1,
    APPEASE: 2
};

// END scenes\Varanar\NightTwo\confront\lewd\NightTwoLewd.js

// scenes\Varanar\NightTwo\confront\NightTwoConfront.js

/**
 * Confrontation section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoConfront(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoLewd(ctrl);
    var toSuccess = new NightTwoCrude(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        if (ctrl.misc.baseGender() === GenderEnum.MALE) {
            return !ctrl.misc.crossdressing() && !ctrl.misc.groinExposed();
        } else {
            return !ctrl.misc.groinExposed();
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.runin.get() !== NightTwoRunInChoice.IGNORE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.runin.get() !== NightTwoRunInChoice.IGNORE) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\confront\NightTwoConfront.js

// scenes\Varanar\NightTwo\flee\alley\hide\attack\NightTwoHideAttackContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoHideAttackContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            After removing yourself from the [i]brothel district[/i] proper,
                            you start walking back towards the town square,
                            where your [i]urban adventure[/i] began.
                            As you begin to near it, you notice the sun slowly rising above the rooftops.
                            You must have lost track of time while exploring the city.
                            Or maybe you were left waiting in the alleyway for the group to scatter for a longer time than you care to remember.
                            Either way, you guess you won't be sleeping tonight.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded, giving way to a sense of [b]vulnerability[/b].
                            You are but a lone, weak, small, kobold in a big city filled with people who can and [b]want[/b] to maim,
                            hurt, kill and perhaps even [b]rape[/b] you.
                            If you let your guard down for even one moment, it could cost you your money, your life or both.
                            Still, you held your own tonight.
                            You waited for exactly the right moment to strike, and you felled a human twice your size with a single precise blow.
                            Maybe living in the city for a while won't be too bad after all.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.hideAttack.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.hideAttack.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\hide\attack\NightTwoHideAttackContinue.js

// scenes\Varanar\NightTwo\flee\alley\hide\attack\NightTwoHideAttack.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoHideAttack(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoHideAttackContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.alleyAttack = true;
        var toReturn = `[p]
                            This guy thinks he can just [b]piss[/b] all over whatever the hell he wants?!
                            Has he no decency?
                            No way.
                            You aren't going to let some [b]filthy human[/b] take a leak in your face, on top of a bunch of barrels that aren't even his.
                            He might not [b]know[/b] you're down here, but that just means you have the advantage of surprise.
                            Time to teach this guy a lesson about keeping his dick in his pants until he gets home.
                        [/p]
                        [p]
                            While a puddle of urine steadily forms next to you, you slowly rise from your huddled-up position on the ground,
                            like a snake rearing its head before [b]striking[/b].
                            Your eyes set on the ruffian's considerable package, you make your move.
                            You leap forwards, bundling all of your [b]anger[/b] and [b]hatred[/b] into a single powerful [b]swipe[/b].
                            Your sharp claws [b]smack[/b] against the human's exposed balls, [b]tearing[/b] through his scrotum, 
                            [b]ripping[/b] open his sack, tearing his nuts directly from his groin, mangling his genitals to beyond recognition.
                        [/p]
                        [p]
                            Your fingers [b]bathe[/b] in the human's blood, which gushes liberally from the gaping hole left where his testicles used to be.
                            His senses dulled by alcohol, it takes the man a moment to [b]realize[/b] what the hell is going on.
                            Looking downwards, he stares at the remnants of what used to be his balls, laying limply on the ground,
                            blood starting to mix with the ever-expanding piss-puddle.
                            His jaw drops agape.
                            He lets out an uncharacteristic high-pitched squeak.
                            His eyes rolling into the back of his head, the scoundrel tumbles backwards, falling lifelessly to the cold, hard ground.
                            Unable to cope with the immense amount of pain, he loses his consciousness.
                            The ruffian faints, with his now neutered, blood-oozing flaccid cock still on full display.
                        [/p]
                        [p]
                            As soon as the hoodlum falls, you immediately rush past him towards the entry to the alleyway.
                            Luckily, none of his [i]friends[/i] seem to have noticed the sound of his body falling to the floor.
                            They have continued walking along the road, and are starting to disperse.
                            You wait for a few more minutes for them to leave the area completely.
                            After wiping your hands on the downed human's clothes, 
                            you quickly and quietly escape the scene,
                            swiftly making your way to a more [i]hospitable[/i] neighbourhood.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.hideAttack.continued() ? "disabled" : "";
        var isSelected = ctrl.hideAttack.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hideAttack.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.hide.get() !== NightTwoHideChoice.ATTACK) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.hideAttack.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.hide.get() !== NightTwoHideChoice.ATTACK) {
            return;
        }

        next.registerListeners();

        if (ctrl.hideAttack.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\hide\attack\NightTwoHideAttack.js

// scenes\Varanar\NightTwo\flee\alley\hide\stay\NightTwoHideStayContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoHideStayContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var pants = ctrl.misc.pants();
        var shirt = ctrl.misc.shirt();
        var clothes = shirt || pants;
        var toReturn = `[p]
                            The moment the human leaves the alleyway, you instantly rise to your feet and clean your face off with your single dry hand.
                            You feel completely [b]disgusted[/b].
                            Unable to weather the stench for even a second longer, you step out from behind the barrels,
                            and wander deeper into the alleyway in search of some fresh air.
                            At this point, you don't even [b]care[/b] about getting caught anymore.
                            The group seems to have wander off, but you'd have broken your cover even if they hadn't.
                        [/p]
                        [p]
                            Leaning against the brick wall sealing off the alley, you bend over and retch for a few solid minutes straight,
                            stuck in between the intense urge to vomit and the desire [b]not[/b] to let your last meal go to waste.
                            You can still [b]reek[/b] the human's piss, splatters of which linger on your ${clothes ? `clothing` : `body`} and on your face.
                            You fear the scent may have [b]seeped[/b] into your very scales, even though you didn't get [i]that much[/i] of it on you.
                        [/p]
                        [p]
                            While sickened by the lasting urine-smell, you are grateful that at least the fragrance of refuse hasn't reached this end of the alley.
                            At least here, you don't have to suffer the mixture of sweaty human groin, reeking barrels and dehydrated pee all at once.
                            Slowly but surely, your retching comes to an end.
                            Holding your breath, you walk past the scene of the crime, 
                            looking everywhere [b]but[/b] at the stagnating puddle in between the barrels to check whether the hoodlums are still around or not.
                            The street seems empty.
                            You think they've left.
                            You [b]hope[/b] they've left.
                        [/p]
                        [p]
                            You sneak out of the alleyway, before quickly and quietly making your way back to a more [i]hospitable[/i] part of town.
                            While, at first, you curse the small, chilly morning wind that picks up,
                            you soon learn to appreciate it for washing the stench of piss off of you,
                            and for fully drying your ${clothes ? `clothes` : `body`} in a matter of minutes.
                        [/p]
                        [p]
                            After removing yourself from the [i]brothel district[/i] proper,
                            you start walking back towards the town square,
                            where your [i]urban adventure[/i] began.
                            As you begin to near it, you notice the sun slowly rising above the rooftops.
                            You must have lost track of time while exploring the city.
                            Or maybe you were left retching in the alleyway for a longer time than you care to remember.
                            Either way, you guess you won't be sleeping tonight.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded, giving way to a sense of [b]vulnerability[/b].
                            You are but a lone, weak, small, kobold in a big city filled with people who can and [b]want[/b] to maim,
                            hurt, kill and perhaps even [b]rape[/b] you.
                            If you let your guard down for even one moment, it could cost you your money, your life or both.
                            And next time, the drunk taking a leak against your hiding spot might actually [b]notice[/b] you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.hideStay.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.hideStay.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\hide\stay\NightTwoHideStayContinue.js

// scenes\Varanar\NightTwo\flee\alley\hide\stay\NightTwoHideStay.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoHideStay(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoHideStayContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var pants = ctrl.misc.pants();
        var shirt = ctrl.misc.shirt();
        var clothes = shirt || pants;
        var toReturn = `[p]
                            While you would [b]love[/b] to [b]rip his nuts off[/b] with your sharp claws,
                            you simply can't risk the human calling his buddies over before he goes down.
                            There's no way you can take on all of them, unless they each approach one of at atime with their pants down and their dicks out.
                            While you [b]hate[/b] being forced to do this, you're just going to have to sit here and be quiet while the dehydrated drunkard does his thing.
                        [/p]
                        [p]
                            The man lets out a loud sigh of relief as he continues to let out an entire night's worth of booze-turned-urine,
                            a considerable puddle of the stuff forming on the ground right next to you.
                            You try to scooch over, to scuttle to the side just a little bit more to avoid the absolutely [b]revolting[/b] liquid,
                            yet after managing to spare yourself for half an inch more, there is simply nowhere left to run.
                            You almost [b]vomit[/b] as you feel the lukewarm fluid flow around your scaly hand, placed on the ground to keep you balanced,
                            but knowing your very life is at stake, you remain staunch and stalwart, resolute and unwavering.
                        [/p]
                        [p]
                            However, there are limits to your determination, limits which are put to the test as the ruffian begins to wobble on his feet,
                            his steady stream of urine erratically flailing about, splatterings of piss ${clothes ? `matting your clothing` : `befouling your scales`}.
                            As the human nearly topples over fully, his torrent deviates the most, the flow splashing directly against your face for just a brief moment,
                            completely [b]sullying[/b] your once-clean visage, drenching you in foul-smelling secretions,
                            your stomach managing to retain its contents by your sheer will to live alone.
                        [/p]
                        [p]
                            While your [b]hatred[/b] for the hoodlum runs deep, and the urge to attack him, to [b]maim[/b] him,
                            to [b]scar[/b] him for [/b]life[/b] as punishment for outright taking a [b]piss[/b] on you,
                            you don't even [b]dare[/b] to wipe your face out of fear that he'll notice.
                            His warm urine nauseatingly runs down your forehead and alongside your muzzle, before dripping to the ground below.
                            Luckily, none of it runs down your neck.
                        [/p]
                        [p]
                            Not long after being outright [b]sprayed[/b], the flood of vile fluid begins to falter.
                            After squirting out a final few jets of piss, and shaking off any lingering droplets,
                            the man repackages his flaccid cock, sloppily buttons up his pants, and walks away,
                            leaving you huddled up in a puddle of his slowly cooling mess.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.hideStay.continued() ? "disabled" : "";
        var isSelected = ctrl.hideStay.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hideStay.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.hide.get() !== NightTwoHideChoice.STAY) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.hideStay.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.hide.get() !== NightTwoHideChoice.STAY) {
            return;
        }

        next.registerListeners();

        if (ctrl.hideStay.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\hide\stay\NightTwoHideStay.js

// scenes\Varanar\NightTwo\flee\alley\hide\NightTwoHide.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoHide(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var stayHiddenId = arguments.callee.name + "ButtonOne";
    var attackId = arguments.callee.name + "ButtonTwo";

    var toStayHidden = new NightTwoHideStay(ctrl);
    var toAttack = new NightTwoHideAttack(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            It's too late to run, and you can't get over a wall that high.
                            The barrels.
                            Those are your best option.
                            Combined with the inherent darkness of the alleyway, the cover provided by them is sure to keep you hidden from a gang of drunkards.
                            At least you hope it will.
                        [/p]
                        [p]
                            You crouch down, crawling in between two of the barrels.
                            They [b]reek[/b] of refuse, probably containing something rotten, or maybe a family of rats made their home inside.
                            You have no choice [b]but[/b] to tolerate the stench, for now.
                            Concealed between a pair of barrels and a stone wall, you slowly turn around in your hiding spot,
                            facing the street-end to monitor for any new developments.
                        [/p]
                        [p]
                            A few members of the group pass by without even casting a glance down the alley in which you are hidden.
                            Your plan is working [b]perfectly[/b].
                            Heck, since they're not even looking this way, you didn't even need to bother with squeezing in between these smelly containers.
                            Taking your safety for granted, you let down your guard for a few seconds,
                            covering your snout to prevent the worst of the smell from reaching your nostrils.
                        [/p]
                        [p]
                            Your gaze returning to the entrance of the alley, a small, inadvertent gasp escapes your lips.
                            One of the men is drowsily standing at the far-end of the corridor, peering into it with half-attentive eyes.
                            You lower the hand covering your nose to your mouth, to muffle the quiet noise of your audible surprise.
                            Scared shitless, you reduce your posture as much as you can, curling up into a tiny little ball in hopes that he hasn't seen you [i]yet[/i].
                        [/p]
                        [p]
                            Luckily, humans are [b]not[/b] as skilled as kobolds are when it comes to seeing in the dark.
                            And even if they [b]were[/b], this one looks [b]so[/b] drunk he can barely stand on his own two feet,
                            let alone spot a tiny little lizard like yourself concealed behind a set of barrels in a pitch-black alley.
                            Still, despite not having seen you, the man casually saunters forwards, approaching your position with an unsteady gait.
                        [/p]
                        [p]
                            In spite of needing to brace himself against one of the walls to stay upright,
                            the human walks up to the very set of barrels you are hiding behind.
                            [b]Like he knows you're there.[/b]
                            Your heart races a million miles an hour.
                            You can [b]smell[/b] the stench of [b]booze[/b] lingering about him even through the odour of rot.
                            You let out a small whimper, almost betraying your location.
                            Luckily, he doesn't seem to notice.
                            You're still safe.
                            He [b]doesn't[/b] know.
                        [/p]
                        [p]
                            The man steadies himself against one of the barrels you're hiding behind, his boot-clad feet mere inches from your face.
                            He moves one of his hands to his groin, undoing the buttons of his linen pants.
                            Wait, what?
                            Reaching into his trousers, the man deftly lowers his underwear, exposing his [b]huge[/b], heavily oversized flaccid member,
                            an unwashed, manly fragrance mixing with the rest of the unpleasant scents.
                            Taking hold of his length, he points it downwards, not directly at you, but in your general vicinity.
                            [i]Oh no.[/i]
                        [/p]
                        [p]
                            Like the unsavory drunk that he is, the man releases his bladder,
                            a dark-yellow stream of pungent piss splattering against the wall besides you,
                            its hefty, acrid fragrance immediately [b]overpowering[/b] all of the other scents,
                            clogging your nose to the point of nearly making you [b]retch[/b].
                            That filthy human wasn't even looking for [b]you[/b], just for a place to take a leak!
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildStayHiddenChoice() {
        var disabled = ctrl.hide.has() ? "disabled" : "";
        var isSelected = ctrl.hide.get() === NightTwoHideChoice.STAY;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${stayHiddenId}" ${disabled}>Stay hidden</button>`;

        var tooltip = global.parser.parse(`Weather through the disgusting ordeal. He'll leave when he's done, right?`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildAttackChoice() {
        var disabled = ctrl.hide.has() ? "disabled" : "";
        var isSelected = ctrl.hide.get() === NightTwoHideChoice.ATTACK;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${attackId}" ${disabled} style="width: 160px;">Attack the man</button>`;

        var tooltip = global.parser.parse(`With his pants down, you have a clear shot at the human's most [i]delicate[/i] parts.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildStayHiddenChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildAttackChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + stayHiddenId), $("#" + attackId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerStayHiddenListener() {
        $("#" + stayHiddenId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hide.set(NightTwoHideChoice.STAY);
            loadNextPart(toStayHidden);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerAttackListener() {
        $("#" + attackId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hide.set(NightTwoHideChoice.ATTACK);
            loadNextPart(toAttack);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.alley.get() !== NightTwoAlleyChoice.HIDE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toStayHidden.build();
        part += toAttack.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.alley.get() !== NightTwoAlleyChoice.HIDE) {
            return;
        }

        toStayHidden.registerListeners();
        toAttack.registerListeners();

        if (ctrl.hide.has()) {
            return;
        }

        registerStayHiddenListener();
        registerAttackListener();
    };
}

var NightTwoHideChoice = {
    STAY: 1,
    ATTACK: 2
};

// END scenes\Varanar\NightTwo\flee\alley\hide\NightTwoHide.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\stay\NightTwoAlleyStayContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyStayContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var underwear = ctrl.misc.underwear();
        var pants = ctrl.misc.pants();
        var shirt = ctrl.misc.shirt();
        var clothing = underwear || pants || shirt;
        var toReturn = `[p]
                            As soon as the man has left the alleyway, you rise out of the puddle of pee.
                            Tears flow liberally from your eyes.
                            You feel [b]unclean[/b], [b]defiled[/b] from head to toe.
                            You wipe your face as best you can, but even dry, your scales seem [b]filthy[/b].
                            There's a horrid [b]stench[/b] lingering about, a revolting fragrance so intense you fear it may seep into your very skin.
                        [/p]
                        [p]
                            You clean off your arms, your legs and your tail as best you can.
                            Luckily, the digusting yellow fluid rolls easily off your scales, dripping down to the ground.
                            While warm at first, the liquid is quickly starting to cool down, leaving you chilly and cold on top of being absolutely [b]revolted[/b].
                            Despite cleaning your body as best as you can, the sleazy, unsanitary feeling remains.
                            ${clothing ? `Your clothes are tainted.
                            You don't want to take them off in the middle of town.
                            They'll just have to dry on their own.` : ""}
                        [/p]
                        [p]
                            While completely disheartened, reduced to a sobbing mess of a piss-stained lizard,
                            you are glad that you escaped that encounter relatively unharmed.
                            After a few solid minutes of bawling your eyes out, you wipe your tears, and stumble towards the alleyway's entrance.
                            The men are nowhere around to be seen.
                            Having traded your dignity for your life, you start walking back towards the town square.
                        [/p]
                        [p]
                            While, at first, you curse the cool morning wind that picks up,
                            you soon learn to appreciate it for washing the stench of piss off of you,
                            and for somewhat helping to dry your ${clothing ? `clothes` : `body`}.
                            As you begin to near your destination, you notice the sun slowly rising above the rooftops.
                            You must have lost track of time while exploring the city.
                            Or maybe you were left crying in the alleyway for a longer period than you care to remember.
                            Either way, you guess you won't be sleeping tonight.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded, giving way to a sense of [b]vulnerability[/b].
                            You are but a lone, weak, small, kobold in a big city filled with people who can and [b]want[/b] to maim,
                            hurt, kill and perhaps even [b]rape[/b] you.
                            If you let your guard down for even one moment, it could cost you your money, your life or both.
                            You'd worry about losing your pride, but you fear you have little of that left as it is.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.alleyStay.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.alleyStay.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\fail\stay\NightTwoAlleyStayContinue.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\stay\NightTwoAlleyStay.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyStay(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoAlleyStayContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.alleyPiss = true;
        var underwear = ctrl.misc.underwear();
        var pants = ctrl.misc.pants();
        var shirt = ctrl.misc.shirt();
        var clothing = underwear || pants || shirt;
        var toReturn = `[p]
                            You don't have a choice.
                            The hoodlum is twice your size,
                            and he has a bunch of equally menacing friends waiting just outside of the alleyway to come [b]fuck you up[/b] unless you do as he says.
                            You glare up at the human.
                            He smirks down at you.
                            Taking your silence for acceptance, he grabs his humongous shaft, and points it directly at you.
                            Before you have a chance to protest, the drunkard  releases his bladder, and a thick stream of dark-yellow, acrid,
                            foul-smelling urine hits you straight on the forehead, splattering across the entirety of your face.
                        [/p]
                        [p]
                            You immediately close your eyes and your mouth, your tooth-baring scowl replaced by a pathetic, submissive frown.
                            The man lets out a domineering chuckle, as he begins to shift his member about, directing the stream of disgusting, lukewarm liquid.
                            First, he ensures your entire face has been coated, from the top of your head to the underside of your muzzle.
                            You gag and retch as a few droplets find their way inside of you nostrils,
                            an absolutely [b]vile[/b] taste building up at the back of your throat, as though you can [b]taste[/b] it.
                        [/p]
                        [p]
                            You lift up your hands and your arms in an attempt to cover your face, trying to [b]block[/b] the flow, but the human simply lowers his aim,
                            ${clothing ? `completely [b]drenching[/b] your clothing` : `soaking your body`} in piss instead.
                            A small puddle of the stuff forms underneath you, which makes you feel even [b]more[/b] dirty and uncomfortable,
                            as though you're [b]wallowing[/b] in it.
                            On the very [b]verge[/b] of throwing up, you finally feel the flow starting to lessen,
                            the once-powerful stream blasting across your chest reduced to a minor trickle spilling down onto your legs.
                        [/p]
                        [p]
                            "[color=dred]You look good in yellow.[/color]" the man taunts, pulling up and re-buttoning his pants.
                            Snickering at his own joke, he realizes you aren't sharing his jolly sentiment.
                            He crudely spits on the ground next to you, his saliva quickly mixing with the expanding pool of urine.
                            "[color=dred]Next time, I'll let my buddies take a leak on you as well, you ungrateful [b]bitch[/b].
                            That's all you [b]filthy lizards[/b] are good for.[/color]"
                            That last part being more of a mumbling to himself, the man turns around and begins to walk away,
                            leaving you crying in the middle of a puddle of his piss.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.alleyStay.continued() ? "disabled" : "";
        var isSelected = ctrl.alleyStay.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alleyStay.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.STAY) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.alleyStay.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.STAY) {
            return;
        }

        next.registerListeners();

        if (ctrl.alleyStay.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\fail\stay\NightTwoAlleyStay.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\eager\NightTwoAlleyEagerContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyEagerContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Dawn breaks.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.alleyEager.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.alleyEager.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\fail\eager\NightTwoAlleyEagerContinue.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\eager\NightTwoAlleyEager.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyEager(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoAlleyEagerContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Eagerly gulp down piss. Slut.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.alleyEager.continued() ? "disabled" : "";
        var isSelected = ctrl.alleyEager.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.hideAttack.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.EAGER) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.alleyEager.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.EAGER) {
            return;
        }

        next.registerListeners();

        if (ctrl.alleyEager.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\fail\eager\NightTwoAlleyEager.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\bite\NightTwoAlleyBite.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyBite(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    var idsToRemove = [
        "NightTwoAlleyBite",
        "NightTwoAlleyRunFail",
        "NightTwoAlleyRun",
        "NightTwoAlley",
        "NightTwoFlee",
        "NightTwoRunIn",
        "NightTwoStage"
    ];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Who the [b]hell[/b] does this guy think he is?!
                            Does he really expect you to [b]stay put[/b] and let him [b]take a piss on you[/b]?!
                            He might have met some meek kobolds in his lifetime, but you're [b]not[/b] one of those.
                            You're a fighter.
                            You don't spit [b]or[/b] swallow.
                            You [b]bite[/b].
                        [/p]
                        [p]
                            Glaring upwards at the man, you take a mental snapshot of the smug, coy look on his face.
                            You're going to [b]enjoy[/b] his look of anguish after you [b]bite his dick off[/b].
                            You shift around a little, positioning yourself in a better way to [b]lash out[/b].
                            "[color=dred]Quit fidgetin'.
                            It's comin', just you wait.[/color]"
                            the hoodlum comments, mistaking your assault-preparations for anxiousness.
                            He probably thinks you're going to attempt to run away.
                            Poor him.
                        [/p]
                        [p]
                            You bide your time for a few more moments,
                            trying to find a good balance between getting the ruffian to [i]relax[/i] so you can catch him off guard,
                            without waiting [i]too[/i] long, lest he start [i]doing his thing[/i].
                            You have no intentions of getting peed on, at least not today.
                            Mere moments before the scoundrel was about to release his bladder, you make your move.
                        [/p]
                        [p]
                            You lift yourself off the ground, and [b]leap[/b] forwards in a single, smooth move.
                            Your muzzle is wide open, and before he even knows what's going on, you've taken the man's disgusting, uncut and unwashed member into your mouth.
                            His thick shaft barely even [b]fits[/b] in your mouth.
                            Ere he has a chance to piss down your throat, you [b]bite down[/b] as [b]hard[/b] as you can,
                            your teeth sinking into the soft flesh of the human's cock, severing off more than a few inches.
                        [/p]
                        [p]
                            The hoodlum [b]cries[/b] out in [b]pain[/b], screaming loud enough to wake up the entire neighbourhood as he slowly steps backwards,
                            tearing the final few bits of flesh holding the tip of his member in place himself.
                            Blood gushes from the remainder of his length, splattering your face.
                            For a few moments, the human clutches his groin in disbelief.
                            Then, unable to process the pain, he falls down to the ground, knocked unconscious through sheer pain alone.
                        [/p]
                        [p]
                            Alas, it seems like the other delinquents [b]heard[/b] their comrade's cry for help.
                            The entire group [b]rushes[/b] into the alleyway, to find a kobold crouching over their downed friend's body.
                            You throw a wicked grin their way, baring your teeth and the fleshy organ in between them.
                            To make your point even more clear, you spit the severed cock-length out onto the unconscious ruffian's body.
                        [/p]
                        [p]
                            "[color=bred]That kobold bit off Carl's [b]dick[/b]! [b]Let's get'em![/b][/color]" the ringleader of the pack proclaims.
                            It is at that point you realize you might have [i]bitten off a bit more than you can chew[/i].
                            Stuck in self-blaming thoughts, you remain frozen in place as the gang rushes towards you.
                            The first human to arrive crudely [b]kicks[/b] you in the face, sending you flying backwards,
                            the rear-end of your head painfully [b]crashing[/b] into the brick wall, almost knocking you out from the get-go.
                        [/p]
                        [p]
                            Alas, you are spared the mercy of unconsciousness.
                            Left reeling from the initial blow, you clutch your hurting face, your nose bleeding profusely.
                            The men give you no respite.
                            Mere moments after kicking you into the wall, a second human jumps on top of you, and begins to [b]pound[/b] your body with blow after blow,
                            using both of his fists to [b]brutalize[/b] your upper body, while pinning you down using nothing but his own weight,
                            preventing you from so much as covering yourself with your arms.
                        [/p]
                        [p]
                            The brutal torment goes on for what feels like [b]ages[/b].
                            You can feel your bones [b]cracking[/b], breaking under the incredible strain of repeated, full-strength blows.
                            Blood begins to pile up at the back of your throat as one of your ribs painfully [b]punctures[/b] your lungs.
                            You try to [b]beg[/b] the man for mercy, yet nothing but a bloody gurgle escapes your lips.
                        [/p]
                        [p]
                            Growing tired, the man rises from your battered frame.
                            The rest of the group is [b]far[/b] from done with you, however.
                            Your next tormentor steps forwards.
                            Looking to get a rise out of you, and wanting to look cool in front of his friends,
                            he cruelly plants his foot in between your slightly parted legs, brutally [b]kicking[/b] you in the groin,
                            harsh enough for the tip of his boot to [b]sink[/b] into your slit.
                            Horrible pain like nothing you've [b]ever[/b] felt before shoots up your spine,
                            but there's barely enough air in your blood-filled lungs to keep you [b]conscious[/b], let alone to voice your agony with.
                        [/p]
                        [p]
                            Upset that he didn't get to hear you [b]squeal[/b], the cruel hoodlum starts a series of [b]kicks[/b],
                            focusing on your lower half, bruising and severely [b]injuring[/b] everything from your stomach to your tail,
                            on which he [b]stomps[/b] on repeatedly.
                            At first, you try your hardest to move your tail out of the way, but after the second time his heavy boot comes down onto it,
                            something [b]snaps[/b], and a new wave of [b]anguish[/b] floods your body.
                            You find yourself unable to move your tail from there on out.
                        [/p]
                        [p]
                            The third delinquent focuses on your limbs.
                            He grabs your arm, and pins it.
                            He demands you [b]beg[/b] him for mercy, yet you manage to utter nothing but a painful, blood-filled cough.
                            Without warning, he cruelly [b]twists[/b] the appendage in his grasp, bending your arm in a way it was [b]never[/b] meant to go.
                            With a loud, sickening [b]crack[/b], your bones break in half.
                            A fresh wave of tears flow from your abused eyesockets as your arm is snapped like a twig.
                            The pain is [b]horrendous[/b], but for whatever reason, your mind [b]refuses[/b] to let you pass out.
                        [/p]
                        [p]
                            Every member of the group takes a turn.
                            Some simply batter your face, others break your fingers, knock out your teeth or torture your feet.
                            One of the humans outright [b]cuts off[/b] the tip of your tail, maiming you like you did their friend.
                            Then, for a moment, the constant beatings come to an end.
                            Barely conscious, you blurrily see the ringleader of the group approach through your one still-functioning eye.
                            He presses his shoe-clad foot onto your throat, cutting off what little air intake you had left.
                            Then, he twists his heel.
                            You hear something crack.
                            Your torment is finally over.
                            You welcome death, and death welcomes you.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Dead end. Just like the alleyway.
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400, function () {
            var main = $("#main");
            main.scrollTop(main[0].scrollHeight - main.height() - 5); // Fix scrolling
        });
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.misc.unsetAll();
            ctrl.alleyRunFail.undo();
            ctrl.alleyRunFail.unset();
            ctrl.alley.unset();
            ctrl.flee.undo();
            ctrl.flee.unset();
            ctrl.runin.unset();
            ctrl.stage.unset();
            for (var i = 0; i < idsToRemove.length; i++) {
                $("#" + idsToRemove[i]).remove();
            }
            loadNextPart(new NightTwoStage(ctrl));
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.BITE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.alleyRunFail.get() !== NightTwoAlleyRunFailChoice.BITE) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\fail\bite\NightTwoAlleyBite.js

// scenes\Varanar\NightTwo\flee\alley\run\fail\NightTwoAlleyRunFail.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyRunFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var biteId = arguments.callee.name + "ButtonOne";
    var stayId = arguments.callee.name + "ButtonTwo";
    var eagerId = arguments.callee.name + "ButtonThree";

    var toBite = new NightTwoAlleyBite(ctrl);
    var toStay = new NightTwoAlleyStay(ctrl);
    var toEager = new NightTwoAlleyEager(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var agiFail = ctrl.misc.agility() < 6;
        var toReturn = `[p]
                            You envision your way up along the side of the wall on the far end of the alley.
                            First you'll run towards it, then you intend to [b]jump[/b], and using the momentum gained from your dash,
                            you'll kick yourself off of one of the walls, ending up high enough to latch on to the top of the brick wall.
                            From then, it's a matter of pulling yourself up, and after that you'll be in the clear.
                        [/p]
                        [p]
                            With your plan visualized, it's time to put it into motion.
                            You [b]sprint[/b] forwards, accelerating at a rapid pace.
                            Running past the last of the barrels, you [b]leap[/b] from the ground as high up as you can.
                            Mid-air, you press your right foot against the alley-wall, and expertly push yourself up even higher,
                            converting horizontal momentum into vertical lift.
                        [/p]`;

        if (agiFail) {
            toReturn += `[p]
                            Alas, you seem to have misjudged the actual height of the wall.
                            You reach for the top, yet come just a few inches short of being able to grab onto it.
                            Without anything to grip onto, your entire body smacks into the solid barrier, and you fall to the ground.
                            Slightly dazed, you slump down with your back against the wall, just in time to witness one of the humans entering the alleyway.
                        [/p]
                        ${global.ui.condFail(`Agility below 6`)}`;
        } else {
            toReturn += `[p]
                            Exactly as planned, you narrowly reach the height required to grab the top of the wall,
                            your claws digging in to the mortar in between the bricks ever so slightly.
                            Despite trying with all your might, however, you utterly [b]fail[/b] to pull your meagre bodyweight up onto the barrier.
                            You struggle in vain for nearly half a minute to try and lift yourself up higher, before your arms can carry your weight no longer.
                            You fall to the ground, landing painfully on your rear.
                            Slightly dazed, you slump down with your back against the wall, just in time to witness one of the humans entering the alleyway.
                        [/p]
                        ${global.ui.condFail(`Strength below 6`)}`;
        }

        toReturn += `   [p]
                            Paralyzed with a mixture of fear and stinging pain, you are frozen in place as the ruffian slowly approaches.
                            He clearly knows you're there, staring directly at your downed form, yet he isn't calling for his friends.
                            Perhaps he wants the pleasure of [i]beating you to death[/i] all for himself.
                            You [i]should[/i] ready yourself for combat,
                            yet your body and your mind both feel [b]far[/b] too exhausted and utterly [b]defeated[/b] to even [b]attempt[/b] to put up a fight.
                        [/p]
                        [p]
                            The scoundrel stops just a few feet in front of you.
                            Glaring down at you with angry, hate-filled eyes, he speaks.
                            "[color=dred]You're that lizard we're chasin'.[/color]"
                            His voice is bass, and some of his words come out slurred due to his heavy intoxication.
                            Still, not unfamiliar with being drunk yourself, you understand every word he's saying.
                            "[color=dred]My friends want to [b]kill[/b] you.
                            I think that'd be a [b]waste[/b].
                            You'll make a fine [b]kobold whore[/b] one day,
                            and Varanar could use more cheap pieces of [b]ass[/b].[/color]"
                        [/p]
                        [p]
                            A sadistic smile forms atop of the bulky hoodlum's face.
                            He moves one of his hands towards his groin, and casually begins to unbutton his pants.
                            "[color=dred]I came into this alley to take a leak,
                            and that's exactly what I'm gonna do.
                            You stay put like the lil' [b]slut[/b] you are,
                            and I won't have a reason to call my buddies over here, you got that?[/color]"
                            Without even bothering to wait for a response, the man promptly lowers his underwear,
                            the stench of unwashed human groin creeping into your nostrils as his [b]huge[/b],
                            heavily oversized flaccid member is made bare to you.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildBiteChoice() {
        var disabled = ctrl.alleyRunFail.has() ? "disabled" : "";
        var isSelected = ctrl.alleyRunFail.get() === NightTwoAlleyRunFailChoice.BITE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${biteId}" ${disabled}>Bite</button>`;

        var tooltip = global.parser.parse(`What the hell does this guy think he's doing?! Bite his dick off. That'll show him.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildStayChoice() {
        var disabled = ctrl.alleyRunFail.has() ? "disabled" : "";
        var isSelected = ctrl.alleyRunFail.get() === NightTwoAlleyRunFailChoice.STAY;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${stayId}" ${disabled}>Stay still</button>`;

        var tooltip = global.parser.parse(`You don't want to get bludgeoned to death by a bunch of angry, drunken humans.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildEagerChoice() {
        var disabled = ctrl.alleyRunFail.has() ? "disabled" : "";
        var isSelected = ctrl.alleyRunFail.get() === NightTwoAlleyRunFailChoice.EAGER;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${eagerId}" ${disabled}>Open your mouth</button>`;

        var tooltip = global.parser.parse(`You thought this guy was going to [b]kill[/b] you. Now that you know he isn't, you might as well have some fun.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildBiteChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildStayChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + biteId), $("#" + stayId), $("#" + eagerId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerBiteListener() {
        $("#" + biteId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alleyRunFail.set(NightTwoAlleyRunFailChoice.BITE);
            loadNextPart(toBite);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerStayListener() {
        $("#" + stayId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alleyRunFail.set(NightTwoAlleyRunFailChoice.STAY);
            loadNextPart(toStay);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerEagerListener() {
        $("#" + eagerId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alleyRunFail.set(NightTwoAlleyRunFailChoice.EAGER);
            loadNextPart(toEager);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.alleyRunFail.change();
        part += buildChoice();
        part += `</div>`;
        part += toBite.build();
        part += toStay.build();
        part += toEager.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        toBite.registerListeners();
        toStay.registerListeners();
        toEager.registerListeners();

        if (ctrl.alleyRunFail.has()) {
            return;
        }

        registerBiteListener();
        registerStayListener();
        registerEagerListener();
    };
}

var NightTwoAlleyRunFailChoice = {
    BITE: 1,
    STAY: 2,
    EAGER: 3
};

// END scenes\Varanar\NightTwo\flee\alley\run\fail\NightTwoAlleyRunFail.js

// scenes\Varanar\NightTwo\flee\alley\run\succeed\NightTwoAlleyRunSuccessContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyRunSuccessContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            After removing yourself from the [i]brothel district[/i] proper,
                            you start walking back towards the town square,
                            where your [i]urban adventure[/i] began.
                            As you begin to near it, you notice the sun slowly rising above the rooftops.
                            You must have lost track of time while exploring the city.
                            Or maybe you were being chased by those men for a longer time than you remember.
                            Either way, you guess you won't be sleeping tonight.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded, giving way to a sense of [b]vulnerability[/b].
                            You are but a lone, weak, small, kobold in a big city filled with people who can and [b]want[/b] to maim,
                            hurt, kill and perhaps even [b]rape[/b] you.
                            If you let your guard down for even one moment, it could cost you your money, your life or both.
                            And next time you might not be so lucky as to be able to freerun your way up a wall without screwing up.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.alleyRunSuccess.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.alleyRunSuccess.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\succeed\NightTwoAlleyRunSuccessContinue.js

// scenes\Varanar\NightTwo\flee\alley\run\succeed\NightTwoAlleyRunSuccess.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyRunSuccess(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoAlleyRunSuccessContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You envision your way up along the side of the wall on the far end of the alley.
                            First you'll run towards it, then you intend to [b]jump[/b], and using the momentum gained from your dash,
                            you'll kick yourself off of one of the walls, ending up high enough to latch on to the top of the brick wall.
                            From then, it's a matter of pulling yourself up, and after that you'll be in the clear.
                        [/p]
                        [p]
                            With your plan visualized, it's time to put it into motion.
                            You [b]sprint[/b] forwards, accelerating at a rapid pace.
                            Running past the last of the barrels, you [b]leap[/b] from the ground as high up as you can.
                            Mid-air, you press your right foot against the alley-wall, and expertly push yourself up even higher,
                            converting horizontal momentum into vertical lift.
                        [/p]
                        [p]
                            Exactly as planned, you narrowly reach the height required to grab the top of the wall,
                            your claws digging in to the mortar in between the bricks ever so slightly.
                            Using all of your strength, you deftly begin to pull yourself upwards,
                            lifting your light frame up onto the barrier, coming to a seated rest on top of it.
                        [/p]
                        ${global.ui.condSucc(`Strength and agility greater than 5`)}
                        [p]
                            Just in time, or so it seems.
                            One of the men chasing you wanders into the alleyway.
                            Whether he didn't see you perched up high, or if he simply had no intention of calling for his friends, you will never know.
                            You lower yourself on the opposite end of the brick wall, and with a feline-resembling grace, you hop down to the ground once more.
                            With the group shaken off, for now, at least, you quickly and quietly make your way to [i]less hostile[/i] parts of Varanar.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.alleyRunSuccess.continued() ? "disabled" : "";
        var isSelected = ctrl.alleyRunSuccess.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alleyRunSuccess.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.alleyRunSuccess.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.alleyRunSuccess.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\succeed\NightTwoAlleyRunSuccess.js

// scenes\Varanar\NightTwo\flee\alley\run\NightTwoAlleyRun.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlleyRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoAlleyRunFail(ctrl);
    var toSuccess = new NightTwoAlleyRunSuccess(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.misc.agility() > 5 && ctrl.misc.strength() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.alley.get() !== NightTwoAlleyChoice.RUN) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.alley.get() !== NightTwoAlleyChoice.RUN) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\alley\run\NightTwoAlleyRun.js

// scenes\Varanar\NightTwo\flee\alley\NightTwoAlley.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoAlley(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var hideId = arguments.callee.name + "ButtonOne";
    var runId = arguments.callee.name + "ButtonTwo";

    var toHide = new NightTwoHide(ctrl);
    var toRun = new NightTwoAlleyRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Desperate to get away from the men, your mind forms a fickle escape plan.
                            You turn a corner, and immediately [b]bolt[/b] for the nearest alleyway.
                            Stepping into the tight, unlit corridor, you let your eyes adjust to the complete darkness for a few moments, your superb nightvision kicking in.
                            The alley you've entered is a dead end, the other side blocked off by a pretty high brick barrier.
                            A collection of barrels are unorderly arrayed against the leftmost wall, leaving barely enough room for a full-grown human to squeeze on by.
                        [/p]
                        [p]
                            This place [b]stinks[/b] like rotten fish.
                            You consider backing out and either picking a different alleyway, or just ditching the idea of hiding completely, but alas, it is already too late.
                            Judging by the noise coming from the street, the men have followed in your footsteps, and they are currently out looking for you.
                            If you remain standing in the entrance, they'll undoubtedly find you.
                            You need to make a decision, and quick.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildHideChoice() {
        var disabled = ctrl.alley.has() ? "disabled" : "";
        var isSelected = ctrl.alley.get() === NightTwoAlleyChoice.HIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hideId}" ${disabled}>Hide</button>`;

        var tooltip = global.parser.parse(`Conceal yourself behind the barrels.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRunChoice() {
        var disabled = ctrl.alley.has() ? "disabled" : "";
        var isSelected = ctrl.alley.get() === NightTwoAlleyChoice.RUN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${runId}" ${disabled}>Run</button>`;

        var tooltip = global.parser.parse(`Try to parkour up and over the wall.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildRunChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHideChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + hideId), $("#" + runId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerRunListener() {
        $("#" + runId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alley.set(NightTwoAlleyChoice.RUN);
            loadNextPart(toRun);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerHideListener() {
        $("#" + hideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.alley.set(NightTwoAlleyChoice.HIDE);
            loadNextPart(toHide);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.flee.get() !== NightTwoFleeChoice.HIDE) {
            return "";
        }

        global.ui.applyTheme("varanar_alley");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toRun.build();
        part += toHide.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.flee.get() !== NightTwoFleeChoice.HIDE) {
            return;
        }

        toRun.registerListeners();
        toHide.registerListeners();

        if (ctrl.alley.has()) {
            return;
        }

        registerRunListener();
        registerHideListener();
    };
}

var NightTwoAlleyChoice = {
    HIDE: 1,
    RUN: 2
};

// END scenes\Varanar\NightTwo\flee\alley\NightTwoAlley.js

// scenes\Varanar\NightTwo\flee\run\fail\die\NightTwoRunDeath.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunDeath(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    var idsToRemove = [
        "NightTwoRunDeath",
        "NightTwoRunBeatdown",
        "NightTwoKeepRunning",
        "NightTwoFlee",
        "NightTwoRunIn",
        "NightTwoStage"
    ];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Desperate to get away from the men, you burst into a sprint the moment you turn a corner,
                            hoping to be able to outrun them before they even [i]know[/i] you're escaping.
                            While your gambit does pay off in the form of a head start, sadly, you don't manage to shake the group from the get-go.
                            Noticing their lizard-prey running away,
                            the men drop their tight-packed group-cohesion and begin to outright [b]chase[/b] after you.
                        [/p]
                        [p]
                            You run from street to street, periodically checking behind you to make sure they haven't given up yet.
                            While far bigger than you, and probably able to outrun you whilst sober, their heavy inebriation hinders the belligerent group significantly.
                            A couple of pursuers tumble over their own feet, falling to the ground, being left behind by their peers.
                            A few of the men get too sick to keep pursuing you, retreating into alleyways to empty their bellies of food and drink.
                            Some of them are simply too weary to keep up pursuit.
                            Initially numbering high in the single-digits,
                            it takes only a few solid minutes of running for their [i]pack[/i] to be reduced to three hard-boiled men.
                        [/p]
                        [p]
                            But still, even [b]one[/b] of them can [b]cave your head in[/b] if they manage to catch you.
                            You aren't out of the woods until you can shake off [b]all three[/b] of them.
                            And you get the feeling that all this running is making them [b]even more frustrated[/b].
                            There's no doubt in your mind: your very life is at stake here.
                            And the humans chasing you are [b]determined[/b] to rid their city of another [b]pest[/b] infesting it.
                        [/p]
                        [p]
                            You push your body to its very limits, sprinting away as fast as you can.
                            You can't hold out for much longer.
                            You can feel the end of your rope approaching.
                            You hope they're every bit as tired as you are.
                            That their bodies are as much on the verge of collapsing as your tiny, scaled frame is.
                        [/p]
                        [p]
                            Crossing into a more dignified area of Varanar, one of the few districts with cobblestone roads,
                            you feel the last of your energy leaving your body.
                            You try to keep running, you [b]really[/b] do, but your legs simply can no longer carry you.
                            As the flat dirt under your feet suddenly makes way for a more uneven, rock-like surface,
                            you lose your balance and plummet to the ground.
                            Time itself seems to slow down as your body falls limply onto the cold, hard street, completely exhausted.
                        [/p]
                        ${global.ui.condFail(`Insufficient stamina`)}
                        [p]
                            You roll onto your side right on time to witness one of your pursuers rapidly approaching.
                            Using the hefty momentum from his drunken running as a catalyst, he cruelly plants the tip of his heavy boot into your stomach,
                            kicking you [b]so[/b] hard your leathery body is lifted off the ground as a whole.
                            You scream in agony as you are hurled through the air, your panicked shout coming to an end as you fall down once more,
                            your frame heavily impacting with the stone street a second time, painfully skimming across it for a few more inches before finally coming to rest.
                        [/p]
                        [p]
                            Seriously hurt, you clutch your abdomen and slowly curl up into a ball.
                            You're pretty certain the [b]immense[/b] force behind the kick [b]broke[/b] something inside of you.
                            As you whimper and cry, the other two men that were chasing you catch up with their peer.
                            They advance upon you, and waste no time in resuming their brutal clobbering.
                        [/p]
                        [p]
                            One of them leans down, and mercilessly begins to [b]pound[/b] your side, battering your ribs with blow after blow of his powerful, clenched fist.
                            A second human begins to repeatedly kick your back with all of his might, your spine on the verge of [b]snapping[/b] in half.
                            The third and final abuser concerns himself with your face,
                            alternating between using his hands and his shoe-clad feet to batter every part of your visage from your forehead to your snout.
                        [/p]
                        [p]
                            You [b]beg[/b] for mercy, but the humans seem determined not to stop until you're [b]dead[/b].
                            Your eyes are beaten shut to the point where you can no longer [b]see[/b],
                            but you [b]hear[/b] someone walking by over the gut-wrenching sounds of your bones breaking and your scales bruising.
                            You cry out for help, hoping it's a kind-hearted guard, yet the clacking of spurred boots on stone don't cease, not even for an instant.
                            Help a kobold?
                            Why would he?
                        [/p]
                        [p]
                            For several minutes, all you know is agony.
                            Every square inch of your body not covered by your cowering arms is viciously attacked.
                            Your ribs are smashed, your nose is broken,
                            your back is [b]pelted[/b] with a barrage of blows and even the back of your head endures more than a few beatings.
                            As a final kick hits you square in the stomach, on the exact same spot as the very first one did, you are sent into a coughing fit,
                            hacking up bloody phlegms from the depths of your injured lungs.
                        [/p]
                        [p]
                            The three burly men are panting.
                            Exhausted, they wipe your own blood off onto your scales.
                            One of them cruelly spits in your face.
                            They're done, [b]for now[/b].
                        [/p]
                        [p]
                            Your abusers hurl various insults at you as they walk away, but you're too far [i]gone[/i] to really pay them much mind.
                            Teetering on the edge of consciousness, on the very brink of [b]death[/b],
                            you try to escape the crippling pain clouding your mind by focusing on your breathing,
                            but even the slight motion of your chest inflating and deflates sends spikes of [b]torment[/b] down your spine.
                            You've been hurt before, but never [b]this[/b] bad.
                            Blood oozes from your battered muzzle as you quietly sob away the immense anguish.
                        [/p]
                        ${global.ui.condFail(`Insufficient health and endurance`)}
                        [p]
                            About half an hour later, you manage to open one of your eyelids anew, the swelling slowly fading.
                            Into your blurry vision walks a man.
                            A big man.
                            A pig, wearing an off-white, completely bloodied apron.
                            A wicked grin is plastered across his plump face.
                            "[color=ppink]Fresh meat. Already tenderized.[/color]" he speaks in a bass voice.
                        [/p]
                        [p]
                            The pig leans down and takes hold of your tail.
                            The last thing you remember is being dragged across the street, leaving a trail of blood in your wake.
                            As the man halts before a large, wooden building with sharp metallic noises coming from inside, you finally slip into sweet unconsciousness,
                            never to wake up again.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Pig end. Oink.
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400, function () {
            var main = $("#main");
            main.scrollTop(main[0].scrollHeight - main.height() - 5); // Fix scrolling
        });
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.misc.unsetAll();
            ctrl.flee.undo();
            ctrl.flee.unset();
            ctrl.runin.unset();
            ctrl.stage.unset();
            for (var i = 0; i < idsToRemove.length; i++) {
                $("#" + idsToRemove[i]).remove();
            }
            loadNextPart(new NightTwoStage(ctrl));
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        registerContinueListener();
    };
}

// END scenes\Varanar\NightTwo\flee\run\fail\die\NightTwoRunDeath.js

// scenes\Varanar\NightTwo\flee\run\fail\live\NightTwoRunSurvivalContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunSurvivalContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Half an hour later, you manage to open one of your eyelids anew, the swelling slowly fading.
                            Just in time to witness the sun slowly starting to rise above the rooftops.
                            The sunrise inspires you.
                            You steel your mind.
                            You aren't dead yet.
                            You can't stay here.
                            There's some things you need to [b]fix[/b] before you can get up, though.
                            This is going to hurt.
                        [/p]
                        [p]
                            Bringing both your hands to your snout, you forcibly [b]twist[/b] your broken nose-bones back into place,
                            a sickening [b]snap[/b] accompanying the horrifying [b]pain[/b] of two of your bones grinding against one another.
                            You let out a shrill shriek of suffering, although self-inflicted this time around.
                            Your cryiing and your breathing both intensify, but as the new pain begins to fade, your muzzle feels slightly [b]better[/b] off as a whole.
                            Like it was [b]bent out of place[/b] during the intense beatdown, and you [b]set it straight[/b] just now.
                            It is a [b]good[/b] pain.
                        [/p]
                        [p]
                            After wallowing in your own misery for a little while longer, you wearily roll onto your heavily bruised stomach,
                            swiftly raising yourself to your knees to avoid having to lie on top of fresh injuries.
                            Your legs were mostly spared during the beating.
                            Leaning back, resting your weight on top of your calves, you inspect the damage done to your body.
                            You are bruised all over, with several small, but lengthy cuts along your side and your chest.
                            You think one of your ribs is broken, but you aren't sure.
                        [/p]
                        [p]
                            Your stomach hurts significantly.
                            You think you may have severe internal injuries, but you're no doctor.
                            And even if you were, you doubt there's a cure for [i]ruptured stomach[/i].
                            But you're a kobold.
                            Your kind has the uncanny ability to live through even the most grievous of injuries and come back swinging.
                            And even if your injuries [b]won't[/b] heal, you simply [b]refuse[/b] to die [b]here[/b], in the middle of the street.
                        [/p]
                        [p]
                            No.
                            You're [b]not[/b] going to die.
                            That's what [b]they[/b] -- the humans that beat you -- would want.
                            You are [b]tough[/b].
                            You're going to [b]live[/b], if just because your very existance serves to [b]taunt[/b] the people who did this to you.
                        [/p]
                        [p]
                            Driven by [b]grit[/b] and [b]determination[/b], by the sheer [b]will[/b] to live, you crawl to your feet.
                            With slow, unsteady paces, you begin to wobble in the general direction of the town square.
                            Every step you take seems to invigorate you further, empowering your sheer [b]stubbornness[/b], your [b]refusal[/b] to let go of life itself.
                            While your wounds remain severe, your mind is stalwartly set on overcoming them.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded.
                            You now realize you are but a lone, weak,
                            small kobold in a big city filled with people who can and [b]will[/b] maim, hurt, kill and perhaps even [b]rape[/b] you.
                            You can't let your guard down, ever.
                            Next time you do, it might end up costing you your life, as it already nearly has.
                            You need to stay tough if you want to perservere in the big city.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.runSurvive.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.runSurvive.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\run\fail\live\NightTwoRunSurvivalContinue.js

// scenes\Varanar\NightTwo\flee\run\fail\live\NightTwoRunSurvival.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunSurvival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoRunSurvivalContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.nightTwo.beatUp = true;
        var toReturn = `[p]
                            Desperate to get away from the men, you burst into a sprint the moment you turn a corner,
                            hoping to be able to outrun them before they even [i]know[/i] you're escaping.
                            While your gambit does pay off in the form of a head start, sadly, you don't manage to shake the group from the get-go.
                            Noticing their lizard-prey running away,
                            the men drop their tight-packed group-cohesion and begin to outright [b]chase[/b] after you.
                        [/p]
                        [p]
                            You run from street to street, periodically checking behind you to make sure they haven't given up yet.
                            While far bigger than you, and probably able to outrun you whilst sober, their heavy inebriation hinders the belligerent group significantly.
                            A couple of pursuers tumble over their own feet, falling to the ground, being left behind by their peers.
                            A few of the men get too sick to keep pursuing you, retreating into alleyways to empty their bellies of food and drink.
                            Some of them are simply too weary to keep up pursuit.
                            Initially numbering high in the single-digits,
                            it takes only a few solid minutes of running for their [i]pack[/i] to be reduced to three hard-boiled men.
                        [/p]
                        [p]
                            But still, even [b]one[/b] of them can [b]cave your head in[/b] if they manage to catch you.
                            You aren't out of the woods until you can shake off [b]all three[/b] of them.
                            And you get the feeling that all this running is making them [b]even more frustrated[/b].
                            There's no doubt in your mind: your very life is at stake here.
                            And the humans chasing you are [b]determined[/b] to rid their city of another [b]pest[/b] infesting it.
                        [/p]
                        [p]
                            You push your body to its very limits, sprinting away as fast as you can.
                            You can't hold out for much longer.
                            You can feel the end of your rope approaching.
                            You hope they're every bit as tired as you are.
                            That their bodies are as much on the verge of collapsing as your tiny, scaled frame is.
                        [/p]
                        [p]
                            Crossing into a more dignified area of Varanar, one of the few districts with cobblestone roads,
                            you feel the last of your energy leaving your body.
                            You try to keep running, you [b]really[/b] do, but your legs simply can no longer carry you.
                            As the flat dirt under your feet suddenly makes way for a more uneven, rock-like surface,
                            you lose your balance and plummet to the ground.
                            Time itself seems to slow down as your body falls limply onto the cold, hard street, completely exhausted.
                        [/p]
                        ${global.ui.condFail(`Insufficient stamina`)}
                        [p]
                            You roll onto your side right on time to witness one of your pursuers rapidly approaching.
                            Using the hefty momentum from his drunken running as a catalyst, he cruelly plants the tip of his heavy boot into your stomach,
                            kicking you [b]so[/b] hard your leathery body is lifted off the ground as a whole.
                            You scream in agony as you are hurled through the air, your panicked shout coming to an end as you fall down once more,
                            your frame heavily impacting with the stone street a second time, painfully skimming across it for a few more inches before finally coming to rest.
                        [/p]
                        [p]
                            Seriously hurt, you clutch your abdomen and slowly curl up into a ball.
                            You're pretty certain the [b]immense[/b] force behind the kick [b]broke[/b] something inside of you.
                            As you whimper and cry, the other two men that were chasing you catch up with their peer.
                            They advance upon you, and waste no time in resuming their brutal clobbering.
                        [/p]
                        [p]
                            One of them leans down, and mercilessly begins to [b]pound[/b] your side, battering your ribs with blow after blow of his powerful, clenched fist.
                            A second human begins to repeatedly kick your back with all of his might, your spine on the verge of [b]snapping[/b] in half.
                            The third and final abuser concerns himself with your face,
                            alternating between using his hands and his shoe-clad feet to batter every part of your visage from your forehead to your snout.
                        [/p]
                        [p]
                            You [b]beg[/b] for mercy, but the humans seem determined not to stop until you're [b]dead[/b].
                            Your eyes are beaten shut to the point where you can no longer [b]see[/b],
                            but you [b]hear[/b] someone walking by over the gut-wrenching sounds of your bones breaking and your scales bruising.
                            You cry out for help, hoping it's a kind-hearted guard, yet the clacking of spurred boots on stone don't cease, not even for an instant.
                            Help a kobold?
                            Why would he?
                        [/p]
                        [p]
                            For several minutes, all you know is agony.
                            Every square inch of your body not covered by your cowering arms is viciously attacked.
                            Your ribs are smashed, your nose is broken,
                            your back is [b]pelted[/b] with a barrage of blows and even the back of your head endures more than a few beatings.
                            As a final kick hits you square in the stomach, on the exact same spot as the very first one did, you are sent into a coughing fit,
                            hacking up bloody phlegms from the depths of your injured lungs.
                        [/p]
                        [p]
                            The three burly men are panting.
                            Exhausted, they wipe your own blood off onto your scales.
                            One of them cruelly spits in your face.
                            They're done, [b]for now[/b].
                        [/p]
                        [p]
                            Your abusers hurl various insults at you as they walk away, but you're too far [i]gone[/i] to really pay them much mind.
                            Teetering on the edge of consciousness, on the very brink of [b]death[/b],
                            you try to escape the crippling pain clouding your mind by focusing on your breathing,
                            but even the slight motion of your chest inflating and deflates sends spikes of [b]torment[/b] down your spine.
                            You've been hurt before, but never [b]this[/b] bad.
                            Blood oozes from your battered muzzle as you quietly sob away the immense anguish.
                        [/p]
                        ${global.ui.condSucc(`Sufficient health and endurance`)}`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.runSurvive.continued() ? "disabled" : "";
        var isSelected = ctrl.runSurvive.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.runSurvive.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runSurvive.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.runSurvive.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\run\fail\live\NightTwoRunSurvival.js

// scenes\Varanar\NightTwo\flee\run\fail\NightTwoRunBeatdown.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunBeatdown(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoRunDeath(ctrl);
    var toSuccess = new NightTwoRunSurvival(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var health = ctrl.misc.health();
        var endurance = ctrl.misc.endurance();
        var damage = Math.floor(400 / endurance);
        return health > damage;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\run\fail\NightTwoRunBeatdown.js

// scenes\Varanar\NightTwo\flee\run\succeed\NightTwoRunGetawayContinue.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunGetawayContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You've overworked yourself once more.
                            Your lungs and your legs both ache [b]so[/b] much it feels like they're [b]on fire[/b].
                            You take a series of deep breaths in an attempt to calm yourself.
                            It takes effort and willpower [b]not[/b] to faint, but you've been through worse.
                            Those humans didn't kill you, so neither will some bodily strain.
                        [/p]
                        [p]
                            After a couple of minutes spent recovering, you begin to get your bearings.
                            You think you recognize this area.
                            You [i]explored[/i] it before the [i]unfortunate run-in[/i] with those [b]ruffians[/b].
                            Slowly, your unsteady legs begin to carry you in the direction of the city's market square once more.
                        [/p]
                        [p]
                            The further you walk, the more you start to regain your composure.
                            Luckily, you're not [b]too[/b] far away from the plaza where your urban adventure began.
                            As you begin to near it, you notice the sun rising above the rooftops.
                            You must have lost track of time while exploring the city.
                            Or maybe that frantic chase took longer than you can remember.
                            Either way, you guess you won't be sleeping tonight.
                        [/p]
                        [p]
                            The twinkle of magic you felt at midnight has all but faded, giving way to a sense of [b]vulnerability[/b].
                            You are but a lone, weak, small, kobold in a big city filled with people who can and [b]want[/b] to maim,
                            hurt, kill and perhaps even [b]rape[/b] you.
                            If you let your guard down for even one moment, it could cost you your money, your life or both.
                            And next time, the men chasing you might [i]not[/i] be drunk off their asses.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.runGetaway.continued()) {
            return "";
        }

        global.ui.applyTheme("varanar_morning");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.endChange.change();
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.runGetaway.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\run\succeed\NightTwoRunGetawayContinue.js

// scenes\Varanar\NightTwo\flee\run\succeed\NightTwoRunGetaway.js

/**
 * Flee section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunGetaway(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoRunGetawayContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Desperate to get away from the men, you burst into a sprint the moment you turn a corner,
                            hoping to be able to outrun them before they even [i]know[/i] you're escaping.
                            While your gambit does pay off in the form of a head start, sadly, you don't manage to shake the group from the get-go.
                            Noticing their lizard-prey running away,
                            the men drop their tight-packed group-cohesion and begin to outright [b]chase[/b] after you.
                        [/p]
                        [p]
                            You run from street to street, periodically checking behind you to make sure they haven't given up yet.
                            While far bigger than you, and probably able to outrun you whilst sober, their heavy inebriation hinders the belligerent group significantly.
                            A couple of pursuers tumble over their own feet, falling to the ground, being left behind by their peers.
                            A few of the men get too sick to keep pursuing you, retreating into alleyways to empty their bellies of food and drink.
                            Some of them are simply too weary to keep up pursuit.
                            Initially numbering high in the single-digits,
                            it takes only a few solid minutes of running for their [i]pack[/i] to be reduced to three hard-boiled men.
                        [/p]
                        [p]
                            But still, even just [b]one[/b] of them can [b]cave your head in[/b] if they manage to catch you.
                            You aren't out of the woods until you can shake off [b]all three[/b] of them.
                            And you get the feeling that all this running is making them [b]even more frustrated[/b].
                            There's no doubt in your mind: your very life is at stake here.
                            And the humans chasing you are [b]determined[/b] to rid their city of another [b]pest[/b] infesting it.
                        [/p]
                        [p]
                            You push your body to its very limits, sprinting away as fast as you can.
                            Your legs are rapidly growing weary, but so are theirs, you hope.
                            Crossing into a more dignified area of Varanar, one of the few districts with cobblestone roads,
                            you almost lose your balance as the flat dirt under your feet suddenly makes way for a more uneven, rock-like surface.
                            [i]Almost[/i].
                        [/p]
                        ${global.ui.condSucc(`Sufficient stamina`)}
                        [p]
                            Luckily, it seems like the men chasing you, too, have difficulty adapting to the new enviroment.
                            Looking behind you, you can see one of them taking a particularly [b]nasty[/b] dive, his forehead [b]slamming[/b] into the rocky ground.
                            You're pretty sure you can hear something [b]crack[/b], although you don't know whether it's his skull or the street he fell atop.
                            Either way, the man's [i]friends[/i] cease their pursuit in order to tend to their fallen comrade, who seems [b]seriously[/b] hurt.
                            Serves him right for chasing you!
                        [/p]
                        [p]
                            While no longer being tailed, you maintain a hasty pace,
                            crossing a few more streets on sheer adrenaline alone to ensure the hoodlums won't catch you.
                            Not tonight, at least.
                            Once you're [b]completely[/b] certain you've lost them, you slowly come to a halt,
                            leaning against a nearby building to keep your overexerted body upright.
                            You catch your breath, your loud panting resonating across the otherwise empty street.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.runGetaway.continued() ? "disabled" : "";
        var isSelected = ctrl.runGetaway.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.runGetaway.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.runGetaway.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.runGetaway.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\flee\run\succeed\NightTwoRunGetaway.js

// scenes\Varanar\NightTwo\flee\run\NightTwoKeepRunning.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoKeepRunning(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toFail = new NightTwoRunBeatdown(ctrl);
    var toSuccess = new NightTwoRunGetaway(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.misc.stamina() > 70;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.flee.get() !== NightTwoFleeChoice.RUN) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.flee.get() !== NightTwoFleeChoice.RUN) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\NightTwo\flee\run\NightTwoKeepRunning.js

// scenes\Varanar\NightTwo\flee\NightTwoFlee.js

/**
 * Fleeing section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoFlee(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var runId = arguments.callee.name + "ButtonOne";
    var hideId = arguments.callee.name + "ButtonTwo";

    var toRun = new NightTwoKeepRunning(ctrl);
    var toHide = new NightTwoAlley(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You wisely decide to head the [i]other[/i] way, which unfortunately leads you back into the [i]brothel district[/i].
                            You aren't sure yet whether they're actually [i]following[/i] you or not, but the men seem to be heading this way as well.
                            They're getting more [b]rowdy[/b] by the minute.
                            You vaguely overhear them cracking rude jokes about...[i]kobolds[/i].
                            To be more precise: they're joking [b]about you[/b].
                            And the things they're saying are far from being nice.
                            While jolly at first, the mood is growing more grim with every passing second.
                        [/p]
                        [p]
                            To see whether you're just a victim of bad luck, or if the group is actively [i]chasing[/i] you,
                            you take a few random turns to nowhere in particular.
                            Still, the men remain on your tail.
                            Like they have some unsettled beef with you, or maybe with your kind as a whole.
                            You pick up your walking pace.
                            So do they.
                            You start jogging.
                            They resort to drunkenly and menacingly speedwalking,
                            their size allowing them to keep up with your relatively higher pace without needing to [i]run[/i].
                        [/p]
                        [p]
                            Your mind is racing a million miles an hour.
                            You feel anxious.
                            Like a rat trapped in a maze, pursued by bunch of hungry cats.
                            The men seem to be gaining on you, despite your best efforts.
                            If you don't do something quick, they'll catch you.
                            And who knows what a gang of angry, drunken humans'll do once they get their hands on a dirty kobold thief like yourself?
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildRunChoice() {
        var disabled = ctrl.flee.has() ? "disabled" : "";
        var isSelected = ctrl.flee.get() === NightTwoFleeChoice.RUN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${runId}" ${disabled}>Run</button>`;

        var tooltip = global.parser.parse(`Run away as fast as you can. Hope they won't be able to keep up.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildHideChoice() {
        var disabled = ctrl.flee.has() ? "disabled" : "";
        var isSelected = ctrl.flee.get() === NightTwoFleeChoice.HIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hideId}" ${disabled}>Hide</button>`;

        var tooltip = global.parser.parse(`Dart into a nearby alleyway and pray they don't find you.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildRunChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHideChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + runId), $("#" + hideId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerRunListener() {
        $("#" + runId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.flee.set(NightTwoFleeChoice.RUN);
            loadNextPart(toRun);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerHideListener() {
        $("#" + hideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.flee.set(NightTwoFleeChoice.HIDE);
            loadNextPart(toHide);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.runin.get() !== NightTwoRunInChoice.FLEE) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.flee.change();
        part += buildChoice();
        part += `</div>`;
        part += toRun.build();
        part += toHide.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.runin.get() !== NightTwoRunInChoice.FLEE) {
            return;
        }

        toRun.registerListeners();
        toHide.registerListeners();

        if (ctrl.flee.has()) {
            return;
        }

        registerRunListener();
        registerHideListener();
    };
}

var NightTwoFleeChoice = {
    RUN: 1,
    HIDE: 2
};

// END scenes\Varanar\NightTwo\flee\NightTwoFlee.js

// scenes\Varanar\NightTwo\arrival\NightTwoRunIn.js

/**
 * Arrival section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoRunIn(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var ignoreId = arguments.callee.name + "ButtonOne";
    var fleeId = arguments.callee.name + "ButtonTwo";

    var toIgnore = new NightTwoConfront(ctrl);
    var toFlee = new NightTwoFlee(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            After resting your feet for a while, you decide it's best to get a lay of the land before catching some shut-eye.
                            The night is still young, and you know pretty much nothing of the cityscape outside of the grand avenue.
                            Actually, you [i]do[/i] know how to make your way to the hideout of the Three-Tailed Rat Clan,
                            nested deep within the kobold district of Varanar,
                            but only because you [i]memorized[/i] the route between there and the eastern city gate.
                            Hoping to get a clearer picture of what road leads where, you pick a street at random,
                            and begin your journey of exploration throughout the city.
                        [/p]
                        [p]
                            You wander through the mercantile district and the residential area,
                            you pause briefly to inspect the magnificent architecture of the local church,
                            you walk alongside the river that splits the capital in two,
                            and you wisely avoid the kobold district at this time of night.
                            Less than two hours after you left the town square,
                            you manage to form a pretty decent idea on how to get around Varanar.
                        [/p]
                        [p]
                            Still, there's plenty more avenues left to explore, and you don't feel like going to bed quite yet,
                            too enthralled by all the sights to see to stop now.
                            Ultimately, after another few hours of discovery and mental mapmaking, you find yourself in a less [i]savory[/i] part of town.
                            You aren't sure whether you should nickname this area the [i]brothel district[/i] or the [i]tavern zone[/i].
                            Houses in this neighbourhood are as poorly built and maintained as they are in the kobold quarter,
                            although they are, admittedly, a lot [i]bigger[/i].
                        [/p]
                        [p]
                            While most of the city's streets are well-patrolled and well-lit, this section seems mostly exempt from sentries and guards.
                            Or maybe it's just getting so late that most of them have already gone to sleep.
                            You, too, are starting to get sleepy.
                            You begin to make your way back to the city square in hopes of finding an inn,
                            a bale of hay or at least an alleyway that [i]doesn't[/i] smell like someone died in it.
                        [/p]
                        [p]
                            As you turn a corner, you notice a small group of burly human males headed your way.
                            They seem drunk and unsavory, probably returning home after a night full of beer and wenches.
                            One of them crudely points his finger your way while loudly chattering to his friends.
                            You can't quite understand them, but whatever he said must have been funny.
                            The group bursts out into laughter.
                            Their eyes become fixated on you.
                            Somehow, you don't feel very safe anymore.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildIgnoreChoice() {
        var disabled = ctrl.runin.has() ? "disabled" : "";
        var isSelected = ctrl.runin.get() === NightTwoRunInChoice.IGNORE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${ignoreId}" ${disabled}>Ignore them</button>`;

        var tooltip = global.parser.parse(`Walk past the group. Maybe they'll leave you alone. `);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 150px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFleeChoice() {
        var disabled = ctrl.runin.has() ? "disabled" : "";
        var isSelected = ctrl.runin.get() === NightTwoRunInChoice.FLEE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fleeId}" ${disabled}>Flee</button>`;

        var tooltip = global.parser.parse(`Walk the other way. Keep a steady pace to stay ahead of them.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildIgnoreChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFleeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + ignoreId), $("#" + fleeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerIgnoreListener() {
        $("#" + ignoreId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.runin.set(NightTwoRunInChoice.IGNORE);
            loadNextPart(toIgnore);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFleeListener() {
        $("#" + fleeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.runin.set(NightTwoRunInChoice.FLEE);
            loadNextPart(toFlee);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.stage.continued()) {
            return "";
        }

        ctrl.bgm.loop();
        global.kobold.inventory.exitSafeZone();
        global.ui.imageBackground.hide();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toIgnore.build();
        part += toFlee.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.stage.continued()) {
            return;
        }

        toIgnore.registerListeners();
        toFlee.registerListeners();

        if (ctrl.runin.has()) {
            return;
        }

        registerIgnoreListener();
        registerFleeListener();
    };
}

var NightTwoRunInChoice = {
    IGNORE: 1,
    FLEE: 2
};

// END scenes\Varanar\NightTwo\arrival\NightTwoRunIn.js

// scenes\Varanar\NightTwo\arrival\NightTwoStage.js

/**
 * Arrival section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoStage(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoRunIn(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            After passing the gate, you are greeted with the sight of Varanar's grand avenue,
                            a single road leading all the way from the eastern entrance to the massive town square.
                            Last time you were in town, you tried pickpocketing people on this very path during the bustling morning rush.
                            Now, the streets are nigh-empty, save for a few patrolling guardsmen.
                        [/p]
                        [p]
                            You idlemindedly walk down the road towards the market square.
                            The tall, human-made buildings on either side almost make you feel a little [i]claustrophobic[/i],
                            despite the street being wide enough to fit several carts side-by-side.
                            To you, this entire city seems like a bit of a [i]miracle[/i].
                            You can't even [b]begin[/b] to imagine the [b]huge[/b] amount of work it must have taken to build all of this.
                            While other races are banding together to construct vast cities,
                            your kind can barely stand one another for long enough to rob people together.
                        [/p]
                        [p]
                            After a brief hike, you arrive at the town square.
                            While the plaza is empty at the moment, the bi-weekly market has left a clear impression on the land.
                            You wouldn't call it completely riddled with garbage, but there is more than a bit of litter remaining.
                            The hard, near-frozen ground is completely [b]trampled[/b], a multitude of holes denoting the former position of the stalls.
                        [/p]
                        [p]
                            Suppressing your natural urge to sift through the trash, you walk up to the small stage on the far-end of the square.
                            This is where important decrees are shared, where traitors are hanged,
                            where stockades are set and where prestigious travelling shows entertain thousands of people.
                            But right now, for you, it is nothing but a place to rest your weary feet at.
                            You crawl up onto the wooden surface, and sit yourself down on the edge.
                        [/p]
                        [p]
                            The church bells ring midnight.
                            You take in a deep breath of cool evening air, staring upwards at the night sky.
                            Stars twinkle back at your inquisitive gaze, mirroring the sparkle in your eyes.
                            While not full moon yet, it will be in a couple of days.
                            There's something magical about this city.
                            It makes you feel [i]safe[/i].
                            Moreso than the forest, at least.
                        [/p]
                        [p]
                            [sneak]
                                You are in a safe zone.
                                You can use items from your inventory here.
                                It may be wise to prepare yourself for the trials up ahead,
                                lest you be found wanting.
                            [/sneak]
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.stage.continued() ? "disabled" : "";
        var isSelected = ctrl.stage.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.stage.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.arrival.continued()) {
            return "";
        }

        ctrl.bgm.stopLoop();
        global.ui.applyTheme("varanar_night"); // Fix theme when reverting
        global.ui.imageBackground.show("scenes/Varanar/NightTwo/arrival/stars.gif");
        global.kobold.inventory.enterSafeZone();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.arrival.continued()) {
            return;
        }

        next.registerListeners();

        if (ctrl.stage.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\arrival\NightTwoStage.js

// scenes\Varanar\NightTwo\arrival\NightTwoArrival.js

/**
 * Arrival section of Varanar, Night Two.
 * @param {NightTwoController} controller The Night Two controller.
 */
function NightTwoArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new NightTwoStage(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[st]
                            Arrival
                        [/st]
                        [p]
                            After a boring, uneventful trek, you finally arrive at the great walls of Varanar.
                            Home to thousands, if not tens of thousands, it is the single largest city outside of the neighbouring Dragon Empire.
                            The capital of humans, one of the most prominent trading hubs in the entire world, and a melting pot for all walks of life,
                            from rich and snobby noblemen, to humble, rundown [b]thieves[/b] like yourself.
                        [/p]
                        [p]
                            The sun may have gone down, but in the darkness,
                            the myriad of flickering torchlights illuminating the battlements high up above showcase how [b]incredibly[/b] well protected the city is.
                            Nothing short of a determined and well-prepared besieging [b]army[/b] could ever hope to take the walls of Varanar.
                            While you've been here before, you've never seen them at night.
                            You always assumed the battlements were [i]empty[/i], that they'd be manned solely during times of war.
                            These fortifications were impressive and imposing enough [i]without[/i] realizing that they are constantly staffed,
                            a dozen archers and crossbowmen probably betting on whether or not they can hit that tiny little kobold 
                            approaching the gates at this very instant.
                        [/p]
                        [p]
                            While kobolds are generally [i]tolerated[/i] in public, there are no official laws protecting your kind.
                            You could be shot, stabbed or keelhauled in front of a crowd, and nobody would bat an eye.
                            The thing that frightens you the most is the fact that people can do [b]far worse[/b] things than simply hurting you, all according to law.
                            The guards rarely intervene, and when they do, you can be [b]darn[/b] sure that they [b]won't[/b] be picking [i]your[/i] side.
                        [/p]
                        [p]
                            But those are worries for later.
                            Right now, you focus on actually getting [b]in[/b] to the city proper.
                            The eastern gate, in front of which you stand, is currently clogged to the brim with people [b]leaving[/b] Varanar.
                            Must have been a long market day or something.
                            Leaning against a nearby road-fence, you decide to wait until the worst of the crowd has cleared.
                        [/p]
                        [p]
                            You had hoped you would only have to wait for a couple of minutes, but it takes almost an entire [b]hour[/b] for the huge mass of daytrippers,
                            marketeers, farmers, supply-running forest bandits and foreign trading caravans to siphon from the great city.
                            You pass the time by observing the huge variety of lifeforms on display.
                            Humans, horses, rodents, birds, cats both big and small, bears, reptiles and all sorts of canines walk on by.
                            Some carry wheelbarrows, others ride carts pulled by beasts of burden and a few are even rich enough to own their own chariots.
                        [/p]
                        [p]
                            Outside of a single kobold asking for directions, nobody really pays you much mind.
                            As the crowd slowly disperses in all directions of the land, you hop off of the fence, and proceed towards the gate.
                            While passing through the wall-opening, you instinctively gaze upwards at the portcullis,
                            ready to jump out of the way should it budge even an inch.
                            Luckily, this time at least, you are spared the wrath of the suspended metal.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.arrival.continued() ? "disabled" : "";
        var isSelected = ctrl.arrival.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.arrival.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.ui.applyTheme("varanar_night");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.arrival.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.arrival.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\NightTwo\arrival\NightTwoArrival.js

// scenes\Varanar\NightTwo\NightTwoController.js

/**
 * Controller for the Varanar Night Two scene.
 * @param {string} sceneId The id of the scene.
 */
function NightTwoController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = true;
    this.nextSceneName = "MorningThree";
    this.nextFolderName = "Varanar/DayThree";

    this.misc = new MiscController(id);

    this.arrival = new ChangeController(id, "arrival", function () {
        global.kobold.status.mental.increase(15, "Amazed by Varanar");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Amazement", "Sanity", +15)
        ]);
    });

    this.stage = new ContinueController(id, "stage");
    this.runin = new ChoiceController(id, "runin");

    this.flee = new UndoController(id, "flee", function change() {
        var sanityBacklog = new ChoiceController(id, "fleeSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "fleeStaminaBL");
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        staminaBacklog.set(global.kobold.status.stamina.lasthit);
        global.kobold.status.mental.decrease(5, "Being chased");
        global.kobold.status.stamina.decrease(5, "Light jogging");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Being pursued", "Sanity", -5),
            new Change("Fearful jogging", "Stamina", -5)
        ]);
    }, function undoChange() {
        var sanityBacklog = new ChoiceController(id, "fleeSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "fleeStaminaBL");
        global.kobold.status.mental.increase(5, sanityBacklog.get());
        global.kobold.status.stamina.increase(5, staminaBacklog.get());
        sanityBacklog.unset();
        staminaBacklog.unset();
    });

    this.alley = new ChoiceController(id, "alley");
    this.hide = new ChoiceController(id, "hide");

    this.hideStay = new ChangeController(id, "hideStay", function change() {
        global.kobold.status.physical.decrease(5, "Breathing in miasma");
        global.kobold.status.mental.decrease(25, "Got pissed on");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Miasma", "Health", -5),
            new Change("Pissed on", "Sanity", -25)
        ]);
    });

    this.hideAttack = new ChangeController(id, "hideAttack", function change() {
        global.kobold.status.mental.increase(10, "Righteous justice");
        global.kobold.status.mental.decrease(25, "Manual neutering");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Righteous justice", "Sanity", +10),
            new Change("Manual neutering", "Sanity", -25)
        ]);
    });

    this.alleyRunSuccess = new ChangeController(id, "alleyRunSuccess", function change() {
        global.kobold.status.physical.increase(5, "Good workout");
        global.kobold.status.mental.increase(10, "Showcased skill");
        global.kobold.status.stamina.decrease(10, "Parkouring");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Good workout", "Health", +5),
            new Change("Skillful escape", "Sanity", +10),
            new Change("Parkouring", "Stamina", -10)
        ]);
    });

    this.alleyRunFail = new UndoController(id, "alleyRunFail", function change() {
        var healthBacklog = new ChoiceController(id, "alleyRunFailHealthBL");
        var sanityBacklog = new ChoiceController(id, "alleyRunFailSanityBL");
        var staminaBacklog = new ChoiceController(id, "alleyRunFailStaminaBL");
        healthBacklog.set(global.kobold.status.physical.lasthit);
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        staminaBacklog.set(global.kobold.status.stamina.lasthit);
        global.kobold.status.physical.decrease(5, "Fell down");
        global.kobold.status.mental.decrease(10, "Plan failed");
        global.kobold.status.stamina.decrease(5, "Attempted climbing");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Fell down", "Health", -5),
            new Change("Plan failed", "Sanity", -5),
            new Change("Cornered", "Sanity", -5),
            new Change("Attempted parkouring", "Stamina", -5)
        ]);
    }, function undo() {
        var healthBacklog = new ChoiceController(id, "alleyRunFailHealthBL");
        var sanityBacklog = new ChoiceController(id, "alleyRunFailSanityBL"); // Todo: Make framework for this
        var staminaBacklog = new ChoiceController(id, "alleyRunFailStaminaBL");
        global.kobold.status.physical.increase(5, healthBacklog);
        global.kobold.status.mental.increase(10, sanityBacklog.get());
        global.kobold.status.stamina.increase(5, staminaBacklog.get());
        healthBacklog.unset();
        sanityBacklog.unset();
        staminaBacklog.unset();
    });

    this.alleyEager = new ChangeController(id, "alleyEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "CHANGE", +1)
        ]);
    });

    this.alleyStay = new ChangeController(id, "alleyStay", function change() {
        if (global.kobold.inventory.equipment.shirt) {
            global.kobold.inventory.equipment.shirt.stain("Piss");
        }
        if (global.kobold.inventory.equipment.pants) {
            global.kobold.inventory.equipment.pants.stain("Urine");
        }
        if (global.kobold.inventory.equipment.underwear) {
            global.kobold.inventory.equipment.underwear.stain("Pee");
        }
        global.kobold.status.physical.decrease(5, "Unsanitary actions");
        global.kobold.status.mental.decrease(25, "Got pissed on");
        global.kobold.stats.decreaseStat(StatsEnum.CHARISMA, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Unsanitary", "Health", -5),
            new Change("Unwilling watersports", "Sanity", -25),
            new Change("Lasting fragrance", "Charisma", -1)
        ]);
    });

    this.runGetaway = new ChangeController(id, "runGetaway", function change() {
        global.kobold.status.physical.decrease(10, "Overexertion");
        global.kobold.status.mental.increase(10, "Escaped certain death");
        global.kobold.status.stamina.decrease(70, "Running for your life");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Overexertion", "Health", -10),
            new Change("Escaped certain death", "Sanity", +10),
            new Change("Out of breath", "Stamina", -70)
        ]);
    });

    this.runSurvive = new ChangeController(id, "runSurvive", function change() {
        var stamina = that.misc.stamina();
        var endurance = that.misc.endurance();
        var damage = Math.floor(400 / endurance);
        global.kobold.status.physical.decrease(damage, "Beat up by humans");
        global.kobold.status.mental.decrease(20, "Victim complex");
        global.kobold.status.stamina.decrease(stamina, "Ran for your life");
    }, function changeNote() {
        var stamina = that.misc.stamina();
        var endurance = that.misc.endurance();
        var damage = Math.floor(400 / endurance);
        return global.ui.change.buildNotification([
            new Change("Beatdown", "Health", -damage),
            new Change("Victim complex", "Sanity", -20),
            new Change("Gave it your all", "Stamina", -stamina)
        ]);
    });

    this.lewdCrude = new UndoController(id, "lewdCrude", function change() {
        var sanityBacklog = new ChoiceController(id, "lewdCrudeSanityBL");
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        global.kobold.status.mental.decrease(5, "Growing concerned");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Insults", "Sanity", -2),
            new Change("Concern", "Sanity", -3)
        ]);
    }, function undo() {
        var sanityBacklog = new ChoiceController(id, "lewdCrudeSanityBL");
        global.kobold.status.mental.increase(5, sanityBacklog.get());
        sanityBacklog.unset();
    });

    this.rude = new UndoController(id, "rude", function change() {
        var healthBacklog = new ChoiceController(id, "rudeHealthBL");
        var sanityBacklog = new ChoiceController(id, "rudeSanityBL");
        healthBacklog.set(global.kobold.status.physical.lasthit);
        sanityBacklog.set(global.kobold.status.mental.lasthit);
        global.kobold.status.physical.decrease(5, "Arm twisted");
        global.kobold.status.mental.decrease(10, "Pain");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Arm twisted", "Health", -5),
            new Change("Panic", "Sanity", -5),
            new Change("Pain", "Sanity", -5)
        ]);
    }, function undo() {
        var healthBacklog = new ChoiceController(id, "rudeHealthBL");
        var sanityBacklog = new ChoiceController(id, "rudeSanityBL");
        global.kobold.status.mental.increase(5, healthBacklog.get());
        global.kobold.status.mental.increase(10, sanityBacklog.get());
        healthBacklog.unset();
        sanityBacklog.unset();
    });

    this.appease = new ChoiceController(id, "appease");

    this.crudeEager = new ChangeController(id, "crudeEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "Change", +1)
        ]);
    });

    this.crudeReluctant = new ChangeController(id, "crudeReluctant", function change() {
        global.kobold.status.physical.decrease(2, "Unsanitary kiss");
        global.kobold.status.mental.decrease(20, "Disgust and humiliation");
        global.kobold.status.exertion.increase(5, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Unsanitary kiss", "Health", -2),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +5, true)
        ]);
    });

    this.crudeUnfit = new ChangeController(id, "crudeUnfit", function change() {
        global.kobold.status.mental.increase(15, "Escaped through wits");
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Out-smarted humans", "Sanity", +15),
            new Change("Out-smarted humans", "Intelligence", +1)
        ]);
    });

    this.crudeStruggle = new ChangeController(id, "crudeStruggle", function change() {
        global.kobold.status.physical.decrease(10, "Pain and unhygienic behavior");
        global.kobold.status.mental.decrease(30, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Pain", "Health", -5),
            new Change("Unsanitary", "Health", -5),
            new Change("Forced", "Sanity", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.lewdEager = new ChangeController(id, "lewdEager", function change() {

    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("TODO", "Change", +1)
        ]);
    });

    this.lewdReluctant = new ChangeController(id, "lewdReluctant", function change() {
        global.kobold.status.physical.decrease(10, "Filth ingestion");
        global.kobold.status.mental.decrease(20, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Ingested filth", "Health", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.lewdUnfit = new ChangeController(id, "lewdUnfit", function change() {
        global.kobold.status.mental.increase(15, "Escaped through wits");
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Out-smarted humans", "Sanity", +15),
            new Change("Out-smarted humans", "Intelligence", +1)
        ]);
    });

    this.lewdStruggle = new ChangeController(id, "lewdStruggle", function change() {
        global.kobold.status.physical.decrease(15, "Pain and unhygienic behavior");
        global.kobold.status.mental.decrease(30, "Disgust and humiliation");
        global.kobold.status.exertion.increase(10, "Disgust");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Pain", "Health", -5),
            new Change("Ingested filth", "Health", -10),
            new Change("Forced", "Sanity", -10),
            new Change("Disgusted", "Sanity", -10),
            new Change("Humiliated", "Sanity", -10),
            new Change("Disgusted", "Sexual exhaustion", +10, true)
        ]);
    });

    this.fightWin = new ChangeController(id, "fightWin", function change() {
        global.kobold.status.mental.increase(10, "Fighting humans");
        global.kobold.status.stamina.decrease(10, "Fighting humans");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Victory", "Sanity", +15),
            new Change("Hurt people", "Sanity", -10),
            new Change("They deserved it", "Sanity", +5),
            new Change("Fight", "Stamina", -10)
        ]);
    });

    this.endChange = new ChangeController(id, "endChange", function change() {
        global.kobold.status.mental.decrease(5, "Lack of sleep");
        global.kobold.status.stamina.decrease(10, "Lack of sleep");
    }, function changeNote() {
        return global.ui.change.buildNotification([
            new Change("Lack of sleep", "Sanity", -5),
            new Change("Lack of sleep", "Stamina", -10)
        ]);
    });
    
    this.bgm = new AudioController(id,"res/audio/varanar/desolated_field.mp3", 60);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

}

// END scenes\Varanar\NightTwo\NightTwoController.js

// scenes\Varanar\NightTwo\NightTwo.js

/**
 * First night in Varanar.
 */
function NightTwo() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var sceneId = "NightTwo";
    var sceneName = "Varanar, Night Two";
    var sceneSubfolder = "Varanar";
    var ctrl = new NightTwoController(sceneId);
    var arrival = new NightTwoArrival(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets all the required global variables such as scene id, scene name,
     * scene stack and theme.
     */
    function setGlobals() {
        global.currentScene = that.id;
        global.currentSceneName = that.name;
        global.currentSceneFolder = that.subfolder ? that.subfolder : "";
        global.kobold.history.addToSceneStack(that.id);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = global.ui.buildGenericHeader();
        tab += global.parser.parse(`[t]Varanar[/t]`);
        tab += arrival.build();
        tab += global.ui.buildGenericFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = sceneId;
    this.name = sceneName;
    this.subfolder = sceneSubfolder;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this scene.
     */
    this.load = function () {
        setGlobals();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        arrival.registerListeners();
        ctrl.autosave();
    };
}

global.sceneLoader.queueRegister(function () {
    var scene = new NightTwo();
    global.sceneLoader.registerScene(scene.id, scene);
});

// END scenes\Varanar\NightTwo\NightTwo.js



// END SCENE NightTwo

